@@ -25,17 +25,20 @@ import hashlib
 import os
 import re
 import shutil
+import ssl
 import sys
 import time
 import urllib
 
 from urllib.parse import urlparse, urljoin
-from http.client import HTTPConnection, _CS_IDLE, HTTPException
+from http.client import HTTPConnection, HTTPSConnection, HTTPException
 from os import listdir
 from os.path import isfile, isdir, join
+from base64 import b64encode
 
+from . import dirindex
 from .exceptions import UserError, NetworkError, RepoDataError, \
-                        InvalidDirIndexFile
+                        InvalidDirIndexFile, UnsupportedURLScheme
 from .virtual_path import VirtualPath
 
 
@@ -79,7 +82,10 @@ def removeDirectoryTree(base, whatToRemove):
             "Unexpected base path for removeDirectoryTree(): {!r}".format(base)
     absPath = os.path.abspath(whatToRemove)
 
-    if _removeDirectoryTree_dangerous_cre.match(absPath):
+    if not os.path.isfile(join(absPath, ".dirindex")):
+        raise UserError("refusing to recursively delete '{}' because "
+                        "it does not contain a .dirindex file".format(absPath))
+    elif _removeDirectoryTree_dangerous_cre.match(absPath):
         raise UserError("in order to protect your data, refusing to "
                         "recursively delete '{}'".format(absPath))
     else:
@@ -116,18 +122,30 @@ class HTTPGetCallback:
                     by HTTPGetter.get().
 
         """
-        self.callback = callback
+        if callback is not None:
+            self.callback = callback
         self.src = src
 
 class HTTPGetter:
-    def __init__(self, baseUrl, maxPending=10):
+    def __init__(self, baseUrl, maxPending=10, auth=""):
         self.baseUrl = baseUrl
         self.parsedBaseUrl = urlparse(baseUrl)
         self.maxPending = maxPending
         self.requests = []
         self.pendingRequests = []
-        self.httpConnection = HTTPConnection(self.parsedBaseUrl.netloc)
+
+        if self.parsedBaseUrl.scheme == "http":
+            self.httpConnection = HTTPConnection(self.parsedBaseUrl.netloc)
+        elif self.parsedBaseUrl.scheme == "https":
+            context = ssl.create_default_context()
+            self.httpConnection = HTTPSConnection(self.parsedBaseUrl.netloc,
+                                                  context=context)
+        else:
+            raise UnsupportedURLScheme(self.parsedBaseUrl.scheme)
+
         self.httpRequestHeaders = headers = {'Host':self.parsedBaseUrl.netloc,'Content-Length':0,'Connection':'Keep-Alive','User-Agent':'FlightGear terrasync.py'}
+        if( auth and not auth.isspace() ):
+            self.httpRequestHeaders['Authorization'] = 'Basic %s' % b64encode(auth.encode("utf-8")).decode("ascii")
 
     def assemblePath(self, httpGetCallback):
         """Return the path-on-server for the file to download.
@@ -147,7 +165,8 @@ class HTTPGetter:
         return urljoin(baseUrl + '/', httpGetCallback.src.asRelative())
 
     def doGet(self, httpGetCallback):
-        conn = self.httpConnection
+        time.sleep(1.25)    # throttle the rate
+
         pathOnServer = self.assemblePath(httpGetCallback)
         self.httpConnection.request("GET", pathOnServer, None,
                                     self.httpRequestHeaders)
@@ -178,83 +197,10 @@ class HTTPGetter:
             nbRetriesLeft -= 1
 
 
-#################################################################################################################################
-class DirIndex:
-
-    def __init__(self, dirIndexFile):
-        self.d = []
-        self.f = []
-        self.t = []
-        self.version = 0
-        self.path = None        # will be a VirtualPath instance when set
-
-        # readFrom() stores the raw contents of the .dirindex file in this
-        # attribute. This is useful for troubleshooting.
-        self._rawContents = None
-
-        with open(dirIndexFile) as f:
-            self.readFrom(f)
-
-        self._sanityCheck()
-
-    def readFrom(self, readable):
-        self._rawContents = readable.read()
-
-        for line in self._rawContents.split('\n'):
-            line = line.strip()
-            if line.startswith('#'):
-                continue
-
-            tokens = line.split(':')
-            if len(tokens) == 0:
-                continue
-
-            if tokens[0] == "version":
-                self.version = int(tokens[1])
-
-            elif tokens[0] == "path":
-                # This is relative to the repository root
-                self.path = VirtualPath(tokens[1])
-
-            elif tokens[0] == "d":
-                self.d.append({ 'name': tokens[1], 'hash': tokens[2] })
-
-            elif tokens[0] == "f":
-                self.f.append({ 'name': tokens[1], 'hash': tokens[2], 'size': tokens[3] })
-
-            elif tokens[0] == "t":
-                self.t.append({ 'name': tokens[1], 'hash': tokens[2], 'size': tokens[3] })
-
-    def _sanityCheck(self):
-        if self.path is None:
-            assert self._rawContents is not None
-
-            firstLines = self._rawContents.split('\n')[:5]
-            raise InvalidDirIndexFile(
-                "no 'path' field found; the first lines of this .dirindex file "
-                "follow:\n\n" + '\n'.join(firstLines))
-
-    def getVersion(self):
-        return self.version
-
-    def getPath(self):
-        return self.path
-
-    def getDirectories(self):
-        return self.d
-
-    def getTarballs(self):
-        return self.t
-
-    def getFiles(self):
-        return self.f
-
-#################################################################################################################################
 class HTTPDownloadRequest(HTTPGetCallback):
-    def __init__(self, terrasync, src, dst, callback = None ):
+    def __init__(self, src, dst, callback=None):
         """Initialize an HTTPDownloadRequest instance.
 
-        terrasync -- a TerraSync instance
         src       -- a VirtualPath instance (corresponding to the path
                      on the server for which a GET request is to be
                      issued)
@@ -267,8 +213,7 @@ class HTTPDownloadRequest(HTTPGetCallback):
                      instance. Its return value is ignored.
 
         """
-        super().__init__(src, self.callback)
-        self.terrasync = terrasync
+        HTTPGetCallback.__init__(self, src, None)
         self.dst = dst
         self.mycallback = callback
 
@@ -309,7 +254,7 @@ class HTTPSocketRequest(HTTPGetCallback):
                (presumably a file)
 
         """
-        HTTPGetCallback.__init__(self, src, self.callback)
+        HTTPGetCallback.__init__(self, src, None)
 
     def callback(self, url, httpResponse):
         # Same comment as for HTTPDownloadRequest.callback()
@@ -498,10 +443,11 @@ class TerraSync:
         check, sync = range(2)
 
     def __init__(self, mode, doReport, url, target, quick, removeOrphan,
-                 downloadBoundaries):
+                 downloadBoundaries, auth):
         self.mode = self.Mode[mode]
         self.doReport = doReport
         self.setUrl(url).setTarget(target)
+        self.auth = auth
         self.quick = quick
         self.removeOrphan = removeOrphan
         self.httpGetter = None
@@ -539,7 +485,7 @@ class TerraSync:
             localSubdir = ""
 
         assert not os.path.isabs(localSubdir), repr(localSubdir)
-        self.httpGetter = HTTPGetter(self.url)
+        self.httpGetter = HTTPGetter(baseUrl=self.url,auth=self.auth)
 
         # Get the hash of the .dirindex file for 'virtualSubdir'
         try:
@@ -579,7 +525,7 @@ class TerraSync:
                 removeDirectoryTree(self.target, localFullPath)
 
             print("Downloading '{}'".format(virtualPath))
-            request = HTTPDownloadRequest(self, virtualPath, localFullPath)
+            request = HTTPDownloadRequest(virtualPath, localFullPath)
             self.httpGetter.get(request)
         else:
             self.abortCheckMode(failedCheckReason, virtualPath)
@@ -611,11 +557,12 @@ class TerraSync:
             if not self.quick:
                 self.handleDirindexFile(localDirIndex)
         elif self.inSyncMode():
+            if os.path.isfile(localFullPath):
+                os.unlink(localFullPath) # file on server became a directory
             if not os.path.exists(localFullPath):
                 os.makedirs(localFullPath)
 
-            request = HTTPDownloadRequest(self,
-                                          virtualPath / ".dirindex",
+            request = HTTPDownloadRequest(virtualPath / ".dirindex",
                                           localDirIndex,
                                           self.handleDirindexRequest)
             self.httpGetter.get(request)
@@ -626,27 +573,27 @@ class TerraSync:
         self.handleDirindexFile(dirindexRequest.dst)
 
     def handleDirindexFile(self, dirindexFile):
-        dirIndex = DirIndex(dirindexFile)
-        virtualBase = dirIndex.getPath()        # VirtualPath instance
+        dirIndex = dirindex.DirIndex(dirindexFile)
+        virtualBase = dirIndex.path             # VirtualPath instance
         relativeBase = virtualBase.asRelative() # string, doesn't start with '/'
         serverFiles = []
         serverDirs = []
 
-        for file in dirIndex.getFiles():
+        for file in dirIndex.files:
             f = file['name']
             self.processFileEntry(virtualBase / f,
                                   join(relativeBase, f),
                                   file['hash'])
             serverFiles.append(f)
 
-        for subdir in dirIndex.getDirectories():
+        for subdir in dirIndex.directories:
             d = subdir['name']
             self.processDirectoryEntry(virtualBase / d,
                                        join(relativeBase, d),
                                        subdir['hash'])
             serverDirs.append(d)
 
-        for tarball in dirIndex.getTarballs():
+        for tarball in dirIndex.tarballs:
             # Tarballs are handled the same as normal files.
             f = tarball['name']
             self.processFileEntry(virtualBase / f,
@@ -709,6 +656,10 @@ def parseCommandLine():
                         default="http://flightgear.sourceforge.net/scenery",
                         help="server URL [default: %(default)s]")
 
+    parser.add_argument("-a", "--auth", dest="auth", metavar="user:password",
+                        default="", help="""\
+      authentication credentials for basic auth [default: empty, no authentication]""")
+
     parser.add_argument("-t", "--target", dest="target", metavar="DIR",
                         default=".", help="""\
       directory where to store the files [default: the current directory]""")
@@ -783,7 +734,7 @@ def main():
     terraSync = TerraSync(args.mode, args.report, args.url, args.target,
                           args.quick, args.removeOrphan,
                           DownloadBoundaries(args.top, args.left, args.bottom,
-                                             args.right))
+                                             args.right),args.auth)
     report = terraSync.start(args.virtualSubdir)
 
     if args.report:
