@@ -1,25 +1,12 @@
-// AIManager.cxx  Based on David Luff's AIMgr:
-// - a global management type for AI objects
-//
-// Written by David Culp, started October 2003.
-// - davidculp2@comcast.net
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * SPDX-FileName: AIManager.cxx
+ * SPDX-FileComment: a global management type for AI objects, based on David Luff's AIMgr
+ * SPDX-FileCopyrightText: Written by David Culp, started October 2003 - davidculp2@comcast.net
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
-#include <cstring>
 #include <algorithm>
+#include <cstring>
 
 #include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/math/sg_geodesy.hxx>
@@ -36,27 +23,27 @@
 #include <Main/sentryIntegration.hxx>
 #include <Scripting/NasalSys.hxx>
 
-#include "AIManager.hxx"
 #include "AIAircraft.hxx"
-#include "AIShip.hxx"
 #include "AIBallistic.hxx"
-#include "AIStorm.hxx"
-#include "AIThermal.hxx"
 #include "AICarrier.hxx"
-#include "AIStatic.hxx"
-#include "AIMultiplayer.hxx"
-#include "AITanker.hxx"
-#include "AIWingman.hxx"
-#include "AIGroundVehicle.hxx"
 #include "AIEscort.hxx"
+#include "AIGroundVehicle.hxx"
+#include "AIManager.hxx"
+#include "AIMultiplayer.hxx"
+#include "AIShip.hxx"
+#include "AIStatic.hxx"
+#include "AIStorm.hxx"
+#include "AITanker.hxx"
+#include "AIThermal.hxx"
+#include "AIWingman.hxx"
+
 
 static bool static_haveRegisteredScenarios = false;
 
 class FGAIManager::Scenario
 {
 public:
-    Scenario(FGAIManager* man, const std::string& nm, SGPropertyNode* scenarios) :
-        _internalName(nm)
+    Scenario(FGAIManager* man, const std::string& nm, SGPropertyNode* scenarios) : _internalName(nm)
     {
         simgear::ErrorReportContext ec("scenario-name", _internalName);
         for (auto scEntry : scenarios->getChildren("entry")) {
@@ -74,7 +61,7 @@ public:
         _unloadScript = nasalScripts->getStringValue("unload");
         std::string loadScript = nasalScripts->getStringValue("load");
         if (!loadScript.empty()) {
-            FGNasalSys* nasalSys = globals->get_subsystem<FGNasalSys>();
+            auto nasalSys = globals->get_subsystem<FGNasalSys>();
             std::string moduleName = "scenario_" + _internalName;
             bool ok = nasalSys->createModule(moduleName.c_str(), moduleName.c_str(),
                                              loadScript.c_str(), loadScript.size(),
@@ -94,7 +81,7 @@ public:
                       [](FGAIBasePtr ai) { ai->setDie(true); });
 
 
-        FGNasalSys* nasalSys = globals->get_subsystem<FGNasalSys>();
+        auto nasalSys = globals->get_subsystem<FGNasalSys>();
         if (!nasalSys) // happens during shutdown / reset
             return;
 
@@ -107,6 +94,7 @@ public:
 
         nasalSys->deleteModule(moduleName.c_str());
     }
+
 private:
     std::vector<FGAIBasePtr> _objects;
     std::string _internalName;
@@ -115,15 +103,13 @@ private:
 
 ///////////////////////////////////////////////////////////////////////////////
 
-FGAIManager::FGAIManager() :
-    cb_ai_bare(SGPropertyChangeCallback<FGAIManager>(this,&FGAIManager::updateLOD,
-               fgGetNode("/sim/rendering/static-lod/aimp-bare", true))),
-    cb_ai_detailed(SGPropertyChangeCallback<FGAIManager>(this, &FGAIManager::updateLOD,
-        fgGetNode("/sim/rendering/static-lod/aimp-detailed", true))),
-    cb_interior(SGPropertyChangeCallback<FGAIManager>(this, &FGAIManager::updateLOD,
-        fgGetNode("/sim/rendering/static-lod/aimp-interior", true)))
+FGAIManager::FGAIManager() : cb_ai_bare(SGPropertyChangeCallback<FGAIManager>(this, &FGAIManager::updateLOD,
+                                                                              fgGetNode("/sim/rendering/static-lod/aimp-bare", true))),
+                             cb_ai_detailed(SGPropertyChangeCallback<FGAIManager>(this, &FGAIManager::updateLOD,
+                                                                                  fgGetNode("/sim/rendering/static-lod/aimp-detailed", true))),
+                             cb_interior(SGPropertyChangeCallback<FGAIManager>(this, &FGAIManager::updateLOD,
+                                                                               fgGetNode("/sim/rendering/static-lod/aimp-interior", true)))
 {
-
 }
 
 FGAIManager::~FGAIManager()
@@ -131,18 +117,18 @@ FGAIManager::~FGAIManager()
     std::for_each(ai_list.begin(), ai_list.end(), std::mem_fn(&FGAIBase::unbind));
 }
 
-void
-FGAIManager::init() {
+void FGAIManager::init()
+{
     root = fgGetNode("sim/ai", true);
 
     enabled = root->getNode("enabled", true);
 
     thermal_lift_node = fgGetNode("/environment/thermal-lift-fps", true);
-    wind_from_east_node  = fgGetNode("/environment/wind-from-east-fps",true);
-    wind_from_north_node = fgGetNode("/environment/wind-from-north-fps",true);
+    wind_from_east_node = fgGetNode("/environment/wind-from-east-fps", true);
+    wind_from_north_node = fgGetNode("/environment/wind-from-north-fps", true);
 
-    user_altitude_agl_node  = fgGetNode("/position/altitude-agl-ft", true);
-    user_speed_node     = fgGetNode("/velocities/uBody-fps", true);
+    user_altitude_agl_node = fgGetNode("/position/altitude-agl-ft", true);
+    user_speed_node = fgGetNode("/velocities/uBody-fps", true);
 
     globals->get_commands()->addCommand("load-scenario", this, &FGAIManager::loadScenarioCommand);
     globals->get_commands()->addCommand("unload-scenario", this, &FGAIManager::unloadScenarioCommand);
@@ -150,17 +136,17 @@ FGAIManager::init() {
     globals->get_commands()->addCommand("remove-aiobject", this, &FGAIManager::removeObjectCommand);
     _environmentVisiblity = fgGetNode("/environment/visibility-m");
     _groundSpeedKts_node = fgGetNode("/velocities/groundspeed-kt", true);
-    
+
     // Create an (invisible) AIAircraft representation of the current
     // users's aircraft, that mimicks the user aircraft's behavior.
 
     _userAircraft = new FGAIAircraft;
-    _userAircraft->setCallSign ( fgGetString("/sim/multiplay/callsign")  );
+    _userAircraft->setCallSign(fgGetString("/sim/multiplay/callsign"));
     _userAircraft->setGeodPos(globals->get_aircraft_position());
     _userAircraft->setPerformance("", "jet_transport");
     _userAircraft->setHeading(fgGetDouble("/orientation/heading-deg"));
     _userAircraft->setSpeed(_groundSpeedKts_node->getDoubleValue());
-    
+
     // radar properties
     _simRadarControl = fgGetNode("/sim/controls/radar", true);
     if (!_simRadarControl->hasValue()) {
@@ -180,44 +166,51 @@ void FGAIManager::registerScenarios(SGPropertyNode_ptr root)
         // depending on if we're using a carrier startup, this function may get
         // called early or during normal FGAIManager init, so guard against double
         // invocation.
-        // we clear this flag on shudtdown so reset works as expected
+        // we clear this flag on shutdown so reset works as expected
         if (static_haveRegisteredScenarios)
             return;
 
         static_haveRegisteredScenarios = true;
         root = globals->get_props();
     }
-    
+
     // find all scenarios at standard locations (for driving the GUI)
     std::vector<SGPath> scenarioSearchPaths;
     scenarioSearchPaths.push_back(globals->get_fg_root() / "AI");
     scenarioSearchPaths.push_back(globals->get_fg_home() / "Scenarios");
     scenarioSearchPaths.push_back(SGPath(fgGetString("/sim/aircraft-dir")) / "Scenarios");
-    
+
     // add-on scenario directories
     const auto& addonsManager = flightgear::addons::AddonManager::instance();
     if (addonsManager) {
+        auto coll = addonsManager->registeredAddons();
+        std::transform(coll.begin(), coll.end(), std::back_inserter(scenarioSearchPaths),
+                       [](flightgear::addons::AddonRef a) {
+                           return a->getBasePath() / "Scenarios";
+                       });
+#if 0
         for (auto a : addonsManager->registeredAddons()) {
             scenarioSearchPaths.push_back(a->getBasePath() / "Scenarios");
         }
+#endif
     }
 
     SGPropertyNode_ptr scenariosNode = root->getNode("/sim/ai/scenarios", true);
     for (auto p : scenarioSearchPaths) {
         if (!p.exists())
             continue;
-        
+
         simgear::Dir dir(p);
         for (auto xmlPath : dir.children(simgear::Dir::TYPE_FILE, ".xml")) {
             registerScenarioFile(root, xmlPath);
         } // of xml files in the scenario dir iteration
-    } // of scenario dirs iteration
+    }     // of scenario dirs iteration
 }
 
 SGPropertyNode_ptr FGAIManager::registerScenarioFile(SGPropertyNode_ptr root, const SGPath& xmlPath)
 {
     if (!xmlPath.exists()) return {};
-    
+
     auto scenariosNode = root->getNode("/sim/ai/scenarios", true);
     SGPropertyNode_ptr sNode;
 
@@ -226,37 +219,37 @@ SGPropertyNode_ptr FGAIManager::registerScenarioFile(SGPropertyNode_ptr root, co
     try {
         SGPropertyNode_ptr scenarioProps(new SGPropertyNode);
         readProperties(xmlPath, scenarioProps);
-        
+
         for (auto xs : scenarioProps->getChildren("scenario")) {
             if (!xs->hasChild("name") || !xs->hasChild("description")) {
                 SG_LOG(SG_AI, SG_DEV_WARN, "Scenario is missing name/description:" << xmlPath);
             }
-            
+
             sNode = scenariosNode->addChild("scenario");
-            
+
             const auto bareName = xmlPath.file_base();
             sNode->setStringValue("id", bareName);
             sNode->setStringValue("path", xmlPath.utf8Str());
-            
+
             if (xs->hasChild("name")) {
                 sNode->setStringValue("name", xs->getStringValue("name"));
             } else {
                 auto cleanedName = bareName;
-               // replace _ and - in bareName with spaces
+                // replace _ and - in bareName with spaces
                 // auto s = simgear::strutils::srep
                 sNode->setStringValue("name", cleanedName);
             }
-            
+
             if (xs->hasChild("description")) {
                 sNode->setStringValue("description", xs->getStringValue("description"));
             }
-            
+
             FGAICarrier::extractCarriersFromScenario(xs, sNode);
         } // of scenarios in the XML file
     } catch (sg_exception& e) {
         SG_LOG(SG_AI, SG_WARN, "Skipping malformed scenario file:" << xmlPath);
         simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::ScenarioLoad,
-                               string{"The scenario couldn't be loaded:"} + e.getFormattedMessage(),
+                               std::string{"The scenario couldn't be loaded:"} + e.getFormattedMessage(),
                                e.getLocation());
         sNode.reset();
     }
@@ -264,8 +257,7 @@ SGPropertyNode_ptr FGAIManager::registerScenarioFile(SGPropertyNode_ptr root, co
     return sNode;
 }
 
-void
-FGAIManager::postinit()
+void FGAIManager::postinit()
 {
     // postinit, so that it can access the Nasal subsystem
 
@@ -275,7 +267,7 @@ FGAIManager::postinit()
 
     // process all scenarios
     for (auto n : root->getChildren("scenario")) {
-        const string& name = n->getStringValue();
+        const std::string& name = n->getStringValue();
         if (name.empty())
             continue;
 
@@ -289,8 +281,7 @@ FGAIManager::postinit()
     }
 }
 
-void
-FGAIManager::reinit()
+void FGAIManager::reinit()
 {
     // shutdown scenarios
     unloadAllScenarios();
@@ -302,8 +293,7 @@ FGAIManager::reinit()
     postinit();
 }
 
-void
-FGAIManager::shutdown()
+void FGAIManager::shutdown()
 {
     unloadAllScenarios();
 
@@ -316,30 +306,37 @@ FGAIManager::shutdown()
 
     ai_list.clear();
     _environmentVisiblity.clear();
-    _userAircraft.clear();
+
+    if (_userAircraft) {
+        _userAircraft->setDie(true);
+        // we can't unbind() but we do need to clear these
+        _userAircraft->clearATCController();
+        _userAircraft.clear();
+    }
+
     static_haveRegisteredScenarios = false;
-    
+
     globals->get_commands()->removeCommand("load-scenario");
     globals->get_commands()->removeCommand("unload-scenario");
     globals->get_commands()->removeCommand("add-aiobject");
     globals->get_commands()->removeCommand("remove-aiobject");
 }
 
-void
-FGAIManager::bind() {
+void FGAIManager::bind()
+{
     root = globals->get_props()->getNode("ai/models", true);
     root->tie("count", SGRawValueMethods<FGAIManager, int>(*this,
-        &FGAIManager::getNumAiObjects));
+                                                           &FGAIManager::getNumAiObjects));
 }
 
-void
-FGAIManager::unbind() {
+void FGAIManager::unbind()
+{
     root->untie("count");
 }
 
 void FGAIManager::removeDeadItem(FGAIBase* base)
 {
-    SGPropertyNode *props = base->_getProps();
+    SGPropertyNode* props = base->_getProps();
 
     props->setBoolValue("valid", false);
     base->unbind();
@@ -352,8 +349,7 @@ void FGAIManager::removeDeadItem(FGAIBase* base)
     props->setIntValue("refuel/tanker", false);
 }
 
-void
-FGAIManager::update(double dt)
+void FGAIManager::update(double dt)
 {
     // initialize these for finding nearest thermals
     range_nearest = 10000.0;
@@ -363,18 +359,18 @@ FGAIManager::update(double dt)
         return;
 
     fetchUserState(dt);
-    
+
     // fetch radar state. Ensure we only do this once per frame.
     _radarEnabled = _simRadarControl->getBoolValue();
     _radarDebugMode = _radarDebugNode->getBoolValue();
     _radarRangeM = _radarRangeNode->getDoubleValue() * SG_NM_TO_METER;
-    
+
     // partition the list into dead followed by alive
     auto firstAlive =
-      std::stable_partition(ai_list.begin(), ai_list.end(), std::mem_fn(&FGAIBase::getDie));
+        std::stable_partition(ai_list.begin(), ai_list.end(), std::mem_fn(&FGAIBase::getDie));
 
     // clean up each item and finally remove from the container
-    for (auto it=ai_list.begin(); it != firstAlive; ++it) {
+    for (auto it = ai_list.begin(); it != firstAlive; ++it) {
         removeDeadItem(*it);
     }
 
@@ -385,51 +381,50 @@ FGAIManager::update(double dt)
     // entire subsystem.
     for (FGAIBase* base : ai_list) {
         try {
-            if (base->isa(FGAIBase::otThermal)) {
+            if (base->isa(FGAIBase::object_type::otThermal)) {
                 processThermal(dt, static_cast<FGAIThermal*>(base));
             } else {
                 base->update(dt);
             }
         } catch (sg_exception& e) {
-            SG_LOG(SG_AI, SG_WARN, "caught exception updating AI model:" << base->_getName()<< ", which will be killed."
-                   "\n\tError:" << e.getFormattedMessage());
+            SG_LOG(SG_AI, SG_WARN, "caught exception updating AI model:" << base->_getName() << ", which will be killed."
+                                                                                                "\n\tError:"
+                                                                         << e.getFormattedMessage());
             base->setDie(true);
         }
-    } // of live AI objects iteration
+    }                                            // of live AI objects iteration
 
-    thermal_lift_node->setDoubleValue( strength );  // for thermals
+    thermal_lift_node->setDoubleValue(strength); // for thermals
 }
 
 /** update LOD settings of all AI/MP models */
-void
-FGAIManager::updateLOD(SGPropertyNode* node)
+void FGAIManager::updateLOD(SGPropertyNode* node)
 {
     SG_UNUSED(node);
     std::for_each(ai_list.begin(), ai_list.end(), std::mem_fn(&FGAIBase::updateLOD));
 }
 
-void
-FGAIManager::attach(const SGSharedPtr<FGAIBase> &model)
+void FGAIManager::attach(const SGSharedPtr<FGAIBase>& model)
 {
-    const char* typeString = model->getTypeString();
-    SGPropertyNode* root = globals->get_props()->getNode("ai/models", true);
+    std::string_view typeString = model->getTypeString();
+    SGPropertyNode* l_root = globals->get_props()->getNode("ai/models", true);
     SGPropertyNode* p;
     int i;
 
     // find free index in the property tree, if we have
-    // more than 10000 mp-aircrafts in the property tree we should optimize the mp-server
+    // more than 10000 mp-aircraft in the property tree we should optimize the mp-server
     for (i = 0; i < 10000; i++) {
-        p = root->getNode(typeString, i, false);
+        p = l_root->getNode(static_cast<std::string>(typeString), i, false);
 
         if (!p || !p->getBoolValue("valid", false))
             break;
 
-        if (p->getIntValue("id",-1)==model->getID()) {
-            p->setStringValue("callsign","***invalid node***"); //debug only, should never set!
+        if (p->getIntValue("id", -1) == model->getID()) {
+            p->setStringValue("callsign", "***invalid node***"); //debug only, should never set!
         }
     }
 
-    p = root->getNode(typeString, i, true);
+    p = l_root->getNode(static_cast<std::string>(typeString), i, true);
     model->setManager(this, p);
     ai_list.push_back(model);
 
@@ -440,24 +435,21 @@ FGAIManager::attach(const SGSharedPtr<FGAIBase> &model)
 
 bool FGAIManager::isVisible(const SGGeod& pos) const
 {
-  double visibility_meters = _environmentVisiblity->getDoubleValue();
-  return ( dist(globals->get_view_position_cart(), SGVec3d::fromGeod(pos)) ) <= visibility_meters;
+    double visibility_meters = _environmentVisiblity->getDoubleValue();
+    return (dist(globals->get_view_position_cart(), SGVec3d::fromGeod(pos))) <= visibility_meters;
 }
 
-int
-FGAIManager::getNumAiObjects() const
+int FGAIManager::getNumAiObjects() const
 {
     return static_cast<int>(ai_list.size());
 }
 
-void
-FGAIManager::fetchUserState( double dt )
+void FGAIManager::fetchUserState(double dt)
 {
-
     globals->get_aircraft_orientation(user_heading, user_pitch, user_roll);
-    user_speed     = user_speed_node->getDoubleValue() * 0.592484;
+    user_speed = user_speed_node->getDoubleValue() * 0.592484;
     wind_from_east = wind_from_east_node->getDoubleValue();
-    wind_from_north   = wind_from_north_node->getDoubleValue();
+    wind_from_north = wind_from_north_node->getDoubleValue();
     user_altitude_agl = user_altitude_agl_node->getDoubleValue();
 
     _userAircraft->setGeodPos(globals->get_aircraft_position());
@@ -467,18 +459,17 @@ FGAIManager::fetchUserState( double dt )
 }
 
 // only keep the results from the nearest thermal
-void
-FGAIManager::processThermal( double dt, FGAIThermal* thermal ) {
+void FGAIManager::processThermal(double dt, FGAIThermal* thermal)
+{
     thermal->update(dt);
 
-    if ( thermal->_getRange() < range_nearest ) {
+    if (thermal->_getRange() < range_nearest) {
         range_nearest = thermal->_getRange();
         strength = thermal->getStrength();
     }
-
 }
 
-bool FGAIManager::loadScenarioCommand(const SGPropertyNode* args, SGPropertyNode *)
+bool FGAIManager::loadScenarioCommand(const SGPropertyNode* args, SGPropertyNode*)
 {
     std::string name = args->getStringValue("name");
     if (args->hasChild("load-property")) {
@@ -506,7 +497,7 @@ bool FGAIManager::loadScenarioCommand(const SGPropertyNode* args, SGPropertyNode
     return ok;
 }
 
-bool FGAIManager::unloadScenarioCommand(const SGPropertyNode * arg, SGPropertyNode * root)
+bool FGAIManager::unloadScenarioCommand(const SGPropertyNode* arg, SGPropertyNode* root)
 {
     SG_UNUSED(root);
     std::string name = arg->getStringValue("name");
@@ -516,7 +507,7 @@ bool FGAIManager::unloadScenarioCommand(const SGPropertyNode * arg, SGPropertyNo
 bool FGAIManager::addObjectCommand(const SGPropertyNode* arg, const SGPropertyNode* root)
 {
     SG_UNUSED(root);
-    if (!arg){
+    if (!arg) {
         return false;
     }
     addObject(arg);
@@ -526,7 +517,7 @@ bool FGAIManager::addObjectCommand(const SGPropertyNode* arg, const SGPropertyNo
 FGAIBasePtr FGAIManager::addObject(const SGPropertyNode* definition)
 {
     const std::string& type = definition->getStringValue("type", "aircraft");
-    
+
     FGAIBase* ai = nullptr;
     if (type == "tanker") { // refueling scenarios
         ai = new FGAITanker;
@@ -553,14 +544,13 @@ FGAIBasePtr FGAIManager::addObject(const SGPropertyNode* definition)
     }
 
     ai->readFromScenario(const_cast<SGPropertyNode*>(definition));
-	if((ai->isValid())){
-		attach(ai);
+    if ((ai->isValid())) {
+        attach(ai);
         SG_LOG(SG_AI, SG_DEBUG, "attached scenario " << ai->_getName());
-	}
-	else{
-		ai->setDie(true);
-		SG_LOG(SG_AI, SG_ALERT, "killed invalid scenario "  << ai->_getName());
-	}
+    } else {
+        ai->setDie(true);
+        SG_LOG(SG_AI, SG_ALERT, "killed invalid scenario " << ai->_getName());
+    }
     return ai;
 }
 
@@ -576,12 +566,21 @@ bool FGAIManager::removeObjectCommand(const SGPropertyNode* arg, const SGPropert
 bool FGAIManager::removeObject(const SGPropertyNode* args)
 {
     int id = args->getIntValue("id");
+    auto coll = get_ai_list();
+    auto it_ai = std::find_if(coll.begin(), coll.end(), [id](FGAIBasePtr ai) {
+        return ai->getID() == id;
+    });
+    if (it_ai != coll.end())
+        (*it_ai)->setDie(true);
+
+#if 0
     for (FGAIBase* ai : get_ai_list()) {
         if (ai->getID() == id) {
             ai->setDie(true);
             break;
         }
     }
+#endif
 
     return false;
 }
@@ -596,8 +595,7 @@ FGAIBasePtr FGAIManager::getObjectFromProperty(const SGPropertyNode* aProp) cons
     return *it;
 }
 
-bool
-FGAIManager::loadScenario( const string &id )
+bool FGAIManager::loadScenario(const std::string& id)
 {
     SGPath path;
     SGPropertyNode_ptr file = loadScenarioFile(id, path);
@@ -620,8 +618,7 @@ FGAIManager::loadScenario( const string &id )
 }
 
 
-bool
-FGAIManager::unloadScenario( const string &filename)
+bool FGAIManager::unloadScenario(const std::string& filename)
 {
     auto it = _scenarios.find(filename);
     if (it == _scenarios.end()) {
@@ -629,7 +626,7 @@ FGAIManager::unloadScenario( const string &filename)
         return false;
     }
 
-// remove /sim/ai node
+    // remove /sim/ai node
     for (auto n : root->getChildren("scenario")) {
         if (n->getStringValue() == filename) {
             root->removeChild(n);
@@ -642,13 +639,12 @@ FGAIManager::unloadScenario( const string &filename)
     return true;
 }
 
-void
-FGAIManager::unloadAllScenarios()
+void FGAIManager::unloadAllScenarios()
 {
     std::for_each(_scenarios.begin(), _scenarios.end(),
                   [](const ScenarioDict::value_type& v) { delete v.second; });
     // remove /sim/ai node
-    if(root) {
+    if (root) {
         root->removeChildren("scenario");
     }
     _scenarios.clear();
@@ -670,25 +666,21 @@ FGAIManager::loadScenarioFile(const std::string& scenarioName, SGPath& outPath)
                 SGPropertyNode_ptr root = new SGPropertyNode;
                 readProperties(path, root);
                 return root;
-            } catch (const sg_exception &t) {
-                SG_LOG(SG_AI, SG_ALERT, "Failed to load scenario '"
-                       << path << "': " << t.getFormattedMessage());
+            } catch (const sg_exception& t) {
+                SG_LOG(SG_AI, SG_ALERT, "Failed to load scenario '" << path << "': " << t.getFormattedMessage());
                 simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::ScenarioLoad,
                                        "Failed to laod scenario XML:" + t.getFormattedMessage(),
                                        t.getLocation());
             }
         }
     }
-    
+
     return {};
 }
 
-const FGAIBase *
+const FGAIBase*
 FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range)
 {
-    // we specify tgt extent (ft) according to the AIObject type
-    double tgt_ht[]     = {0,  50, 100, 250, 0, 100, 0, 0,  50,  50, 20, 100,  50};
-    double tgt_length[] = {0, 100, 200, 750, 0,  50, 0, 0, 200, 100, 40, 200, 100};
     ai_list_iterator ai_list_itr = ai_list.begin();
     ai_list_iterator end = ai_list.end();
 
@@ -696,22 +688,22 @@ FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range
     SGVec3d cartPos(SGVec3d::fromGeod(pos));
 
     while (ai_list_itr != end) {
-        double tgt_alt = (*ai_list_itr)->_getAltitude();
-        int type       = (*ai_list_itr)->getType();
-        tgt_ht[type] += fuse_range;
+        FGAIBasePtr aiModel = *ai_list_itr;
+        FGAIBase::object_type type = aiModel->getType();
+        double tgt_alt = aiModel->_getAltitude();
+        int l_tgt_ht = aiModel->getCollisionHeight() + fuse_range;
 
-        if (fabs(tgt_alt - alt) > tgt_ht[type] || type == FGAIBase::otBallistic
-            || type == FGAIBase::otStorm || type == FGAIBase::otThermal ) {
-                //SG_LOG(SG_AI, SG_DEBUG, "AIManager: skipping "
-                //    << fabs(tgt_alt - alt)
-                //    << " "
-                //    << type
-                //    );
-                ++ai_list_itr;
-                continue;
+        if (fabs(tgt_alt - alt) > l_tgt_ht || type == FGAIBase::object_type::otBallistic || type == FGAIBase::object_type::otStorm || type == FGAIBase::object_type::otThermal) {
+            //SG_LOG(SG_AI, SG_DEBUG, "AIManager: skipping "
+            //    << fabs(tgt_alt - alt)
+            //    << " "
+            //    << type
+            //    );
+            ++ai_list_itr;
+            continue;
         }
 
-        int id         = (*ai_list_itr)->getID();
+        int id = (*ai_list_itr)->getID();
 
         double range = calcRangeFt(cartPos, (*ai_list_itr));
 
@@ -724,16 +716,13 @@ FGAIManager::calcCollision(double alt, double lat, double lon, double fuse_range
         //    << " alt " << tgt_alt
         //    );
 
-        tgt_length[type] += fuse_range;
+        int l_tgt_length = aiModel->getCollisionLength() + fuse_range;
 
-        if (range < tgt_length[type]){
+        if (range < l_tgt_length) {
             SG_LOG(SG_AI, SG_DEBUG, "AIManager: HIT! "
-                << " type " << type
-                << " ID " << id
-                << " range " << range
-                << " alt " << tgt_alt
-                );
-            return (*ai_list_itr).get();
+                                        << " (h:" << l_tgt_ht << ", w:" << l_tgt_length << ")"
+                                        << " type " << static_cast<int>(type) << " ID " << id << " range " << range << " alt " << tgt_alt);
+            return aiModel.get();
         }
         ++ai_list_itr;
     }
