@@ -42,13 +42,11 @@
 using std::string;
 
 /**
-Constructer, initializes values to private boolean and FGATCController instances
+Constructor, initializes values to private boolean and FGATCController instances
 */
 FGATCManager::FGATCManager() :
-    controller(NULL),
-    prevController(NULL),
-    networkVisible(false),
-    initSucceeded(false)
+    controller(nullptr),
+    prevController(nullptr)
 {
 }
 
@@ -59,15 +57,20 @@ FGATCManager::~FGATCManager() {
 }
 
 /**
-Sets up ATC subsystem parts depending on other subsystems
-Override of SGSubsystem::postinit()
-Will set private boolean flag "initSucceeded" to true upon conclusion
+* Sets up ATC subsystem parts depending on other subsystems
+* Override of SGSubsystem::postinit()
+* Will set private boolean flag "initSucceeded" to true upon conclusion
 */
 void FGATCManager::postinit()
 {
-    int leg = 0;
+    transNum = fgGetNode("/sim/atc/transmission-num", true);
+    splashAlpha = fgGetNode("/sim/startup/splash-alpha", true);
+    initControllers();
+    initSucceeded = true;
+}
 
-    trans_num = globals->get_props()->getNode("/sim/atc/transmission-num", true);
+void FGATCManager::initControllers() {
+    int leg = 0;
 
     // Assign a controller to the user's aircraft.
     // Three scenarios are considered:
@@ -78,21 +81,23 @@ void FGATCManager::postinit()
     string runway  = fgGetString("/sim/atc/runway");
     string curAirport = fgGetString("/sim/presets/airport-id");
     string parking = fgGetString("/sim/presets/parkpos");
-    destination = fgGetString("/autopilot/route-manager/destination/airport");
-    
-    FGAIManager* aiManager = globals->get_subsystem<FGAIManager>();
+
+    _routeManagerDestinationAirportNode = globals->get_props()->getNode("/autopilot/route-manager/destination/airport", true);
+    destination = _routeManagerDestinationAirportNode->getStringValue();
+
+    auto aiManager = globals->get_subsystem<FGAIManager>();
     auto userAircraft = aiManager->getUserAircraft();
     string callsign = userAircraft->getCallSign();
-    
+
     double aircraftRadius = 40; // note that this is currently hardcoded to a one-size-fits all JumboJet value. Should change later.
-    
+
     // In case a destination is not set yet, make it equal to the current airport
-    if (destination == "") {
+    if (destination.empty()) {
         destination = curAirport;
     }
 
     // NEXT UP: Create a traffic schedule and fill that with appropriate information. This we can use for flight planning.
-    // Note that these are currently only defaults. 
+    // Note that these are currently only defaults.
 	userAircraftTrafficRef.reset(new FGAISchedule);
     userAircraftTrafficRef->setFlightType("gate");
 
@@ -102,16 +107,15 @@ void FGATCManager::postinit()
 	userAircraftScheduledFlight->initializeAirports();
 	userAircraftScheduledFlight->setFlightRules("IFR");
 	userAircraftScheduledFlight->setCallSign(callsign);
-    
+
     userAircraftTrafficRef->assign(userAircraftScheduledFlight.get());
     std::unique_ptr<FGAIFlightPlan> fp ;
     userAircraft->setTrafficRef(userAircraftTrafficRef.get());
-    
-    string flightPlanName = curAirport + "-" + destination + ".xml";
-    //double cruiseAlt = 100; // Doesn't really matter right now.
-    //double courseToDest = 180; // Just use something neutral; this value might affect the runway that is used though...
-    //time_t deptime = 0;        // just make sure how flightplan processing is affected by this...
 
+    // string flightPlanName = curAirport + "-" + _routeManagerDestinationAirportNode->getStringValue() + ".xml";
+    // double cruiseAlt = 100; // Doesn't really matter right now.
+    // double courseToDest = 180; // Just use something neutral; this value might affect the runway that is used though...
+    // time_t deptime = 0;        // just make sure how flightplan processing is affected by this...
 
     FGAirportDynamicsRef dcs(flightgear::AirportDynamicsManager::find(curAirport));
     if (dcs && onGround) {// && !runway.empty()) {
@@ -127,7 +131,7 @@ void FGATCManager::postinit()
                     fgSetString("/sim/presets/parkpos", pk.parking()->getName());
                 }
             }
-            
+
             if (!pk.isValid()) {
                 FGParkingList pkl(dcs->getParkings(true, "gate"));
                 if (!pkl.empty()) {
@@ -141,7 +145,7 @@ void FGATCManager::postinit()
         } else if (!parking.empty()) {
             pk = dcs->getAvailableParkingByName(parking);
         }
-      
+
         if (pk.isValid()) {
             dcs->setParkingAvailable(pk.parking(), false);
             fp.reset(new FGAIFlightPlan);
@@ -152,7 +156,7 @@ void FGATCManager::postinit()
                 SG_LOG(SG_ATC, SG_DEBUG, "Setting radio frequency to : " << stationFreq);
                 fgSetDouble("/instrumentation/comm[0]/frequencies/selected-mhz", ((double) stationFreq / 100.0));
             }
-            leg = 1;
+            leg = AILeg::STARTUP_PUSHBACK;
             //double, lat, lon, head; // Unused variables;
             //int getId = apt->getDynamics()->getParking(gateId, &lat, &lon, &head);
             aircraftRadius = pk.parking()->getRadius();
@@ -171,8 +175,8 @@ void FGATCManager::postinit()
                 return;
             }
 
-            
-            
+
+
         } else if (!runway.empty()) {
             // on a runway
 
@@ -184,11 +188,10 @@ void FGATCManager::postinit()
                 fgSetDouble("/instrumentation/comm[0]/frequencies/selected-mhz", ((double) stationFreq / 100.0));
             }
             fp.reset(new FGAIFlightPlan);
-            leg = 3;
+            leg = AILeg::TAKEOFF;
             string fltType = "ga";
             fp->setRunway(runway);
-            fp->createTakeOff(userAircraft, false, dcs->parent(), 0, fltType);
-            userAircraft->setTakeOffStatus(2);
+            fp->createTakeOff(userAircraft, false, dcs->parent(), userAircraft->getGeodPos(), 0, fltType);
         } else {
             // We're on the ground somewhere. Handle this case later.
 
@@ -199,7 +202,7 @@ void FGATCManager::postinit()
             fp.reset(FGAIFlightPlan::createDummyUserPlan());
             userAircraft->FGAIBase::setFlightPlan(std::move(fp));
             controller = nullptr;
-            
+
             initSucceeded = true; // should be false?
             return;
         }
@@ -208,7 +211,7 @@ void FGATCManager::postinit()
             fp->getLastWaypoint()->setName( fp->getLastWaypoint()->getName() + string("legend"));
         }
      } else {
-        controller = 0;
+        controller = nullptr;
      }
 
     // Create an initial flightplan and assign it to the ai_ac. We won't use this flightplan, but it is necessary to
@@ -220,7 +223,7 @@ void FGATCManager::postinit()
         fp->setLeg(leg);
         userAircraft->FGAIBase::setFlightPlan(std::move(fp));
     }
-    
+
     if (controller) {
         FGAIFlightPlan* plan = userAircraft->GetFlightPlan();
         const int routeIndex = (plan && plan->getCurrentWaypoint()) ? plan->getCurrentWaypoint()->getRouteIndex() : 0;
@@ -233,7 +236,6 @@ void FGATCManager::postinit()
                                      userAircraft->getAltitude(),
                                      aircraftRadius, leg, userAircraft);
     }
-    initSucceeded = true;
 }
 
 /**
@@ -246,15 +248,16 @@ void FGATCManager::shutdown()
     activeStations.clear();
     userAircraftTrafficRef.reset();
     userAircraftScheduledFlight.reset();
+    _routeManagerDestinationAirportNode.clear();
 }
 
 void FGATCManager::reposition()
 {
     prevController = controller = nullptr;
-    
+
 // remove any parking assignment form the user flight-plan, so it's
 // available again. postinit() will recompute a new value if required
-    FGAIManager* aiManager = globals->get_subsystem<FGAIManager>();
+    auto aiManager = globals->get_subsystem<FGAIManager>();
     auto userAircraft = aiManager->getUserAircraft();
     if (userAircraft) {
         if (userAircraft->GetFlightPlan()) {
@@ -265,7 +268,7 @@ void FGATCManager::reposition()
         userAircraft->clearATCController();
     }
 
-    postinit(); // critical for position-init logic
+    initControllers(); // critical for position-init logic
 }
 
 /**
@@ -288,6 +291,21 @@ void FGATCManager::removeController(FGATCController *controller)
     }
 }
 
+void FGATCManager::signalReady()
+{
+    auto aiManager = globals->get_subsystem<FGAIManager>();
+    auto userAircraft = aiManager->getUserAircraft();
+    if (userAircraft) {
+        auto userAIFP = userAircraft->GetFlightPlan();
+        if (userAIFP && 
+            fgGetBool("/sim/ai/enabled") &&
+            userAIFP->getLeg() == AILeg::TAKEOFF && 
+            userAircraft->getTakeOffStatus() == AITakeOffStatus::NONE) {
+            userAircraft->setTakeOffStatus(AITakeOffStatus::QUEUED);
+        }
+    }
+}
+
 /**
 Update the subsystem.
 FlightGear invokes this method every time the subsystem should
@@ -297,22 +315,30 @@ update its state.
 update.  On first update, delta time will be 0.
 */
 void FGATCManager::update ( double time ) {
-    SG_LOG(SG_ATC, SG_BULK, "ATC update code is running at time: " << time);
-    
+    // SG_LOG(SG_ATC, SG_BULK, "ATC update code is running at time: " << time);
+
+    if (splashAlpha->getDoubleValue() == 0.0 && startTime == 0) {
+        // start after the splash has disapeared and a few seconds
+        startTime = globals->get_sim_time_sec() + 1.0 + rand()%10;
+    }
+    if (startTime > 0 && startTime < globals->get_sim_time_sec()) {
+        signalReady();
+    }
     // Test code: let my virtual co-pilot handle ATC
-    FGAIManager* aiManager = globals->get_subsystem<FGAIManager>();
+    auto aiManager = globals->get_subsystem<FGAIManager>();
     FGAIAircraft* user_ai_ac = aiManager->getUserAircraft();
     FGAIFlightPlan *fp = user_ai_ac->GetFlightPlan();
-    
+
     // Update destination
-    string result = fgGetString("/autopilot/route-manager/destination/airport");
+    string result = _routeManagerDestinationAirportNode->getStringValue();
+
     if (destination != result && result != "") {
         destination = result;
         userAircraftScheduledFlight->setArrivalAirport(destination);
-		userAircraftScheduledFlight->initializeAirports();
-		userAircraftTrafficRef->clearAllFlights();
-		userAircraftTrafficRef->assign(userAircraftScheduledFlight.get());
-        
+        userAircraftScheduledFlight->initializeAirports();
+        userAircraftTrafficRef->clearAllFlights();
+        userAircraftTrafficRef->assign(userAircraftScheduledFlight.get());
+
         auto userAircraft = aiManager->getUserAircraft();
         userAircraft->setTrafficRef(userAircraftTrafficRef.get());
     }
@@ -324,13 +350,13 @@ void FGATCManager::update ( double time ) {
         //SG_LOG(SG_ATC, SG_DEBUG, "Setting intentions");
         // This indicates that we have run out of waypoints: Im future versions, the
         // user should be able to select a new route, but for now just shut down the
-        // system. 
+        // system.
         if (size < 3) {
             return;
         }
 #if 0
-        // Test code: Print how far we're progressing along the taxi route. 
-        SG_LOG(SG_ATC, SG_DEBUG, "Size of waypoint cue " << size);
+        // Test code: Print how far we're progressing along the taxi route.
+        SG_LOG(SG_ATC, SG_DEBUG, "Size of waypoint queue " << size);
         for (int i = 0; i < size; i++) {
             int val = fp->getRouteIndex(i);
             SG_LOG(SG_ATC, SG_BULK, fp->getWayPoint(i)->getName() << " ");
@@ -343,63 +369,107 @@ void FGATCManager::update ( double time ) {
 #endif
     }
     if (fp) {
-        SG_LOG(SG_ATC, SG_DEBUG, "Currently at leg : " << fp->getLeg());
+        SG_LOG(SG_ATC, SG_BULK, "User aircraft currently at leg : " << fp->getLeg());
     }
-    
+
     // Call getATCController method; returns what FGATCController presently controls the user aircraft
     // - e.g. FGStartupController
-    controller = user_ai_ac->getATCController();
+    // controller = user_ai_ac->getATCController();
+    // FIXME the AIAircraft currently doesn't set this for the user aircraft
 
     // Update the ATC dialog
-    FGATCDialogNew::instance()->update(time);
+    //FGATCDialogNew::instance()->update(time);
 
     // Controller manager - if controller is set, then will update controller
     if (controller) {
-        SG_LOG(SG_ATC, SG_DEBUG, "name of previous waypoint : " << fp->getPreviousWaypoint()->getName());
-        SG_LOG(SG_ATC, SG_DEBUG, "Currently under control of " << controller->getName());
+//        SG_LOG(SG_ATC, SG_DEBUG, "name of previous waypoint : " << fp->getPreviousWaypoint()->getName());
+        SG_LOG(SG_ATC, SG_BULK, "Currently under control of " << controller->getName());
 
         // update aircraft information (simulates transponder)
 
         controller->updateAircraftInformation(user_ai_ac->getID(),
-                                              user_ai_ac->_getLatitude(),
-                                              user_ai_ac->_getLongitude(),
-                                              user_ai_ac->_getHeading(),
-                                              user_ai_ac->getSpeed(),
-                                              user_ai_ac->getAltitude(), time);
+                                            user_ai_ac->getGeodPos(),
+                                            user_ai_ac->_getHeading(),
+                                            user_ai_ac->getSpeed(),
+                                            user_ai_ac->getAltitude(), time);
+
+        if (fp) {
+            switch (fp->getLeg()) {
+            case AILeg::STARTUP_PUSHBACK:              // Startup and Push back
+                if (userAircraftTrafficRef->getDepartureAirport()->getDynamics())
+                    controller = userAircraftTrafficRef->getDepartureAirport()->getDynamics()->getStartupController();
+                break;
+            case AILeg::TAXI:              // Taxiing to runway
+                if (userAircraftTrafficRef->getDepartureAirport()->getDynamics()->getGroundController()->exists())
+                    controller = userAircraftTrafficRef->getDepartureAirport()->getDynamics()->getGroundController();
+                break;
+            case AILeg::TAKEOFF:              //Take off tower controller
+                if (userAircraftTrafficRef->getDepartureAirport()->getDynamics()) {
+                    controller = userAircraftTrafficRef->getDepartureAirport()->getDynamics()->getTowerController();
+                } else {
+                    SG_LOG(SG_AI, SG_BULK, "Error: Could not find Dynamics at airport : " << userAircraftTrafficRef->getDepartureAirport()->getId());
+                }
+                break;
+                /* TODO: link up with state system?
+            case AILeg::APPROACH:
+                if (userAircraftTrafficRef->getArrivalAirport()->getDynamics()) {
+                    controller = trafficRef->getArrivalAirport()->getDynamics()->getApproachController();
+                }
+                break;
+            case AILeg::PARKING_TAXI:              // Taxiing for parking
+                if (trafficRef->getArrivalAirport()->getDynamics()->getGroundController()->exists())
+                    controller = trafficRef->getArrivalAirport()->getDynamics()->getGroundController();
+                break;
+                */
+            default:
+                if(prevController) {
+                    SG_LOG(SG_AI, SG_BULK, "Will be signing off user ai " << user_ai_ac->getID() << " from " << prevController->getName());
+                }
+                controller = nullptr;
+                break;
+            }
+
+            if ((controller != prevController) && prevController && !user_ai_ac->getDie()) {
+                //If we are dead we are automatically erased
+                prevController->signOff(user_ai_ac->getID());
+            }
+            prevController = controller;
+        }
 
         //string airport = fgGetString("/sim/presets/airport-id");
-        //FGAirport *apt = FGAirport::findByIdent(airport); 
-        // AT this stage we should update the flightplan, so that waypoint incrementing is conducted as well as leg loading. 
+        //FGAirport *apt = FGAirport::findByIdent(airport);
+        // AT this stage we should update the flightplan, so that waypoint incrementing is conducted as well as leg loading.
 
         // Ground network visibility:
         // a) check to see if the message to toggle visibility was called
         // b) if so, toggle network visibility and reset the transmission
-        // c) therafter disable rendering for the old controller (TODO: should this be earlier?)
+        // c) thereafter disable rendering for the old controller (TODO: should this be earlier?)
         // d) and render if enabled for the new controller
-        int n = trans_num->getIntValue();
+        int n = transNum->getIntValue();
 
         if (n == 1) {
             SG_LOG(SG_ATC, SG_DEBUG, "Toggling ground network visibility " << networkVisible);
             networkVisible = !networkVisible;
-            trans_num->setIntValue(-1);
+            transNum->setIntValue(-1);
         }
-        
+
         // stop rendering the old controller's groundnetwork
         if ((controller != prevController) && (prevController)) {
             prevController->render(false);
         }
 
-        // render the path for the present controller if the ground network is set to visible
-        controller->render(networkVisible);
-
-        SG_LOG(SG_ATC, SG_BULK, "Adding ground network to the scenegraph::update");
+        if (controller) {
+            // render the path for the present controller if the ground network is set to visible
+            controller->render(networkVisible);
+            SG_LOG(SG_ATC, SG_BULK, "Adding ground network to the scenegraph::update");
+        }
 
         // reset previous controller for next update() iteration
         prevController = controller;
-   }
+    }
 
    // update the active ATC stations
-   for (AtcVecIterator atc = activeStations.begin(); atc != activeStations.end(); atc++) {
+   for (AtcVecIterator atc = activeStations.begin(); atc != activeStations.end(); ++atc) {
        (*atc)->update(time);
    }
 }
