@@ -1,39 +1,25 @@
-// airport.cxx -- Classes representing airports, seaports and helipads
-//
-// Written by Curtis Olson, started April 1998.
-// Updated by Durk Talsma, started December, 2004.
-//
-// Copyright (C) 1998  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileName: airport.cxx
+ * SPDX-FileComment: Classes representing airports, seaports and helipads
+ * SPDX-FileCopyrightText: Copyright (C) 1998  Curtis L. Olson  - http://www.flightgear.org/~curt
+ * SPDX-FileContributor: Updated by Durk Talsma, started December, 2004
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include "airport.hxx"
+#include "simgear/debug/debug_types.h"
 
 #include <algorithm>
 #include <cassert>
+#include <utility>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/debug/logstream.hxx>
 #include <simgear/sg_inlines.h>
 #include <simgear/structure/exception.hxx>
 
@@ -70,10 +56,12 @@ FGAirport::FGAirport( PositionedID aGuid,
                       const SGGeod& location,
                       const std::string &name,
                       bool has_metar,
-                      Type aType ):
+                      Type aType,
+                      SGPath sceneryPath):
     FGPositioned(aGuid, aType, id, location),
     _name(name),
     _has_metar(has_metar),
+    _sceneryPath(std::move(sceneryPath)),
     mTowerDataLoaded(false),
     mHasTower(false),
     mRunwaysLoaded(false),
@@ -106,6 +94,11 @@ bool FGAirport::isHeliport() const
   return type() == HELIPORT;
 }
 
+SGPath FGAirport::sceneryPath() const
+{
+  return _sceneryPath;
+}
+
 //------------------------------------------------------------------------------
 unsigned int FGAirport::numRunways() const
 {
@@ -144,7 +137,7 @@ FGRunwayMap FGAirport::getRunwayMap() const
 
   for (auto rwy : mRunways) {
     // ignore unusably short runways
-    // TODO other methods don't check this...
+    // TODO: other methods don't check this...
     if( rwy->lengthFt() >= minLengthFt )
       map[ rwy->ident() ] = rwy;
   }
@@ -189,11 +182,14 @@ bool FGAirport::hasHelipadWithIdent(const std::string& aIdent) const
 //------------------------------------------------------------------------------
 FGRunwayRef FGAirport::getRunwayByIdent(const std::string& aIdent) const
 {
-  loadRunways();
-  for (auto rwy : mRunways) {
-    if (rwy->ident() == aIdent) {
-      return rwy;
-    }
+    if (aIdent.empty())
+        return {};
+
+    loadRunways();
+    for (auto rwy : mRunways) {
+        if (rwy->ident() == aIdent) {
+            return rwy;
+        }
   }
 
   SG_LOG(SG_GENERAL, SG_ALERT, "no such runway '" << aIdent << "' at airport " << ident());
@@ -354,39 +350,71 @@ FGTaxiwayList FGAirport::getTaxiways() const
 //------------------------------------------------------------------------------
 unsigned int FGAirport::numPavements() const
 {
-  loadTaxiways();
   return mPavements.size();
 }
 
-//------------------------------------------------------------------------------
-FGPavementRef FGAirport::getPavementByIndex(unsigned int aIndex) const
-{
-  loadTaxiways();
-  return loadById<FGPavement>(mPavements, aIndex);
-}
-
 //------------------------------------------------------------------------------
 FGPavementList FGAirport::getPavements() const
 {
-  loadTaxiways();
-  return loadAllById<FGPavement>(mPavements);
+  return mPavements;
+}
+
+void FGAirport::addPavement(FGPavementRef pavement)
+{
+  mPavements.push_back(pavement);
+}
+
+//------------------------------------------------------------------------------
+unsigned int FGAirport::numBoundary() const
+{
+  return mBoundary.size();
+}
+
+//------------------------------------------------------------------------------
+FGPavementList FGAirport::getBoundary() const
+{
+  return mBoundary;
+}
+
+void FGAirport::addBoundary(FGPavementRef boundary)
+{
+  mBoundary.push_back(boundary);
+}
+
+//------------------------------------------------------------------------------
+unsigned int FGAirport::numLineFeatures() const
+{
+  return mLineFeatures.size();
+}
+
+//------------------------------------------------------------------------------
+FGPavementList FGAirport::getLineFeatures() const
+{
+  return mLineFeatures;
+}
+
+void FGAirport::addLineFeature(FGPavementRef linefeature)
+{
+  mLineFeatures.push_back(linefeature);
 }
 
 //------------------------------------------------------------------------------
 FGRunwayRef FGAirport::getActiveRunwayForUsage() const
 {
-  FGEnvironmentMgr* envMgr = (FGEnvironmentMgr *) globals->get_subsystem("environment");
-
+  auto envMgr = globals->get_subsystem<FGEnvironmentMgr>();
+  
   // This forces West-facing rwys to be used in no-wind situations
   // which is consistent with Flightgear's initial setup.
   double hdg = 270;
 
   if (envMgr) {
-    FGEnvironment stationWeather(envMgr->getEnvironment(geod()));
+      // FIXME : this should use the weather at the airport, not the player's
+      // location.
+      const auto stationWeather = envMgr->getAircraftEnvironment();
 
-    double windSpeed = stationWeather.get_wind_speed_kt();
+    double windSpeed = stationWeather->get_wind_speed_kt();
     if (windSpeed > 0.0) {
-      hdg = stationWeather.get_wind_from_heading_deg();
+      hdg = stationWeather->get_wind_from_heading_deg();
     }
   }
 
@@ -559,6 +587,29 @@ void FGAirport::loadProcedures() const
   RouteBase::loadAirportProcedures(path, const_cast<FGAirport*>(this));
 }
 
+void FGAirport::loadRunwayRenames() const
+{
+    if (mRunwayRenamesLoaded) {
+        return;
+    }
+
+    SGPath path;
+    if (!XMLLoader::findAirportData(ident(), "runway_rename", path)) {
+        // No rename for airport; ignore
+        mRunwayRenamesLoaded = true;
+        return;
+    }
+
+    try {
+        SGPropertyNode_ptr rootNode = new SGPropertyNode;
+        readProperties(path, rootNode);
+        const_cast<FGAirport*>(this)->parseRunwayRenameData(rootNode);
+        mRunwayRenamesLoaded = true;
+    } catch (sg_exception& e) {
+        SG_LOG(SG_NAVAID, SG_WARN, ident() << "loading runway rename XML failed:" << e.getFormattedMessage());
+    }
+}
+
 void FGAirport::loadSceneryDefinitions() const
 {
   if (mThresholdDataLoaded) {
@@ -577,7 +628,9 @@ void FGAirport::loadSceneryDefinitions() const
     readProperties(path, rootNode);
     const_cast<FGAirport*>(this)->readThresholdData(rootNode);
   } catch (sg_exception& e) {
-    SG_LOG(SG_NAVAID, SG_WARN, ident() << "loading threshold XML failed:" << e.getFormattedMessage());
+    simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::TerraSync,
+                           "Airport threshold data could not be loaded:" + e.getFormattedMessage(),
+                           path);
   }
 }
 
@@ -588,8 +641,10 @@ void FGAirport::readThresholdData(SGPropertyNode* aRoot)
   for (; (runway = aRoot->getChild("runway", runwayIndex)) != NULL; ++runwayIndex) {
     SGPropertyNode* t0 = runway->getChild("threshold", 0),
       *t1 = runway->getChild("threshold", 1);
-    assert(t0);
-    assert(t1); // too strict? maybe we should finally allow single-ended runways
+    if (!t0 || !t1) {
+      throw sg_io_exception("Mis-configured runway threshold data: exactly two thresholds must be defined",
+                            sg_location{runway}, "FGAirport::readThresholdData", false);
+    }
 
     processThreshold(t0);
     processThreshold(t1);
@@ -612,12 +667,14 @@ void FGAirport::processThreshold(SGPropertyNode* aThreshold)
   double newStopway = aThreshold->getDoubleValue("stopw-m");
 
   if (id == 0) {
-    SG_LOG(SG_GENERAL, SG_DEBUG, "FGAirport::processThreshold: "
-           "found runway not defined in the global data:" << ident() << "/" << rwyIdent);
+    const auto runwayIdent = ident() + "/" + rwyIdent;
+    SG_LOG(SG_NAVAID, SG_DEV_WARN, "Threshold.xml contains runway not included in apt.dat:" << 
+      sg_location{aThreshold}.asString() << ": runway=" << runwayIdent);
+#if 0
     // enable this code when threshold.xml contains sufficient data to
     // fully specify a new runway, *and* we figure out how to assign runtime
     // Positioned IDs and insert temporary items into the spatial map.
-#if 0
+
     double newLength = 0.0, newWidth = 0.0;
     int surfaceCode = 0;
     FGRunway* rwy = new FGRunway(id, guid(), rwyIdent, newThreshold,
@@ -703,6 +760,38 @@ void FGAirport::readTowerData(SGPropertyNode* aRoot)
   mTowerPosition = SGGeod::fromDegM(lon, lat, fieldElevationM + elevM);
 }
 
+void FGAirport::parseRunwayRenameData(SGPropertyNode* aRoot)
+{
+    SGPropertyNode* overrideNode = aRoot->getChild("runway-rename");
+    for (auto rnm : overrideNode->getChildren("runway")) {
+        const std::string oldIdent = rnm->getStringValue("old-ident");
+        const std::string newIdent = rnm->getStringValue("new-ident");
+        if (oldIdent.empty() || newIdent.empty()) {
+            SG_LOG(SG_NAVAID, SG_WARN, ident() << ": runway rename: Skipping bad runway rename entry");
+            continue;
+        }
+
+        if (!hasRunwayWithIdent(oldIdent)) {
+            SG_LOG(SG_NAVAID, SG_WARN, ident() << ": no old runway with ident:" << oldIdent);
+            continue;
+        }
+
+        _renamedRunways[newIdent] = oldIdent;
+    }
+}
+
+std::string FGAirport::findAPTRunwayForNewName(const std::string& newIdent) const
+{
+    loadRunwayRenames();
+
+    auto it = _renamedRunways.find(newIdent);
+    if (it == _renamedRunways.end())
+        return {};
+
+    return it->second;
+}
+
+
 void FGAirport::validateILSData()
 {
   if (mILSDataLoaded) {
@@ -919,7 +1008,7 @@ FGAirport::commStationsOfType(FGPositioned::Type aTy) const
 class AirportWithSize
 {
 public:
-    AirportWithSize(FGPositionedRef pos) :
+    explicit AirportWithSize(FGPositionedRef pos) :
         _pos(pos),
         _sizeMetric(0)
     {
@@ -965,9 +1054,7 @@ FGGroundNetwork *FGAirport::groundNetwork() const
 {
     if (!_groundNetwork.get()) {
         _groundNetwork.reset(new FGGroundNetwork(const_cast<FGAirport*>(this)));
-
         XMLLoader::load(_groundNetwork.get());
-
         _groundNetwork->init();
     }
 
@@ -986,10 +1073,13 @@ flightgear::Transition* FGAirport::selectSIDByEnrouteTransition(FGPositioned* en
     return nullptr;
 }
 
-Transition *FGAirport::selectSIDByTransition(const string &aIdent) const
+Transition *FGAirport::selectSIDByTransition(const FGRunway* runway,  const std::string &aIdent) const
 {
     loadProcedures();
     for (auto sid : mSIDs) {
+        if (runway && !sid->isForRunway(runway))
+            continue;
+
         auto trans = sid->findTransitionByName(aIdent);
         if (trans) {
             return trans;
@@ -1010,10 +1100,13 @@ flightgear::Transition* FGAirport::selectSTARByEnrouteTransition(FGPositioned* e
     return nullptr;
 }
 
-Transition *FGAirport::selectSTARByTransition(const string &aIdent) const
+Transition *FGAirport::selectSTARByTransition(const FGRunway* runway, const std::string &aIdent) const
 {
     loadProcedures();
     for (auto star : mSTARs) {
+        if (runway && !star->isForRunway(runway))
+            continue;
+
         auto trans = star->findTransitionByName(aIdent);
         if (trans) {
             return trans;
