@@ -18,6 +18,7 @@
 #endif
 
 #include <cstdio>
+#include <utility>
 
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/xml/easyxml.hxx>
@@ -54,39 +55,51 @@ void XMLLoader::load(FGGroundNetwork* net)
   SGTimeStamp t;
   t.stamp();
   try {
-      flightgear::sentryThreadReportXMLErrors(false);
       FGGroundNetXMLLoader visitor(net);
       readXML(path, visitor);
+
+      if (visitor.hasErrors() && fgGetBool("/sim/terrasync/enabled")) {
+          flightgear::updateSentryTag("ground-net", net->airport()->ident());
+          
+          // disabled for 2024.1.4 to reduce Sentry quota spam
+          //flightgear::sentryReportException("Ground-net load error", path.utf8Str());
+      }
   } catch (sg_exception& e) {
     SG_LOG(SG_NAVAID, SG_DEV_WARN, "parsing groundnet XML failed:" << e.getFormattedMessage());
   }
 
-  flightgear::sentryThreadReportXMLErrors(true);
   SG_LOG(SG_NAVAID, SG_DEBUG, "parsing groundnet XML took " << t.elapsedMSec());
 }
 
 void XMLLoader::loadFromStream(FGGroundNetwork* net, std::istream& inData)
 {
   try {
-      flightgear::sentryThreadReportXMLErrors(false);
       FGGroundNetXMLLoader visitor(net);
       readXML(inData, visitor);
+
+      if (visitor.hasErrors()) {
+          // disabled for 2024.1.4 to reduce Sentry quota spam
+          // flightgear::sentryReportException("Ground-net load error", {});
+      }
   } catch (sg_exception& e) {
     SG_LOG(SG_NAVAID, SG_DEV_WARN, "parsing groundnet XML failed:" << e.getFormattedMessage());
   }
-  flightgear::sentryThreadReportXMLErrors(true);
 }
 
 void XMLLoader::loadFromPath(FGGroundNetwork* net, const SGPath& path)
 {
   try {
-      flightgear::sentryThreadReportXMLErrors(false);
       FGGroundNetXMLLoader visitor(net);
       readXML(path, visitor);
+
+      if (visitor.hasErrors() && fgGetBool("/sim/terrasync/enabled")) {
+          flightgear::updateSentryTag("ground-net", net->airport()->ident());
+          // disabled for 2024.1.4 to reduce Sentry quota spam
+          // flightgear::sentryReportException("Ground-net load error", path.utf8Str());
+      }
   } catch (sg_exception& e) {
     SG_LOG(SG_NAVAID, SG_DEV_WARN, "parsing groundnet XML failed:" << e.getFormattedMessage());
   }
-  flightgear::sentryThreadReportXMLErrors(true);
 }
 
 void XMLLoader::load(FGRunwayPreference* p) {
@@ -97,11 +110,19 @@ void XMLLoader::load(FGRunwayPreference* p) {
 bool XMLLoader::findAirportData(const std::string& aICAO, 
     const std::string& aFileName, SGPath& aPath)
 {
+  FGAirportRef airport = FGAirport::findByIdent(aICAO);
+  if (!airport) {
+      return false;
+  }
+
   string fileName(aFileName);
   if (!simgear::strutils::ends_with(aFileName, ".xml")) {
     fileName.append(".xml");
   }
-  
+
+  const bool performFullTraversal = airport->sceneryPath().isNull() ||
+    !fileName.compare("procedures.xml");
+
   PathList sc = globals->get_fg_scenery();
   char buffer[128];
   ::snprintf(buffer, 128, "%c/%c/%c/%s.%s", 
@@ -111,13 +132,18 @@ bool XMLLoader::findAirportData(const std::string& aICAO,
     for (PathList::const_iterator it = sc.begin(); it != sc.end(); ++it) {
     // fg_senery contains empty strings as "markers" (see FGGlobals::set_fg_scenery)
     if (!it->isNull()) {
-        SGPath path(*it);
-        path.append("Airports");
-        path.append(string(buffer));
+        const SGPath path = *it / "Airports" / buffer;
         if (path.exists()) {
-          aPath = path;
+          aPath = std::move(path);
           return true;
         } // of path exists
+
+        // Unless we are in “full traversal mode”, don't look in scenery paths
+        // that come after the one which contributed the apt.dat file for the
+        // airport.
+        if (!performFullTraversal && *it == airport->sceneryPath()) {
+            return false;
+        }
     }
   } // of scenery path iteration
   return false;
@@ -134,7 +160,6 @@ bool XMLLoader::loadAirportXMLDataIntoVisitor(const string& aICAO,
 
     bool readXMLOk = true;
     try {
-        flightgear::sentryThreadReportXMLErrors(false);
         SG_LOG(SG_NAVAID, SG_DEBUG, "loadAirportXMLDataIntoVisitor: loading from " << path);
         readXML(path, aVisitor);
     } catch (sg_exception& e) {
@@ -142,7 +167,6 @@ bool XMLLoader::loadAirportXMLDataIntoVisitor(const string& aICAO,
         SG_LOG(SG_NAVAID, SG_WARN, "XML errors trying to read:" << path);
     }
     
-    flightgear::sentryThreadReportXMLErrors(true);
     return readXMLOk;
 }
 
