@@ -115,8 +115,8 @@ void readInterfaceProperties( SGPropertyNode_ptr prop_root,
     {
       prop->setDoubleValue( val->getDoubleValue() );
 
-      // TODO should we keep the _attr_ node, as soon as the property browser is
-      //      able to cope with it?
+      // TODO: should we keep the _attr_ node, as soon as the property browser is
+      //       able to cope with it?
       (*it)->removeChild("_attr_", 0);
     }
   }
@@ -167,7 +167,7 @@ Autopilot::Autopilot( SGPropertyNode_ptr rootNode, SGPropertyNode_ptr configNode
   for( int i = 0; i < count; ++i )
   {
     SGPropertyNode_ptr node = configNode->getChild(i);
-    string childName = node->getName();
+    string childName = node->getNameString();
     if(    childName == "property"
         || childName == "property-root" )
       continue;
