@@ -1,26 +1,8 @@
 // route_mgr.cxx - manage a route (i.e. a collection of waypoints)
-//
-// Written by Curtis Olson, started January 2004.
-//            Norman Vine
-//            Melchior FRANZ
-//
-// Copyright (C) 2004  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileCopyrightText: (C) 2004 Curtis L. Olson http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 
 #include <config.h>
@@ -54,19 +36,24 @@
 #define RM "/autopilot/route-manager/"
 
 using namespace flightgear;
+using namespace std::string_literals;
 using std::string;
 namespace su = simgear::strutils;
 
+// we add this entry to some combo-boxes in route-manager.xml (the dialog),
+// and hence PUI will send them to us.
+const std::string kNoneString = "(none)"s;
+
 static bool commandLoadFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   SGPath path = SGPath::fromUtf8(arg->getStringValue("path"));
   return self->loadRoute(path);
 }
 
 static bool commandSaveFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   SGPath path = SGPath::fromUtf8(arg->getStringValue("path"));
   const SGPath authorizedPath = SGPath(path).validate(true /* write */);
 
@@ -88,7 +75,7 @@ static bool commandSaveFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 
 static bool commandActivateFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   bool activate = arg->getBoolValue("activate", true);
   if (activate) {
     self->activate();
@@ -101,14 +88,14 @@ static bool commandActivateFlightPlan(const SGPropertyNode* arg, SGPropertyNode
 
 static bool commandClearFlightPlan(const SGPropertyNode*, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   self->clearRoute();
   return true;
 }
 
 static bool commandSetActiveWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   int index = arg->getIntValue("index");
   if ((index < 0) || (index >= self->numLegs())) {
     return false;
@@ -120,29 +107,40 @@ static bool commandSetActiveWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 
 static bool commandInsertWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
-  const bool haveIndex = arg->hasChild("index");
-  int index = arg->getIntValue("index");
+    auto self = globals->get_subsystem<FGRouteMgr>();
+    const bool haveIndex = arg->hasChild("index");
+    int index = arg->getIntValue("index");
 
-  std::string ident(arg->getStringValue("id"));
-  int alt = arg->getIntValue("altitude-ft", -999);
-  int ias = arg->getIntValue("speed-knots", -999);
-  
-  WayptRef wp;
-// lat/lon may be supplied to narrow down navaid search, or to specify
-// a raw waypoint
-  SGGeod pos;
-  if (arg->hasChild("longitude-deg")) {
-    pos = SGGeod::fromDeg(arg->getDoubleValue("longitude-deg"),
-                               arg->getDoubleValue("latitude-deg"));
+    std::string ident(arg->getStringValue("id"));
+    int alt = arg->getIntValue("altitude-ft", -999);
+    int ias = arg->getIntValue("speed-knots", -999);
+
+    WayptRef wp;
+    // lat/lon may be supplied to narrow down navaid search, or to specify
+    // a raw waypoint
+    SGGeod pos = SGGeod::invalid();
+    if (arg->hasChild("longitude-deg")) {
+        pos = SGGeod::fromDeg(arg->getDoubleValue("longitude-deg"),
+                              arg->getDoubleValue("latitude-deg"));
   }
   
   if (arg->hasChild("navaid")) {
-    FGPositionedRef p = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid"), pos);
-    
+      if (!pos.isValid()) {
+          pos = self->flightPlan()->vicinityForInsertIndex(haveIndex ? index : -1 /* append */);
+      }
+
+      FGPositioned::TypeFilter filter({FGPositioned::Type::NDB, FGPositioned::Type::VOR,
+                                       FGPositioned::Type::FIX, FGPositioned::Type::WAYPOINT});
+
+      FGPositionedRef p = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid"), pos, &filter);
+      if (!p) {
+          SG_LOG(SG_AUTOPILOT, SG_WARN, "Unable to find navaid with ident:" << arg->getStringValue("navaid"));
+          return false;
+      }
+
     if (arg->hasChild("navaid", 1)) {
       // intersection of two radials
-      FGPositionedRef p2 = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid[1]"), pos);
+      FGPositionedRef p2 = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid[1]"), pos, &filter);
       if (!p2) {
         SG_LOG( SG_AUTOPILOT, SG_INFO, "Unable to find FGPositioned with ident:" << arg->getStringValue("navaid[1]"));
         return false;
@@ -223,7 +221,7 @@ static bool commandInsertWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 
 static bool commandDeleteWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   int index = arg->getIntValue("index");
   self->removeLegAtIndex(index);
   return true;
@@ -289,7 +287,11 @@ void FGRouteMgr::init() {
   departure->tie("sid", SGStringValueMethods<FGRouteMgr>(*this, 
                                                          &FGRouteMgr::getSID,
                                                          &FGRouteMgr::setSID));
-  
+
+  departure->tie("sid-with-trans", SGStringValueMethods<FGRouteMgr>(*this,
+                                                                    &FGRouteMgr::getSIDWithTransition,
+                                                                    &FGRouteMgr::setSID));
+
   departure->tie("name", SGStringValueMethods<FGRouteMgr>(*this, 
     &FGRouteMgr::getDepartureName, nullptr));
   departure->tie("field-elevation-ft", SGRawValueMethods<FGRouteMgr, double>(*this, 
@@ -308,10 +310,16 @@ void FGRouteMgr::init() {
   destination->tie("star", SGStringValueMethods<FGRouteMgr>(*this, 
                                                             &FGRouteMgr::getSTAR,
                                                             &FGRouteMgr::setSTAR));
+  destination->tie("star-with-trans", SGStringValueMethods<FGRouteMgr>(*this,
+                                                                       &FGRouteMgr::getSTARWithTransition,
+                                                                       &FGRouteMgr::setSTAR));
   destination->tie("approach", SGStringValueMethods<FGRouteMgr>(*this, 
                                                                 &FGRouteMgr::getApproach,
                                                                 &FGRouteMgr::setApproach));
-  
+  destination->tie("approach-with-trans", SGStringValueMethods<FGRouteMgr>(*this,
+                                                                           &FGRouteMgr::getApproachWithTransition,
+                                                                           &FGRouteMgr::setApproach));
+
   destination->tie("name", SGStringValueMethods<FGRouteMgr>(*this, 
     &FGRouteMgr::getDestinationName, nullptr));
   destination->tie("field-elevation-ft", SGRawValueMethods<FGRouteMgr, double>(*this, 
@@ -361,7 +369,8 @@ void FGRouteMgr::init() {
     
   _edited = fgGetNode(RM "signals/edited", true);
   _flightplanChanged = fgGetNode(RM "signals/flightplan-changed", true);
-  
+  _isRoute = fgGetNode(RM "is-route", true);
+
   _currentWpt = fgGetNode(RM "current-wp", true);
   _currentWpt->setAttribute(SGPropertyNode::LISTENER_SAFE, true);
   _currentWpt->tie(SGRawValueMethods<FGRouteMgr, int>
@@ -391,7 +400,7 @@ void FGRouteMgr::init() {
 
 void FGRouteMgr::postinit()
 {
-  setFlightPlan(new FlightPlan());
+  setFlightPlan(FlightPlan::create());
   _plan->setIdent("default-flightplan");
     
   SGPath path = SGPath::fromUtf8(_pathNode->getStringValue());
@@ -400,22 +409,22 @@ void FGRouteMgr::postinit()
     loadRoute(path);
   }
   
+  _isRoute->setBoolValue(_plan->isRoute());
+
 // this code only matters for the --wp option now - perhaps the option
 // should be deprecated in favour of an explicit flight-plan file?
 // then the global initial waypoint list could die.
   string_list *waypoints = globals->get_initial_waypoints();
   if (waypoints) {
-    string_list::iterator it;
-    for (it = waypoints->begin(); it != waypoints->end(); ++it) {
-        WayptRef w = waypointFromString(*it, -1);
+    for (const auto& wpStr : *waypoints) {
+        WayptRef w = waypointFromString(wpStr, -1);
         if (w) {
             _plan->insertWayptAtIndex(w, -1);
         } else {
-            SG_LOG(SG_AUTOPILOT, SG_WARN, "Failed to create waypoint from '" << *it << "'");
+            SG_LOG(SG_AUTOPILOT, SG_WARN, "Failed to create waypoint from '" << wpStr << "'");
         }
     }
     
-    SG_LOG(SG_AUTOPILOT, SG_INFO, "loaded initial waypoints:" << numLegs());
     update_mirror();
   }
 
@@ -444,9 +453,8 @@ bool FGRouteMgr::saveRoute(const SGPath& p)
 
 bool FGRouteMgr::loadRoute(const SGPath& p)
 {
-  FlightPlan* fp = new FlightPlan;
+  FlightPlan* fp = FlightPlan::create();
   if (!fp->load(p)) {
-    delete fp;
     return false;
   }
   
@@ -477,7 +485,7 @@ void FGRouteMgr::setFlightPlan(const FlightPlanRef& plan)
   
   _plan = plan;
   _plan->addDelegate(this);
-  
+  _isRoute->setBoolValue(_plan->isRoute());
   _flightplanChanged->fireValueChanged();
   
 // fire all the callbacks!
@@ -487,6 +495,25 @@ void FGRouteMgr::setFlightPlan(const FlightPlanRef& plan)
   currentWaypointChanged();
 }
 
+void FGRouteMgr::departureChanged()
+{
+    auto gui = globals->get_subsystem<NewGUI>();
+    FGDialog* rmDlg = gui ? gui->getDialog("route-manager") : NULL;
+    if (rmDlg) {
+        rmDlg->runCallback("departure-changed");
+    }
+}
+
+void FGRouteMgr::arrivalChanged()
+{
+    auto gui = globals->get_subsystem<NewGUI>();
+    FGDialog* rmDlg = gui ? gui->getDialog("route-manager") : NULL;
+    if (rmDlg) {
+        rmDlg->runCallback("arrival-changed");
+    }
+}
+
+
 void FGRouteMgr::update( double dt )
 {
   if (dt <= 0.0) {
@@ -526,8 +553,11 @@ void FGRouteMgr::update( double dt )
   }
   
   // use RoutePath to compute location of active WP
-  RoutePath path(_plan);
-  SGGeod wpPos = path.positionForIndex(_plan->currentIndex());
+  if (!_routePath) {
+    _routePath.reset(new RoutePath{_plan});
+  }
+
+  SGGeod wpPos = _routePath->positionForIndex(_plan->currentIndex());
   double courseDeg, az2, distanceM;
   SGGeodesy::inverse(currentPos, wpPos, courseDeg, az2, distanceM);
 
@@ -552,7 +582,7 @@ void FGRouteMgr::update( double dt )
   
   FlightPlan::Leg* nextLeg = _plan->nextLeg();
   if (nextLeg) {
-    wpPos = path.positionForIndex(_plan->currentIndex() + 1);
+    wpPos = _routePath->positionForIndex(_plan->currentIndex() + 1);
     SGGeodesy::inverse(currentPos, wpPos, courseDeg, az2, distanceM);
 
     wp1->setDoubleValue("dist", distanceM * SG_METER_TO_NM);
@@ -574,6 +604,7 @@ void FGRouteMgr::update( double dt )
 
 void FGRouteMgr::clearRoute()
 {
+  _routePath.reset();
   if (_plan) {
       _plan->clearLegs();
   }
@@ -658,8 +689,9 @@ void FGRouteMgr::waypointsChanged()
 // mirror internal route to the property system for inspection by other subsystems
 void FGRouteMgr::update_mirror()
 {
+    _routePath.reset(); // wipe this so we re-compute on next update()
   mirror->removeChildren("wp");
-  NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
+  auto gui = globals->get_subsystem<NewGUI>();
   FGDialog* rmDlg = gui ? gui->getDialog("route-manager") : NULL;
 
   if (!_plan) {
@@ -721,7 +753,7 @@ void FGRouteMgr::update_mirror()
   
   // set number as listener attachment point
   mirror->setIntValue("num", _plan->numLegs());
-    
+
   if (rmDlg) {
     rmDlg->updateValues();
   }
@@ -917,6 +949,22 @@ std::string FGRouteMgr::getSID() const
   return "";
 }
 
+std::string FGRouteMgr::getSIDWithTransition() const
+{
+    if (!_plan || !_plan->sid()) {
+        return {};
+    }
+
+    auto sid = _plan->sid();
+    auto trans = _plan->sidTransition();
+    if (trans) {
+        return sid->ident() + "-"s + trans->ident();
+    }
+
+    // no transition defined
+    return sid->ident();
+}
+
 static double headingDiffDeg(double a, double b)
 {
   double rawDiff = b - a;
@@ -987,11 +1035,11 @@ void FGRouteMgr::setSID(const std::string& aIdent)
     }
     
   FGAirport* apt = _plan->departureAirport();
-  if (!apt || aIdent.empty()) {
-    _plan->setSID((flightgear::SID*) NULL);
-    return;
-  } 
-  
+  if (!apt || aIdent.empty() || (aIdent == kNoneString)) {
+      _plan->setSID((flightgear::SID*)NULL);
+      return;
+  }
+
   if (aIdent == "DEFAULT") {
     double enrouteCourse = -1.0;
     if (_plan->destinationAirport()) {
@@ -1078,6 +1126,22 @@ std::string FGRouteMgr::getApproach() const
   return "";
 }
 
+std::string FGRouteMgr::getApproachWithTransition() const
+{
+    if (!_plan || !_plan->approach()) {
+        return {};
+    }
+
+    auto app = _plan->approach();
+    auto trans = _plan->approachTransition();
+    if (trans) {
+        return app->ident() + "-"s + trans->ident();
+    }
+
+    // no transition defined
+    return app->ident();
+}
+
 flightgear::Approach* createDefaultApproach(FGRunway* aRunway, double aEnrouteCourse)
 {
   if (!aRunway) {
@@ -1154,11 +1218,11 @@ void FGRouteMgr::setApproach(const std::string& aIdent)
     _plan->setApproach(createDefaultApproach(_plan->destinationRunway(), enrouteCourse));
     return;
   }
-  
-  if (!apt || aIdent.empty()) {
+
+  if (!apt || aIdent.empty() || (aIdent == kNoneString)) {
       _plan->setApproach(static_cast<Approach*>(nullptr));
   } else {
-    _plan->setApproach(apt->findApproachWithIdent(aIdent));
+      _plan->setApproach(apt->findApproachWithIdent(aIdent));
   }
 }
 
@@ -1171,6 +1235,22 @@ std::string FGRouteMgr::getSTAR() const
   return "";
 }
 
+std::string FGRouteMgr::getSTARWithTransition() const
+{
+    if (!_plan || !_plan->star()) {
+        return {};
+    }
+
+    auto star = _plan->star();
+    auto trans = _plan->starTransition();
+    if (trans) {
+        return star->ident() + "-"s + trans->ident();
+    }
+
+    // no transition defined
+    return star->ident();
+}
+
 void FGRouteMgr::setSTAR(const std::string& aIdent)
 {
     if (!_plan) {
@@ -1178,11 +1258,11 @@ void FGRouteMgr::setSTAR(const std::string& aIdent)
     }
     
   FGAirport* apt = _plan->destinationAirport();
-  if (!apt || aIdent.empty()) {
-    _plan->setSTAR((STAR*) NULL);
-    return;
-  } 
-  
+  if (!apt || aIdent.empty() || (aIdent == kNoneString)) {
+      _plan->setSTAR((STAR*)NULL);
+      return;
+  }
+
   string ident(aIdent);
   size_t hyphenPos = ident.find('-');
   if (hyphenPos != string::npos) {
@@ -1199,24 +1279,7 @@ void FGRouteMgr::setSTAR(const std::string& aIdent)
 
 WayptRef FGRouteMgr::waypointFromString(const std::string& target, int insertPosition)
 {
-    SGGeod vicinity;
-    if (insertPosition < 0) { // appending, not inserting
-        insertPosition = _plan->numLegs();
-        if (insertPosition > 0) {
-            // if we have at least one existing leg, use its position
-            // for the search vicinity
-            vicinity = _plan->pointAlongRoute(insertPosition - 1, 0.0);
-        }
-    } else {
-        // if we're somewhere in the middle of the route compute a search
-        // vicinity halfwya between the previous waypoint and the one we are
-        // inserting at, i.e th emiddle of the leg.
-        // if we're at the beginning, just use zero of course.
-        const double normOffset = (insertPosition > 0) ? -0.5 : 0.0;
-        vicinity = _plan->pointAlongRouteNorm(insertPosition, normOffset);
-    }
-
-    return _plan->waypointFromString(target, vicinity);
+    return _plan->waypointFromString(target, _plan->vicinityForInsertIndex(insertPosition));
 }
 
 double FGRouteMgr::getDepartureFieldElevation() const
@@ -1351,9 +1414,12 @@ bool FGRouteMgr::commandDefineUserWaypoint(const SGPropertyNode * arg, SGPropert
         return false;
     }
 
+    const bool temporary = arg->getBoolValue("temporary");
     SGGeod pos(SGGeod::fromDeg(arg->getDoubleValue("longitude-deg"),
                                arg->getDoubleValue("latitude-deg")));
-    FGPositioned::createUserWaypoint(ident, pos);
+    const auto name = arg->getStringValue("name");
+    FGPositioned::createWaypoint(FGPositioned::WAYPOINT, ident, pos,
+                                 temporary, name);
     return true;
 }
 
@@ -1364,8 +1430,15 @@ bool FGRouteMgr::commandDeleteUserWaypoint(const SGPropertyNode * arg, SGPropert
         SG_LOG(SG_AUTOPILOT, SG_WARN, "missing ident deleting user waypoint");
         return false;
     }
-    
-    return FGPositioned::deleteUserWaypoint(ident);
+
+    FGPositioned::TypeFilter f(FGPositioned::WAYPOINT);
+    auto existing = FGPositioned::findFirstWithIdent(ident, &f);
+    if (!existing) {
+      SG_LOG(SG_AUTOPILOT, SG_WARN, "no user waypoint with ident:" << ident);
+      return false;
+    }
+
+    return FGPositioned::deleteWaypoint(existing);
 }
 
 
