@@ -1,22 +1,9 @@
-// environment-mgr.cxx -- manager for natural environment information.
-//
-// Written by David Megginson, started February 2002.
-//
-// Copyright (C) 2002  David Megginson - david@megginson.com
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * SPDX-FileName: environment_mgr.cxx
+ * SPDX-FileComment: manager for natural environment information
+ * SPDX-FileCopyrightText: Copyright (C) 2002  David Megginson - david@megginson.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -48,8 +35,11 @@
 #include "terrainsampler.hxx"
 #include "Airports/airport.hxx"
 #include "gravity.hxx"
+#include "climate.hxx"
 #include "magvarmanager.hxx"
 
+#include "AIModel/AINotifications.hxx"
+
 class FG3DCloudsListener : public SGPropertyChangeListener {
 public:
   FG3DCloudsListener( FGClouds * fgClouds );
@@ -81,20 +71,25 @@ void FG3DCloudsListener::valueChanged( SGPropertyNode * node )
   _fgClouds->set_3dClouds( _enableNode->getBoolValue() );
 }
 
-FGEnvironmentMgr::FGEnvironmentMgr () :
-  _environment(new FGEnvironment()),
-  _sky(globals->get_renderer()->getSky())
+FGEnvironmentMgr::FGEnvironmentMgr() : _environment(new FGEnvironment()),
+                                       _sky(globals->get_renderer()->getSky()),
+                                       nearestCarrier(nullptr),
+                                       nearestAirport(nullptr)
 {
   fgClouds = new FGClouds;
   _3dCloudsEnableListener = new FG3DCloudsListener(fgClouds);
   set_subsystem("controller", Environment::LayerInterpolateController::createInstance( fgGetNode("/environment/config", true ) ));
 
+  set_subsystem("climate", new FGClimate);
   set_subsystem("precipitation", new FGPrecipitationMgr);
   set_subsystem("realwx", Environment::RealWxController::createInstance( fgGetNode("/environment/realwx", true ) ), 1.0 );
   set_subsystem("terrainsampler", Environment::TerrainSampler::createInstance( fgGetNode("/environment/terrain", true ) ));
   set_subsystem("ridgelift", new FGRidgeLift);
 
   set_subsystem("magvar", new FGMagVarManager);
+  max_tower_height_feet = fgGetDouble("/sim/airport/max-tower-height-ft", 70);
+  min_tower_height_feet = fgGetDouble("/sim/airport/min-tower-height-ft", 6);
+  default_tower_height_feet = fgGetDouble("default-tower-height-ft", 30);
 }
 
 FGEnvironmentMgr::~FGEnvironmentMgr ()
@@ -111,36 +106,55 @@ FGEnvironmentMgr::~FGEnvironmentMgr ()
   delete _environment;
 }
 
-struct FGEnvironmentMgrMultiplayerListener : SGPropertyChangeListener {
-    FGEnvironmentMgrMultiplayerListener(FGEnvironmentMgr* environmentmgr)
-    :
-    _environmentmgr(environmentmgr)
+
+struct FGEnvironmentMgrPropertyListener : SGPropertyChangeListener {
+    FGEnvironmentMgrPropertyListener(FGEnvironmentMgr* environmentmgr) : _environmentmgr(environmentmgr)
     {
-        _node = fgGetNode("/sim/current-view/model-view", true /*create*/);
-        _node->addChangeListener(this);
+        _modelViewNode = fgGetNode("/sim/current-view/model-view", true /*create*/);
+        _modelViewNode->addChangeListener(this);
+        _viewNumberNode = fgGetNode("/sim/current-view/view-number-raw", true /*create*/);
+        _viewNumberNode->addChangeListener(this);
+        _airportIdNode = fgGetNode("/sim/tower/airport-id", true /*create*/);
+        _airportIdNode->addChangeListener(this);
+        _autoTowerNode = fgGetNode("/sim/tower/auto-position", true /*create*/);
+        _autoTowerNode->addChangeListener(this);
     }
-    virtual void valueChanged(SGPropertyNode* node)
+
+    void valueChanged(SGPropertyNode* node) override
     {
-        _environmentmgr->updateClosestAirport();
+        if ((node == _modelViewNode) || (_autoTowerNode == node)) {
+            _environmentmgr->updateClosestAirport();
+        }
+
+        if ((node == _viewNumberNode) || (node == _airportIdNode)) {
+            _environmentmgr->onTowerAirportIDChanged();
+        }
     }
-    virtual ~FGEnvironmentMgrMultiplayerListener()
+    virtual ~FGEnvironmentMgrPropertyListener()
     {
-        _node->removeChangeListener(this);
+        _modelViewNode->removeChangeListener(this);
+        _viewNumberNode->removeChangeListener(this);
+        _airportIdNode->removeChangeListener(this);
+        _airportIdNode->removeChangeListener(this);
     }
     private:
         FGEnvironmentMgr*   _environmentmgr;
-        SGPropertyNode_ptr  _node;
+        SGPropertyNode_ptr _modelViewNode;
+        SGPropertyNode_ptr _viewNumberNode;
+        SGPropertyNode_ptr _airportIdNode;
+        SGPropertyNode_ptr _autoTowerNode;
 };
 
+
 SGSubsystem::InitStatus FGEnvironmentMgr::incrementalInit()
 {
 
   InitStatus r = SGSubsystemGroup::incrementalInit();
   if (r == INIT_DONE) {
     fgClouds->Init();
-    _multiplayerListener = new FGEnvironmentMgrMultiplayerListener(this);
-    globals->get_event_mgr()->addTask("updateClosestAirport", this,
-                                      &FGEnvironmentMgr::updateClosestAirport, 10 );
+    _listener.reset(new FGEnvironmentMgrPropertyListener(this));
+    globals->get_event_mgr()->addTask("updateClosestAirport",
+        [this](){ this->updateClosestAirport(); }, 10 );
   }
 
   return r;
@@ -150,8 +164,7 @@ void
 FGEnvironmentMgr::shutdown()
 {
   globals->get_event_mgr()->removeTask("updateClosestAirport");
-  delete _multiplayerListener;
-  _multiplayerListener = nullptr;
+  _listener.reset();
   SGSubsystemGroup::shutdown();
 }
 
@@ -245,6 +258,8 @@ FGEnvironmentMgr::bind ()
   _tiedProperties.Tie("clouds3d-use-impostors", _sky,
           &SGSky::get_3dCloudUseImpostors,
           &SGSky::set_3dCloudUseImpostors);
+
+    _automaticTowerEnableNode = fgGetNode("/sim/tower/auto-position", true);
 }
 
 void
@@ -258,105 +273,202 @@ FGEnvironmentMgr::unbind ()
 void
 FGEnvironmentMgr::update (double dt)
 {
+  SGGeod aircraftPos(globals->get_aircraft_position());
+
   SGSubsystemGroup::update(dt);
 
-    SGGeod aircraftPos(globals->get_aircraft_position());
   _environment->set_elevation_ft( aircraftPos.getElevationFt() );
 
   auto particlesManager = simgear::ParticlesGlobalManager::instance();
   particlesManager->setWindFrom(_environment->get_wind_from_heading_deg(),
                                 _environment->get_wind_speed_kt());
+  particlesManager->update(dt, globals->get_aircraft_position());
+
   if( _cloudLayersDirty ) {
     _cloudLayersDirty = false;
     fgClouds->set_update_event( fgClouds->get_update_event()+1 );
   }
+  updateDynamicTowerPosition();
 
   fgSetDouble( "/environment/gravitational-acceleration-mps2",
     Environment::Gravity::instance()->getGravity(aircraftPos));
 }
 
-void
-FGEnvironmentMgr::updateClosestAirport()
+void FGEnvironmentMgr::onTowerAirportIDChanged()
 {
-  SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "FGEnvironmentMgr::update: updating closest airport");
+    FGAirportRef apt;
+    const auto automaticTowerActive = _automaticTowerEnableNode->getBoolValue();
 
-  SGGeod pos = globals->get_aircraft_position();
-  FGAirport * nearestAirport = FGAirport::findClosest(pos, 100.0);
-  if( nearestAirport == NULL )
-  {
-    SG_LOG(SG_ENVIRONMENT,SG_INFO,"FGEnvironmentMgr::update: No airport within 100NM range");
-  }
-  else
-  {
-    const string currentId = fgGetString("/sim/airport/closest-airport-id", "");
-    if (currentId != nearestAirport->ident())
-    {
-      SG_LOG(SG_ENVIRONMENT, SG_INFO, "FGEnvironmentMgr::updateClosestAirport: selected:" << nearestAirport->ident());
-      fgSetString("/sim/airport/closest-airport-id",
-                  nearestAirport->ident().c_str());
+    if (automaticTowerActive) {
+        apt = FGAirport::findByIdent(fgGetString("/sim/airport/closest-airport-id"));
+    } else {
+        apt = FGAirport::findByIdent(fgGetString("/sim/tower/airport-id"));
     }
-  }
-  
-  {
-    /* If we are viewing a multiplayer aircraft, find nearest airport so that
-    Tower View etc works. */
-    std::string   view_config_root = ViewPropertyEvaluator::getStringValue("(/sim/view[(/sim/current-view/view-number-raw)]/config/root)");
-    
-    if (view_config_root != "/" && view_config_root != "") {
-      /* We are currently viewing a multiplayer aircraft. */
-      
-      SGGeod  pos = SGGeod::fromDegFt(
-          ViewPropertyEvaluator::getDoubleValue("((/sim/view[(/sim/current-view/view-number-raw)]/config/root)/position/longitude-deg)"),
-          ViewPropertyEvaluator::getDoubleValue("((/sim/view[(/sim/current-view/view-number-raw)]/config/root)/position/latitude-deg)"),
-          ViewPropertyEvaluator::getDoubleValue("((/sim/view[(/sim/current-view/view-number-raw)]/config/root)/position/altitude-ft)")
-          );
-      FGAirport * nearestAirport = FGAirport::findClosest(pos, 100.0);
-      
-      if (nearestAirport) {
-        std::string   path = ViewPropertyEvaluator::getStringValue("(/sim/view[(/sim/current-view/view-number-raw)]/config/root)/sim/tower/");
-        fgSetString(path + "airport-id", nearestAirport->getId());
-        
-        if (nearestAirport->hasTower()) {
-          SGGeod  tower_pos = nearestAirport->getTowerLocation();
-          fgSetDouble(path + "latitude-deg", tower_pos.getLatitudeDeg());
-          fgSetDouble(path + "longitude-deg", tower_pos.getLongitudeDeg());
-          fgSetDouble(path + "altitude-ft", tower_pos.getElevationFt());
-          SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "airport-id=" << nearestAirport->getId() << " tower_pos=" << tower_pos);
-        }
-        else {
-          /* Use location of airport. */
-          fgSetDouble(path + "latitude-deg", nearestAirport->getLatitude());
-          fgSetDouble(path + "longitude-deg", nearestAirport->getLongitude());
-          fgSetDouble(path + "altitude-ft", nearestAirport->getElevation() + 20);
-          SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "no tower for airport-id=" << nearestAirport->getId());
-        }
-      }
-      else {
-        SG_LOG(SG_ENVIRONMENT,SG_DEBUG,"FGEnvironmentMgr::update: No airport within 100NM range of current multiplayer aircraft");
-      }
+
+    if (!apt) {
+        return;
     }
-  }
+
+    SGGeod towerPos;
+    if (apt->hasTower()) {
+        towerPos = apt->getTowerLocation();
+        SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "airport-id=" << apt->getId() << " tower_pos=" << towerPos);
+    } else {
+        towerPos = apt->geod();
+        SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "no tower for airport-id=" << apt->getId());
+    }
+    //Ensure that the tower isn't at ground level by adding a nominal amount
+    // TODO: (fix the data so that too short or too tall towers aren't present in the data)
+    auto towerAirpotDistance = abs(towerPos.getElevationFt() - apt->geod().getElevationFt());
+    if (towerAirpotDistance < min_tower_height_feet) {
+        towerPos.setElevationFt(towerPos.getElevationFt() + default_tower_height_feet);
+        SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "Tower altitude adjusted because it was at below minimum height above ground (" << min_tower_height_feet << "feet) for airport " << apt->getId());
+    } else if (towerAirpotDistance > max_tower_height_feet) {
+        towerPos.setElevationFt(towerPos.getElevationFt() + default_tower_height_feet);
+        SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "Tower altitude adjusted because it was taller than the permitted maximum of (" << max_tower_height_feet << "feet) for airport " << apt->getId());
+    }
+
+    std::string path = ViewPropertyEvaluator::getStringValue("(/sim/view[(/sim/current-view/view-number-raw)]/config/root)/sim/tower/");
+
+    fgSetDouble(path + "latitude-deg", towerPos.getLatitudeDeg());
+    fgSetDouble(path + "longitude-deg", towerPos.getLongitudeDeg());
+    fgSetDouble(path + "altitude-ft", towerPos.getElevationFt());
 }
 
+void FGEnvironmentMgr::updateDynamicTowerPosition()
+{
+    if (towerViewPositionLatDegNode != nullptr && towerViewPositionLonDegNode != nullptr && towerViewPositionAltFtNode != nullptr) {
+        const auto automaticTowerActive = _automaticTowerEnableNode->getBoolValue();
+
+        fgSetDouble("/sim/airport/nearest-tower-latitude-deg",  towerViewPositionLatDegNode->getDoubleValue());
+        fgSetDouble("/sim/airport/nearest-tower-longitude-deg", towerViewPositionLonDegNode->getDoubleValue());
+        fgSetDouble("/sim/airport/nearest-tower-altitude-ft",   towerViewPositionAltFtNode->getDoubleValue());
+
+        if (automaticTowerActive) {
+            fgSetDouble("/sim/tower/latitude-deg", towerViewPositionLatDegNode->getDoubleValue());
+            fgSetDouble("/sim/tower/longitude-deg", towerViewPositionLonDegNode->getDoubleValue());
+            fgSetDouble("/sim/tower/altitude-ft", towerViewPositionAltFtNode->getDoubleValue());
+        }
+    }
+}
+
+void FGEnvironmentMgr::updateClosestAirport()
+{
+    SG_LOG(SG_ENVIRONMENT, SG_DEBUG, "FGEnvironmentMgr::update: updating closest airport");
+
+    SGGeod pos = globals->get_aircraft_position();
+
+    //
+    // If we are viewing a multiplayer aircraft, find nearest airport so that
+    // Tower View etc works.
+    std::string view_config_root = ViewPropertyEvaluator::getStringValue("(/sim/view[(/sim/current-view/view-number-raw)]/config/root)");
+
+    if (view_config_root != "/" && view_config_root != "") {
+        /* We are currently viewing a multiplayer aircraft. */
+        pos = SGGeod::fromDegFt(
+            ViewPropertyEvaluator::getDoubleValue("((/sim/view[(/sim/current-view/view-number-raw)]/config/root)/position/longitude-deg)"),
+            ViewPropertyEvaluator::getDoubleValue("((/sim/view[(/sim/current-view/view-number-raw)]/config/root)/position/latitude-deg)"),
+            ViewPropertyEvaluator::getDoubleValue("((/sim/view[(/sim/current-view/view-number-raw)]/config/root)/position/altitude-ft)"));
+    }
+
+    // nearest tower logic;
+    // 1. find nearest airport
+    // 2. find nearest carrier
+    // - select the nearest one as the tower.
+
+    auto nearestAirport = FGAirport::findClosest(pos, 100.0);
+    const auto automaticTowerActive = _automaticTowerEnableNode->getBoolValue();
+
+    SGGeod nearestTowerPosition;
+    std::string nearestIdent;
+    double towerDistance = std::numeric_limits<double>::max();
+    if (nearestAirport) {
+        const std::string currentId = fgGetString("/sim/airport/closest-airport-id", "");
+        if (currentId != nearestAirport->ident()) {
+            SG_LOG(SG_ENVIRONMENT, SG_INFO, "FGEnvironmentMgr::updateClosestAirport: selected:" << nearestAirport->ident());
+            fgSetString("/sim/airport/closest-airport-id", nearestAirport->ident().c_str());
+        }
+
+        nearestTowerPosition = nearestAirport->geod();
+        nearestIdent = nearestAirport->ident();
+        towerDistance = SGGeodesy::distanceM(nearestTowerPosition, pos);
+
+        // clear these so we don't do dynamic updates unless a carrier is active
+        towerViewPositionLatDegNode = towerViewPositionLonDegNode = towerViewPositionAltFtNode = nullptr;
+    }
+    else {
+        SG_LOG(SG_ENVIRONMENT, SG_INFO, "FGEnvironmentMgr::update: No airport within 100NM range");
+    }
+
+    // check for closer carrier
+    auto nctn = SGSharedPtr< NearestCarrierToNotification> (new NearestCarrierToNotification(pos));
+    if (simgear::Emesary::ReceiptStatus::OK == simgear::Emesary::GlobalTransmitter::instance()->NotifyAll(nctn)) {
+        if (nearestCarrier != nctn->GetCarrier()) {
+            nearestCarrier = nctn->GetCarrier();
+            fgSetString("/sim/airport/nearest-carrier", nctn->GetCarrierIdent());
+        }
+    } else {
+        fgSetString("/sim/airport/nearest-carrier", "");
+        fgSetDouble("/sim/airport/nearest-carrier-latitude-deg", 0);
+        fgSetDouble("/sim/airport/nearest-carrier-longitude-deg", 0);
+        fgSetDouble("/sim/airport/nearest-carrier-altitude-ft", 0);
+        fgSetDouble("/sim/airport/nearest-carrier-deck-height", 0);
+        nearestCarrier = nullptr;
+    }
+
+    // figure out if the carrier's tower is closer
+    if (nearestCarrier && (nctn->GetDistanceMeters() < towerDistance)) {
+        nearestIdent = nctn->GetCarrierIdent();
+
+        //
+        // these will be used to determine and update the tower position
+        towerViewPositionLatDegNode = nctn->GetViewPositionLatNode();
+        towerViewPositionLonDegNode = nctn->GetViewPositionLonNode();
+        towerViewPositionAltFtNode = nctn->GetViewPositionAltNode();
+
+        // although the carrier is moving - these values can afford to be 10 seconds old so we don't need to
+        // update them.
+        fgSetDouble("/sim/airport/nearest-carrier-latitude-deg", nctn->GetPosition()->getLatitudeDeg());
+        fgSetDouble("/sim/airport/nearest-carrier-longitude-deg", nctn->GetPosition()->getLongitudeDeg());
+        fgSetDouble("/sim/airport/nearest-carrier-altitude-ft", nctn->GetPosition()->getElevationFt());
+        fgSetDouble("/sim/airport/nearest-carrier-deck-height", nctn->GetDeckheight());
+    }
+
+    if (fgGetString("/sim/airport/nearest-tower-ident") != nearestIdent) {
+        SG_LOG(SG_ENVIRONMENT, SG_INFO, "Nearest airport tower now " << nearestIdent);
+        fgSetString("/sim/airport/nearest-tower-ident", nearestIdent);
+    }
+    if (automaticTowerActive) {
+        if (fgGetString("/sim/tower/airport-id") != nearestIdent) {
+            fgSetString("/sim/tower/airport-id", nearestIdent);
+            SG_LOG(SG_ENVIRONMENT, SG_INFO, "Auto Tower: now " << nearestIdent);
+        }
+
+        std::string path = ViewPropertyEvaluator::getStringValue("(/sim/view[(/sim/current-view/view-number-raw)]/config/root)/sim/tower/");
+        auto currentViewAirportIdNode = fgGetNode(path + "airport-id", true);
+        const auto curId = currentViewAirportIdNode->getStringValue();
+        if (curId != nearestIdent) {
+          currentViewAirportIdNode->setStringValue(nearestIdent);
+        }
+    }
+
+    updateDynamicTowerPosition();
+}
+
+
 FGEnvironment
 FGEnvironmentMgr::getEnvironment () const
 {
   return *_environment;
 }
 
-FGEnvironment
-FGEnvironmentMgr::getEnvironment (double lat, double lon, double alt) const
+const FGEnvironment* FGEnvironmentMgr::getAircraftEnvironment() const
 {
-  // Always returns the same environment
-  // for now; we'll make it interesting
-  // later.
-  FGEnvironment env = *_environment;
-  env.set_elevation_ft(alt);
-  return env;
+    return _environment;
 }
 
 FGEnvironment
-FGEnvironmentMgr::getEnvironment(const SGGeod& aPos) const
+FGEnvironmentMgr::getEnvironmentAtPosition(const SGGeod& aPos) const
 {
   // Always returns the same environment
   // for now; we'll make it interesting
