@@ -41,6 +41,8 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <memory>
+
 #include "models/FGPropagate.h"
 #include "models/FGOutput.h"
 #include "math/FGTemplateFunc.h"
@@ -69,6 +71,11 @@ class FGPropulsion;
 class FGMassBalance;
 class FGTrim;
 
+class TrimFailureException : public BaseException {
+  public:
+    TrimFailureException(const std::string& msg) : BaseException(msg) {}
+};
+
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -84,7 +91,7 @@ CLASS DOCUMENTATION
     executive is subsequently directed to load the chosen aircraft specification
     file:
 
-    @code
+    @code{.cpp}
     fdmex = new FGFDMExec( ... );
     result = fdmex->LoadModel( ... );
     @endcode
@@ -100,7 +107,7 @@ CLASS DOCUMENTATION
     from JSBSim. The state variables are used to drive the instrument displays
     and to place the vehicle model in world space for visual rendering:
 
-    @code
+    @code{.cpp}
     copy_to_JSBsim(); // copy control inputs to JSBSim
     fdmex->RunIC(); // loop JSBSim once w/o integrating
     copy_from_JSBsim(); // update the bus
@@ -108,7 +115,7 @@ CLASS DOCUMENTATION
 
     Once initialization is complete, cyclic execution proceeds:
 
-    @code
+    @code{.cpp}
     copy_to_JSBsim(); // copy control inputs to JSBSim
     fdmex->Run(); // execute JSBSim
     copy_from_JSBsim(); // update the bus
@@ -123,7 +130,7 @@ CLASS DOCUMENTATION
     file can be supplied to the stub program. Scripting (see FGScript) provides
     a way to supply command inputs to the simulation:
 
-    @code
+    @code{.cpp}
     FDMExec = new JSBSim::FGFDMExec();
     FDMExec->LoadScript( ScriptName ); // the script loads the aircraft and ICs
     result = FDMExec->Run();
@@ -174,7 +181,7 @@ CLASS DOCUMENTATION
 CLASS DECLARATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-class FGFDMExec : public FGJSBBase
+class JSBSIM_API FGFDMExec : public FGJSBBase
 {
   struct childData {
     FGFDMExec* exec;
@@ -191,7 +198,7 @@ class FGFDMExec : public FGJSBBase
       mated = true;
       internal = false;
     }
-    
+
     void Run(void) {exec->Run();}
     void AssignState(FGPropagate* source_prop) {
       exec->GetPropagate()->SetVState(source_prop->GetVState());
@@ -249,14 +256,13 @@ public:
       @return true if successful */
   bool RunIC(void);
 
-  /** Sets the ground callback pointer. For optimal memory management, a shared
-      pointer is used internally that maintains a reference counter. The calling
-      application must therefore use FGGroundCallback_ptr 'smart pointers' to
-      manage their copy of the ground callback.
-      @param gc A pointer to a ground callback object
-      @see FGGroundCallback
-   */
-  void SetGroundCallback(FGGroundCallback* gc) { FGLocation::SetGroundCallback(gc); }
+  /** Loads the planet.
+      Loads the definition of the planet on which the vehicle will evolve such as
+      its radius, gravity or its atmosphere characteristics.
+      @param PlanetPath The name of a planet definition file
+      @param useAircraftPath true if path is given relative to the aircraft path.
+      @return true if successful */
+  bool LoadPlanet(const SGPath& PlanetPath, bool useAircraftPath = true);
 
   /** Loads an aircraft model.
       @param AircraftPath path to the aircraft/ directory. For instance:
@@ -289,44 +295,65 @@ public:
       @return true if successful*/
   bool LoadModel(const std::string& model, bool addModelToPath = true);
 
-  /** Loads a script
+  /** Load a script
       @param Script The full path name and file name for the script to be loaded.
-      @param deltaT The simulation integration step size, if given.  If no value is supplied
-                    then 0.0 is used and the value is expected to be supplied in
-                    the script file itself.
-      @param initfile The initialization file that will override the initialization file
-                      specified in the script file. If no file name is given on the command line,
-                      the file specified in the script will be used. If an initialization file 
-                      is not given in either place, an error will result.
+      @param deltaT The simulation integration step size, if given.  If no value
+                    is supplied then 0.0 is used and the value is expected to be
+                    supplied in the script file itself.
+      @param initfile The initialization file that will override the
+                      initialization file specified in the script file. If no
+                      file name is given on the command line, the file specified
+                      in the script will be used. If an initialization file is
+                      not given in either place, an error will result.
       @return true if successfully loads; false otherwise. */
   bool LoadScript(const SGPath& Script, double deltaT=0.0,
                   const SGPath& initfile=SGPath());
 
-  /** Sets the path to the engine config file directories.
-      @param path path to the directory under which engine config
-      files are kept, for instance "engine"  */
+  /** Set the path to the engine config file directories.
+      Relative paths are taken from the root directory.
+      @param path path to the directory under which engine config files are
+                  kept, for instance "engine".
+      @see SetRootDir
+      @see GetEnginePath */
   bool SetEnginePath(const SGPath& path) {
     EnginePath = GetFullPath(path);
     return true;
   }
 
-  /** Sets the path to the aircraft config file directories.
-      @param path path to the aircraft directory. For instance:
-      "aircraft". Under aircraft, then, would be directories for various
-      modeled aircraft such as C172/, x15/, etc.  */
+  /** Set the path to the aircraft config file directories.
+      Under this path, then, would be directories for various modeled aircraft
+      such as C172/, x15/, etc.
+      Relative paths are taken from the root directory.
+      @param path path to the aircraft directory, for instance "aircraft".
+      @see SetRootDir
+      @see GetAircraftPath */
   bool SetAircraftPath(const SGPath& path) {
     AircraftPath = GetFullPath(path);
     return true;
   }
-  
-  /** Sets the path to the systems config file directories.
-      @param path path to the directory under which systems config
-      files are kept, for instance "systems"  */
+
+  /** Set the path to the systems config file directories.
+      Relative paths are taken from the root directory.
+      @param path path to the directory under which systems config files are
+                  kept, for instance "systems"
+      @see SetRootDir
+      @see GetSystemsPath */
   bool SetSystemsPath(const SGPath& path) {
     SystemsPath = GetFullPath(path);
     return true;
   }
-  
+
+  /** Set the directory where the output files will be written.
+      Relative paths are taken from the root directory.
+      @param path path to the directory under which the output files will be
+                  written.
+      @see SetRootDir
+      @see GetOutputPath */
+  bool SetOutputPath(const SGPath& path) {
+    OutputPath = GetFullPath(path);
+    return true;
+  }
+
   /// @name Top-level executive State and Model retrieval mechanism
   ///@{
   /// Returns the FGAtmosphere pointer.
@@ -361,12 +388,6 @@ public:
   FGInput* GetInput(void)              {return (FGInput*)Models[eInput];}
   /// Returns the FGOutput pointer.
   FGOutput* GetOutput(void)            {return (FGOutput*)Models[eOutput];}
-  /** Get a pointer to the ground callback currently used. It is recommanded
-      to store the returned pointer in a 'smart pointer' FGGroundCallback_ptr.
-      @return A pointer to the current ground callback object.
-      @see FGGroundCallback
-   */
-  FGGroundCallback* GetGroundCallback(void) {return FGLocation::GetGroundCallback();}
   /// Retrieves the script object
   FGScript* GetScript(void) {return Script;}
   /// Returns a pointer to the FGInitialCondition object
@@ -376,13 +397,15 @@ public:
   ///@}
 
   /// Retrieves the engine path.
-  const SGPath& GetEnginePath(void)    {return EnginePath;}
+  const SGPath& GetEnginePath(void) { return EnginePath; }
   /// Retrieves the aircraft path.
-  const SGPath& GetAircraftPath(void)  {return AircraftPath;}
+  const SGPath& GetAircraftPath(void) { return AircraftPath; }
   /// Retrieves the systems path.
-  const SGPath& GetSystemsPath(void)   {return SystemsPath;}
+  const SGPath& GetSystemsPath(void) { return SystemsPath; }
   /// Retrieves the full aircraft path name.
-  const SGPath& GetFullAircraftPath(void) {return FullAircraftPath;}
+  const SGPath& GetFullAircraftPath(void) { return FullAircraftPath; }
+  /// Retrieves the path to the output files.
+  const SGPath& GetOutputPath(void) { return OutputPath; }
 
   /** Retrieves the value of a property.
       @param property the name of the property
@@ -472,10 +495,15 @@ public:
   void Resume(void) {holding = false;}
   /// Returns true if the simulation is Holding (i.e. simulation time is not moving).
   bool Holding(void) {return holding;}
+  /// Mode flags for ResetToInitialConditions
+  static const int START_NEW_OUTPUT    = 0x1;
+  static const int DONT_EXECUTE_RUN_IC = 0x2;
   /** Resets the initial conditions object and prepares the simulation to run
-      again. If mode is set to 1 the output instances will take special actions
-      such as closing the current output file and open a new one with a
-      different name.
+      again. If the mode's first bit is set the output instances will take special actions
+      such as closing the current output file and open a new one with a different name.
+      If the second bit is set then RunIC() won't be executed, leaving it to the caller
+      to call RunIC(), e.g. in case the caller wants to set some other state like control
+      surface deflections which would've been reset.
       @param mode Sets the reset mode.*/
   void ResetToInitialConditions(int mode);
   /// Sets the debug level.
@@ -498,9 +526,10 @@ public:
   *   A string is returned that contains a carriage return delimited list of all
   *   strings in the property catalog that matches the supplied check string.
   *   @param check The string to search for in the property catalog.
+  *   @param end_of_line End of line (CR+LF if needed for Windows).
   *   @return the carriage-return-delimited string containing all matching strings
   *               in the catalog.  */
-  std::string QueryPropertyCatalog(const std::string& check);
+  std::string QueryPropertyCatalog(const std::string& check, const std::string& end_of_line="\n");
 
   // Print the contents of the property catalog for the loaded aircraft.
   void PrintPropertyCatalog(void);
@@ -536,35 +565,35 @@ public:
   /** Sets the current sim time.
       @param cur_time the current time
       @return the current simulation time.      */
-  double Setsim_time(double cur_time) {
-    sim_time = cur_time;
-    GetGroundCallback()->SetTime(sim_time);
-    return sim_time;
-  }
+  double Setsim_time(double cur_time);
 
   /** Sets the integration time step for the simulation executive.
       @param delta_t the time step in seconds.     */
   void Setdt(double delta_t) { dT = delta_t; }
 
-  /** Sets the root directory where JSBSim starts looking for its system directories.
-      @param rootDir the string containing the root directory. */
+  /** Set the root directory that is used to obtain absolute paths from
+      relative paths.
+      Aircraft, engine, systems and output paths are not updated by this
+      method. You must call each methods (SetAircraftPath(), SetEnginePath(),
+      etc.) individually if you need to update these paths as well.
+      @param rootDir the path to the root directory.
+      @see GetRootDir
+      @see SetAircraftPath
+      @see SetEnginePath
+      @see SetSystemsPath
+      @see SetOutputPath
+       */
   void SetRootDir(const SGPath& rootDir) {RootDir = rootDir;}
 
-  /** Retrieves the Root Directory.
-      @return the string representing the root (base) JSBSim directory. */
+  /** Retrieve the Root Directory.
+      @return the path to the root (base) JSBSim directory.
+      @see SetRootDir */
   const SGPath& GetRootDir(void) const {return RootDir;}
 
   /** Increments the simulation time if not in Holding mode. The Frame counter
       is also incremented.
       @return the new simulation time.     */
-  double IncrTime(void) {
-    if (!holding && !IntegrationSuspended()) {
-      sim_time += dT;
-      GetGroundCallback()->SetTime(sim_time);
-      Frame++;
-    }
-    return sim_time;
-  }
+  double IncrTime(void);
 
   /** Retrieves the current frame count. */
   unsigned int GetFrame(void) const {return Frame;}
@@ -595,8 +624,10 @@ public:
     TemplateFunctions[name] = new FGTemplateFunc(this, el);
   }
 
+  const std::shared_ptr<RandomNumberGenerator>& GetRandomGenerator(void) const
+  { return RandomGenerator; }
+
 private:
-  int Error;
   unsigned int Frame;
   unsigned int IdFDM;
   int disperse;
@@ -607,7 +638,6 @@ private:
   bool holding;
   bool IncrementThenHolding;
   int TimeStepsUntilHold;
-  int RandomSeed;
   bool Constructing;
   bool modelLoaded;
   bool IsChild;
@@ -616,6 +646,7 @@ private:
   SGPath FullAircraftPath;
   SGPath EnginePath;
   SGPath SystemsPath;
+  SGPath OutputPath;
   std::string CFGVersion;
   std::string Release;
   SGPath RootDir;
@@ -652,6 +683,9 @@ private:
 
   bool HoldDown;
 
+  unsigned int RandomSeed;
+  std::shared_ptr<RandomNumberGenerator> RandomGenerator;
+
   // The FDM counter is used to give each child FDM an unique ID. The root FDM
   // has the ID 0
   unsigned int*      FDMctr;
@@ -668,9 +702,11 @@ private:
   int  SRand(void) const {return RandomSeed;}
   void LoadInputs(unsigned int idx);
   void LoadPlanetConstants(void);
+  bool LoadPlanet(Element* el);
   void LoadModelConstants(void);
   bool Allocate(void);
   bool DeAllocate(void);
+  void InitializeModels(void);
   int GetDisperse(void) const {return disperse;}
   SGPath GetFullPath(const SGPath& name) {
     if (name.isRelative())
