@@ -230,7 +230,7 @@ bool FGOutputTextFile::OpenFile(void)
       outstream << delimeter << OutputParameters[i]->GetFullyQualifiedName();
   }
 
-  if (PreFunctions.size() > 0) {
+  if (!PreFunctions.empty()) {
     for (unsigned int i=0;i<PreFunctions.size();i++) {
       outstream << delimeter << PreFunctions[i]->GetName();
     }
@@ -319,6 +319,7 @@ void FGOutputTextFile::Print(void)
     outstream << Accelerations->GetMoments().Dump(delimeter);
   }
   if (SubSystems & ssAtmosphere) {
+    const auto Atmosphere = FDMExec->GetAtmosphere();
     outstream << delimeter;
     outstream << Atmosphere->GetDensity() << delimeter;
     outstream << Atmosphere->GetAbsoluteViscosity() << delimeter;
