@@ -34,8 +34,6 @@
   INCLUDES
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <stdexcept>
-
 #include "math/FGRealValue.h"
 #include "math/FGPropertyValue.h"
 #include "input_output/FGXMLElement.h"
@@ -60,18 +58,18 @@ class FGPropertyManager;
   DECLARATION: FGParameterValue
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-class FGParameterValue : public FGParameter
+class JSBSIM_API FGParameterValue : public FGParameter
 {
 public:
   FGParameterValue(Element* el, FGPropertyManager* pm) {
-    string value = el->GetDataLine();
+    std::string value = el->GetDataLine();
 
     if (el->GetNumDataLines() != 1 || value.empty()) {
-      cerr << el->ReadFrom()
-           << "The element <" << el->GetName()
-           << "> must either contain a value number or a property name."
-           << endl;
-      throw invalid_argument("Illegal argument");
+      std::cerr << el->ReadFrom()
+                << "The element <" << el->GetName()
+                << "> must either contain a value number or a property name."
+                << std::endl;
+      throw BaseException("FGParameterValue: Illegal argument defining: " + el->GetName());
     }
 
     Construct(value, pm);
@@ -89,7 +87,7 @@ public:
     if (v)
       return v->GetNameWithSign();
     else
-      return to_string(param->GetValue());
+      return param->GetName();
   }
 
   bool IsLateBound(void) const {
