@@ -29,8 +29,11 @@ Purpose: Stores property values
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <assert.h>
+
 #include "FGPropertyValue.h"
-#include "input_output/FGPropertyManager.h"
+
+using namespace std;
 
 namespace JSBSim {
 
@@ -58,10 +61,10 @@ FGPropertyNode* FGPropertyValue::GetNode(void) const
 {
   if (!PropertyNode) {
     FGPropertyNode* node = PropertyManager->GetNode(PropertyName);
-    
+
     if (!node)
-      throw(std::string("FGPropertyValue::GetValue() The property " +
-                        PropertyName + " does not exist."));
+      throw BaseException("FGPropertyValue::GetValue() The property " +
+                          PropertyName + " does not exist.");
 
     PropertyNode = node;
   }
@@ -80,6 +83,9 @@ double FGPropertyValue::GetValue(void) const
 
 void FGPropertyValue::SetValue(double value)
 {
+  // SetValue() ignores the Sign flag. So make sure it is never called with a
+  // negative sign.
+  assert(Sign == 1);
   GetNode()->setDoubleValue(value);
 }
 
