@@ -45,6 +45,7 @@ INCLUDES
 #include "models/FGGroundReactions.h"
 #include "math/FGTable.h"
 #include "input_output/FGXMLElement.h"
+#include "models/FGInertial.h"
 
 using namespace std;
 
@@ -68,7 +69,6 @@ CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 FGLGear::FGLGear(Element* el, FGFDMExec* fdmex, int number, const struct Inputs& inputs) :
-  FGSurface(fdmex, number),
   FGForce(fdmex),
   in(inputs),
   GearNumber(number),
@@ -179,7 +179,12 @@ FGLGear::FGLGear(Element* el, FGFDMExec* fdmex, int number, const struct Inputs&
 
   Element* element = el->FindElement("location");
   if (element) vXYZn = element->FindElementTripletConvertTo("IN");
-  else {cerr << "No location given for contact " << name << endl; exit(-1);}
+  else {
+    stringstream s;
+    s << "No location given for contact " << name;
+    cerr << endl << s.str() << endl;
+    throw BaseException(s.str());
+  }
   SetTransformType(FGForce::tCustom);
 
   element = el->FindElement("orientation");
@@ -270,7 +275,7 @@ void FGLGear::ResetToIC(void)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
+const FGColumnVector3& FGLGear::GetBodyForces(void)
 {
   double gearPos = 1.0;
 
@@ -288,17 +293,17 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
 
     // Compute the height of the theoretical location of the wheel (if strut is
     // not compressed) with respect to the ground level
-    double height = gearLoc.GetContactPoint(contact, normal, terrainVel, dummy);
+    double height = fdmex->GetInertial()->GetContactPoint(gearLoc, contact,
+                                                          normal, terrainVel,
+                                                          dummy);
 
-    // Does this surface contact point interact with another surface?
-    if (surface) {
-      if (!fdmex->GetTrimStatus())
-        height -= (*surface).GetBumpHeight();
-      staticFFactor = (*surface).GetStaticFFactor();
-      rollingFFactor = (*surface).GetRollingFFactor();
-      maximumForce = (*surface).GetMaximumForce();
-      isSolid =  (*surface).GetSolid();
-    }
+    if (!fdmex->GetTrimStatus())
+      height -= GroundReactions->GetBumpHeight();
+    staticFFactor = GroundReactions->GetStaticFFactor();
+    rollingFFactor = GroundReactions->GetRollingFFactor();
+    maximumForce = GroundReactions->GetMaximumForce();
+    bumpiness = GroundReactions->GetBumpiness();
+    isSolid = GroundReactions->GetSolid();
 
     FGColumnVector3 vWhlDisplVec;
     double LGearProj = 1.0;
@@ -532,12 +537,11 @@ void FGLGear::ReportTakeoffOrLanding(void)
     if (debug_lvl > 0) Report(erTakeoff);
   }
 
-  if (lastWOW != WOW)
-  {
-    ostringstream buf;
-    buf << "GEAR_CONTACT: " << fdmex->GetSimTime() << " seconds: " << name;
-    PutMessage(buf.str(), WOW);
-  }
+  // if (lastWOW != WOW)
+  // {
+  //   cout << "GEAR_CONTACT: " << fdmex->GetSimTime() << " seconds: " << name
+  //        << " " << WOW << endl;
+  // }
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -550,9 +554,7 @@ void FGLGear::CrashDetect(void)
       GetMoments().Magnitude() > 5000000000.0 ||
       SinkRate > 1.4666*30 ) && !fdmex->IntegrationSuspended())
   {
-    ostringstream buf;
-    buf << "*CRASH DETECTED* " << fdmex->GetSimTime() << " seconds: " << name;
-    PutMessage(buf.str());
+    cout << "*CRASH DETECTED* " << fdmex->GetSimTime() << " seconds: " << name;
     // fdmex->SuspendIntegration();
   }
 }
@@ -603,13 +605,11 @@ void FGLGear::ComputeSideForceCoefficient(void)
 
 void FGLGear::ComputeVerticalStrutForce()
 {
-  double springForce = 0;
-  double dampForce = 0;
-
   if (fStrutForce)
     StrutForce = min(fStrutForce->GetValue(), (double)0.0);
   else {
-    springForce = -compressLength * kSpring;
+    double springForce = -compressLength * kSpring;
+    double dampForce = 0;
 
     if (compressSpeed >= 0.0) {
 
@@ -768,17 +768,14 @@ void FGLGear::bind(void)
 
   switch(eContactType) {
   case ctBOGEY:
-    eSurfaceType = FGSurface::ctBOGEY;
     base_property_name = CreateIndexedPropertyName("gear/unit", GearNumber);
     break;
   case ctSTRUCTURE:
-    eSurfaceType = FGSurface::ctSTRUCTURE;
     base_property_name = CreateIndexedPropertyName("contact/unit", GearNumber);
     break;
   default:
     return;
   }
-  FGSurface::bind();
 
   property_name = base_property_name + "/WOW";
   PropertyManager->Tie( property_name.c_str(), &WOW );
@@ -832,6 +829,17 @@ void FGLGear::bind(void)
     string tmp = CreateIndexedPropertyName("fcs/steer-pos-deg", GearNumber);
     PropertyManager->Tie(tmp.c_str(), this, &FGLGear::GetSteerAngleDeg, &FGLGear::SetSteerAngleDeg);
   }
+
+  property_name = base_property_name + "/solid";
+  PropertyManager->Tie( property_name.c_str(), &isSolid);
+  property_name = base_property_name + "/bumpiness";
+  PropertyManager->Tie( property_name.c_str(), &bumpiness);
+  property_name = base_property_name + "/maximum-force-lbs";
+  PropertyManager->Tie( property_name.c_str(), &maximumForce);
+  property_name = base_property_name + "/rolling_friction-factor";
+  PropertyManager->Tie( property_name.c_str(), &rollingFFactor);
+  property_name = base_property_name + "/static-friction-factor";
+  PropertyManager->Tie( property_name.c_str(), &staticFFactor);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
