@@ -9,21 +9,21 @@
  ------------- Copyright (C) 2000  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -44,10 +44,8 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
-#include <sstream>
-#include <cstdlib>
 #include <iomanip>
+#include <array>
 
 #include "FGFDMExec.h"
 #include "FGPropulsion.h"
@@ -59,7 +57,6 @@ INCLUDES
 #include "models/propulsion/FGTurboProp.h"
 #include "models/propulsion/FGTank.h"
 #include "input_output/FGModelLoader.h"
-#include "math/FGColumnVector3.h"
 
 using namespace std;
 
@@ -208,7 +205,7 @@ void FGPropulsion::ConsumeFuel(FGEngine* engine)
       if (TankPriority != 0) {
         switch(Tank->GetType()) {
         case FGTank::ttFUEL:
-          if ((Tank->GetContents() > 0.0) && Tank->GetSelected() && (TankPriority == CurrentFuelTankPriority)) {
+          if ((Tank->GetContents() > Tank->GetUnusable()) && Tank->GetSelected() && (TankPriority == CurrentFuelTankPriority)) {
             TanksWithFuel++;
             Starved = false;
             FeedListFuel.push_back(TankId);
@@ -240,7 +237,7 @@ void FGPropulsion::ConsumeFuel(FGEngine* engine)
             break;
           case FGTank::ttOXIDIZER:
             hasOxTanks = true;
-            if (Tank->GetContents() > 0.0 && Tank->GetSelected() && TankPriority == CurrentOxidizerTankPriority) {
+            if (Tank->GetContents() > Tank->GetUnusable() && Tank->GetSelected() && TankPriority == CurrentOxidizerTankPriority) {
               TanksWithOxidizer++;
               if (TanksWithFuel > 0) Starved = false;
               FeedListOxi.push_back(TankId);
@@ -369,7 +366,7 @@ bool FGPropulsion::Load(Element* el)
   Name = "Propulsion Model: " + el->GetAttributeValue("name");
 
   // Perform base class Pre-Load
-  if (!FGModel::Load(el, true))
+  if (!FGModel::Upload(el, true))
     return false;
 
   // Process tank definitions first to establish the number of fuel tanks
@@ -452,7 +449,7 @@ bool FGPropulsion::Load(Element* el)
   }
 
 
-  PostLoad(el, PropertyManager);
+  PostLoad(el, FDMExec);
 
   return true;
 }
@@ -461,11 +458,23 @@ bool FGPropulsion::Load(Element* el)
 
 SGPath FGPropulsion::FindFullPathName(const SGPath& path) const
 {
-  if (!ReadingEngine) return FGModel::FindFullPathName(path);
+  SGPath name = FGModel::FindFullPathName(path);
+  if (!ReadingEngine && !name.isNull()) return name;
 
-  SGPath name = CheckPathName(FDMExec->GetFullAircraftPath()/string("Engines"),
-                              path);
-  if (!name.isNull()) return name;
+#ifdef _WIN32
+  // Singular and plural are allowed for the folder names for consistency with
+  // the default engine folder name "engine" and for backward compatibility
+  // regarding the folder name "Engines".
+  const array<string, 2> dir_names = {"Engines", "engine"};
+#else
+  // Allow alternative capitalization for case sensitive OSes.
+  const array<string, 4> dir_names = {"Engines", "engines", "Engine", "engine"};
+#endif
+
+  for(const string& dir_name: dir_names) {
+    name = CheckPathName(FDMExec->GetFullAircraftPath()/dir_name, path);
+    if (!name.isNull()) return name;
+  }
 
   return CheckPathName(FDMExec->GetEnginePath(), path);
 }
@@ -585,7 +594,7 @@ const FGMatrix33& FGPropulsion::CalculateTankInertias(void)
 
   if (size == 0) return tankJ;
 
-  tankJ = FGMatrix33();
+  tankJ.InitMatrix();
 
   for (unsigned int i=0; i<size; i++) {
     tankJ += FDMExec->GetMassBalance()->GetPointmassInertia( lbtoslug * Tanks[i]->GetContents(),
@@ -709,7 +718,6 @@ int FGPropulsion::GetCutoff(void) const
       return ((FGTurbine*)Engines[ActiveEngine])->GetCutoff() ? 1 : 0;
     case FGEngine::etTurboprop:
       return ((FGTurboProp*)Engines[ActiveEngine])->GetCutoff() ? 1 : 0;
-      break;
     default:
       break;
     }
@@ -807,9 +815,10 @@ void FGPropulsion::bind(void)
 {
   typedef double (FGPropulsion::*PMF)(int) const;
   typedef int (FGPropulsion::*iPMF)(void) const;
+  typedef bool (FGPropulsion::*bPMF)(void) const;
 
   IsBound = true;
-  PropertyManager->Tie("propulsion/set-running", this, (iPMF)0, &FGPropulsion::InitRunning, false);
+  PropertyManager->Tie("propulsion/set-running", this, (iPMF)0, &FGPropulsion::InitRunning);
   if (HaveTurbineEngine || HaveTurboPropEngine) {
     PropertyManager->Tie("propulsion/starter_cmd", this, &FGPropulsion::GetStarter, &FGPropulsion::SetStarter);
     PropertyManager->Tie("propulsion/cutoff_cmd", this,  &FGPropulsion::GetCutoff, &FGPropulsion::SetCutoff);
@@ -817,11 +826,11 @@ void FGPropulsion::bind(void)
 
   if (HavePistonEngine) {
     PropertyManager->Tie("propulsion/starter_cmd", this, &FGPropulsion::GetStarter, &FGPropulsion::SetStarter);
-    PropertyManager->Tie("propulsion/magneto_cmd", this, (iPMF)0, &FGPropulsion::SetMagnetos, false);
+    PropertyManager->Tie("propulsion/magneto_cmd", this, (iPMF)0, &FGPropulsion::SetMagnetos);
   }
 
   PropertyManager->Tie("propulsion/active_engine", this, (iPMF)&FGPropulsion::GetActiveEngine,
-                        &FGPropulsion::SetActiveEngine, true);
+                        &FGPropulsion::SetActiveEngine);
   PropertyManager->Tie("forces/fbx-prop-lbs", this, eX, (PMF)&FGPropulsion::GetForces);
   PropertyManager->Tie("forces/fby-prop-lbs", this, eY, (PMF)&FGPropulsion::GetForces);
   PropertyManager->Tie("forces/fbz-prop-lbs", this, eZ, (PMF)&FGPropulsion::GetForces);
@@ -832,6 +841,7 @@ void FGPropulsion::bind(void)
   TotalOxidizerQuantity = PropertyManager->CreatePropertyObject<double>("propulsion/total-oxidizer-lbs");
   refuel = PropertyManager->CreatePropertyObject<bool>("propulsion/refuel");
   dump = PropertyManager->CreatePropertyObject<bool>("propulsion/fuel_dump");
+  PropertyManager->Tie("propulsion/fuel_freeze", this, (bPMF)nullptr, &FGPropulsion::SetFuelFreeze);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
