@@ -67,6 +67,7 @@ FORWARD DECLARATIONS
 namespace JSBSim {
 
 class FGFDMExec;
+class FGMassBalance;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
@@ -217,17 +218,16 @@ and vMn, the moments, can be made directly. Otherwise, the usage is similar.<br>
 CLASS DECLARATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-class FGForce : public FGJSBBase
+class JSBSIM_API FGForce : public FGJSBBase
 {
 public:
   /// Constructor
   explicit FGForce(FGFDMExec *FDMExec);
-  FGForce(const FGForce& force);
 
   /// Destructor
   virtual ~FGForce();
 
-  enum TransformType { tNone, tWindBody, tLocalBody, tCustom };
+  enum TransformType { tNone, tWindBody, tLocalBody, tInertialBody, tCustom };
 
   virtual const FGColumnVector3& GetBodyForces(void);
 
@@ -305,6 +305,7 @@ public:
 
 protected:
   FGFDMExec *fdmex;
+  FGMassBalance* MassBalance;
   FGColumnVector3 vFn;
   FGColumnVector3 vMn;
   FGColumnVector3 vOrient;
