@@ -1,5 +1,4 @@
-#ifndef _1D_DATA_FILE_READER_H_
-#define _1D_DATA_FILE_READER_H_
+#pragma once
 
 #include <simgear/compiler.h>
 
@@ -10,15 +9,12 @@
 #include "uiuc_aircraft.h"
 #include "uiuc_warnings_errors.h"
 
-//using std::istrstream;
 
-int uiuc_1DdataFileReader( string file_name, 
+int uiuc_1DdataFileReader( std::string file_name, 
                             double x[], 
                             double y[], 
                             int &xmax );
-int uiuc_1DdataFileReader( string file_name, 
+int uiuc_1DdataFileReader( std::string file_name, 
                            double x[], 
                            int y[], 
                            int &xmax );
-
-#endif // _1D_DATA_FILE_READER_H_
