@@ -18,14 +18,14 @@
 
 #include "Add-ons/Addon.hxx"
 #include "Add-ons/addon_fwd.hxx"
-#include "LocalAircraftCache.hxx"
-#include "LauncherMainWindow.hxx"
-#include "CatalogListModel.hxx"
 #include "AddonsModel.hxx"
-#include "InstallSceneryDialog.hxx"
-#include "QtLauncher.hxx"
-#include "PathListModel.hxx"
+#include "CatalogListModel.hxx"
 #include "LaunchConfig.hxx"
+#include "LauncherMainWindow.hxx"
+#include "LocalAircraftCache.hxx"
+#include "PathListModel.hxx"
+#include "QtLauncher.hxx"
+#include "SettingsWrapper.hxx"
 
 //////////////////////////////////////////////////////////////////////////
 
@@ -63,7 +63,7 @@ AddOnsController::AddOnsController(LauncherMainWindow *parent, LaunchConfig* con
         setLocalAircraftPaths();
     });
 
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     int size = settings.beginReadArray("addon-modules");
     for (int i = 0; i < size; ++i) {
         settings.setArrayIndex(i);
@@ -83,7 +83,7 @@ AddOnsController::AddOnsController(LauncherMainWindow *parent, LaunchConfig* con
             m_addonsModuleModel->append(path, addon, enable);
         }
         catch (const sg_exception &e) {
-            string msg = "Error getting add-on metadata: " + e.getFormattedMessage();
+            std::string msg = "Error getting add-on metadata: " + e.getFormattedMessage();
             SG_LOG(SG_GENERAL, SG_ALERT, msg);
         }
     }
@@ -218,7 +218,7 @@ QString AddOnsController::addAddOnModulePath() const
             path = QString("");
         }
     } catch (const sg_exception &e) {
-        string msg = "Error getting add-on metadata: " + e.getFormattedMessage();
+        std::string msg = "Error getting add-on metadata: " + e.getFormattedMessage();
         SG_LOG(SG_GENERAL, SG_ALERT, msg);
     }
 
@@ -237,7 +237,7 @@ QString AddOnsController::addSceneryPath() const
     SGPath p(path.toStdString());
     bool isValid = false;
 
-    for (const auto& dir: {"Objects", "Terrain", "Buildings", "Roads", "Pylons", "NavData", "Airports"}) {
+    for (const auto& dir: {"Objects", "Terrain", "Buildings", "Roads", "Pylons", "NavData", "Airports", "Orthophotos", "vpb"}) {
         if ((p / dir).exists()) {
             isValid = true;
             break;
@@ -250,7 +250,7 @@ QString AddOnsController::addSceneryPath() const
         mb.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
         mb.setDefaultButton(QMessageBox::No);
         mb.setInformativeText(tr("Added scenery should contain at least one of the following "
-                                 "folders: Objects, Terrain, Buildings, Roads, Pylons, NavData, Airports."));
+                                 "folders: Objects, Terrain, Buildings, Roads, Pylons, NavData, Airports, Orthophotos, vpb."));
         mb.exec();
 
         if (mb.result() == QMessageBox::No) {
@@ -262,18 +262,6 @@ QString AddOnsController::addSceneryPath() const
     return path;
 }
 
-QString AddOnsController::installCustomScenery()
-{
-    QSettings settings;
-    QString downloadDir = settings.value("download-dir").toString();
-    InstallSceneryDialog dlg(nullptr, downloadDir);
-    if (dlg.exec() == QDialog::Accepted) {
-        return dlg.sceneryPath();
-    }
-
-    return {};
-}
-
 void AddOnsController::openDirectory(QString path)
 {
     QUrl u = QUrl::fromLocalFile(path);
@@ -295,7 +283,7 @@ void AddOnsController::setModulePaths(QStringList modulePaths)
 
     m_addonsModuleModel->resetData(modulePaths);
 
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     int i = 0;
     settings.beginWriteArray("addon-modules");
     for (const QString& path : modulePaths) {
@@ -314,7 +302,7 @@ void AddOnsController::setModulePaths(QStringList modulePaths)
 void AddOnsController::officialCatalogAction(QString s)
 {
     if (s == "hide") {
-        QSettings settings;
+        auto settings = flightgear::getQSettings();
         settings.setValue("hide-official-catalog-message", true);
     } else if (s == "add-official") {
         flightgear::addSentryBreadcrumb("user requested to add the default catalog", "info");
@@ -326,7 +314,7 @@ void AddOnsController::officialCatalogAction(QString s)
 
 bool AddOnsController::shouldShowOfficialCatalogMessage() const
 {
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     bool showOfficialCatalogMesssage = !globals->get_subsystem<FGHTTPClient>()->isDefaultCatalogInstalled();
     if (settings.value("hide-official-catalog-message").toBool()) {
         showOfficialCatalogMesssage = false;
@@ -354,7 +342,7 @@ void AddOnsController::onCatalogsChanged()
 
 void AddOnsController::onAddonsChanged()
 {
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
 
     int i = 0;
     settings.beginWriteArray("addon-modules");
@@ -386,7 +374,7 @@ void AddOnsController::collectArgs()
 
     // add-on module paths
     // we could query this directly from AddonsModel, but this is simpler right now
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     int size = settings.beginReadArray("addon-modules");
     for (int i = 0; i < size; ++i) {
         settings.setArrayIndex(i);
