@@ -22,11 +22,12 @@
 
 #include <limits>
 
-#include <QPainter>
 #include <QDebug>
-#include <QVector2D>
 #include <QMouseEvent>
 #include <QPaintDevice>
+#include <QPainter>
+#include <QQmlEngine>
+#include <QVector2D>
 
 #include <Navaids/navrecord.hxx>
 #include <Navaids/positioned.hxx>
@@ -35,6 +36,7 @@
 #include <Navaids/NavDataCache.hxx>
 #include <Navaids/airways.hxx>
 
+#include "QmlColoredImageProvider.hxx"
 #include "QtLauncher_fwd.hxx"
 
 using namespace flightgear;
@@ -157,8 +159,13 @@ void BaseDiagram::paintAirplaneIcon(QPainter* painter, const SGGeod& geod, int h
 
 void BaseDiagram::paintCarrierIcon(QPainter* painter, const SGGeod& geod, int headingDeg)
 {
+    if (m_carrierPixmap.isNull()) {
+        auto iconProvider = QmlColoredImageProvider::instance();
+        QSize sz;
+        m_carrierPixmap = iconProvider->requestPixmap("aircraft-carrier?#ff00ff", &sz, {});
+    }
+
     QPointF pos = project(geod);
-    QPixmap pix(":/svg/aircraft-carrier");
     pos = m_viewportTransform.map(pos);
     painter->save();
     painter->setWorldTransform(m_baseDeviceTransform);
@@ -167,9 +174,9 @@ void BaseDiagram::paintCarrierIcon(QPainter* painter, const SGGeod& geod, int he
     painter->rotate(headingDeg);
 
     painter->setRenderHint(QPainter::SmoothPixmapTransform, true);
-    QRect carrierIconRect = pix.rect();
+    QRect carrierIconRect = m_carrierPixmap.rect();
     carrierIconRect.moveCenter(QPoint(0,0));
-    painter->drawPixmap(carrierIconRect, pix);
+    painter->drawPixmap(carrierIconRect, m_carrierPixmap);
 
     painter->restore();
 }
@@ -253,6 +260,11 @@ public:
         addType(FGPositioned::NDB);
         addType(FGPositioned::VOR);
 
+        // this doesn't work, since the cache does not actually contain TACAN entries
+        // addType(FGPositioned::TACAN);
+        // instead we look for DMEs and filter on the name
+        addType(FGPositioned::DME);
+
         if (aircraft == LauncherController::Helicopter) {
             addType(FGPositioned::HELIPAD);
         }
@@ -266,16 +278,24 @@ public:
         addType(FGPositioned::AIRPORT);
     }
 
-    virtual bool pass(FGPositioned* aPos) const
+    bool pass(FGPositioned* aPos) const override
     {
         bool ok = TypeFilter::pass(aPos);
 
-        if (ok && (aPos->type() == FGPositioned::FIX)) {
+        const auto ty = aPos->type();
+        if (ok && (ty == FGPositioned::FIX)) {
             // ignore fixes which end in digits
             if (aPos->ident().length() > 4 && isdigit(aPos->ident()[3]) && isdigit(aPos->ident()[4])) {
                 return false;
             }
         }
+
+        if (ok && (ty == FGPositioned::DME)) {
+            if (!simgear::strutils::ends_with(aPos->name(), "TACAN")) {
+                return false;
+            }
+        }
+
         return ok;
     }
 };
@@ -808,6 +828,11 @@ QPixmap BaseDiagram::iconForPositioned(const FGPositionedRef& pos,
 
         return QPixmap(":/vor-icon");
 
+    // our filter only passes DMEs which are TACANs, so this is correct,
+    // until we actually record TACANs in the NavCache
+    case FGPositioned::DME:
+        return QPixmap(":/tacan-icon");
+
     case FGPositioned::AIRPORT:
         return iconForAirport(static_cast<FGAirport*>(pos.ptr()), options);
 
