@@ -1,24 +1,17 @@
 // CocoaFileDialog.mm - Cocoa implementation of file-dialog interface
-
-// Copyright (C) 2013 James Turner <zakalawe@mac.com>
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-
+// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-FileCopyrightText: Copyright (C) 2013  James Turner - james@flightgear.org
 
 #include "CocoaFileDialog.hxx"
+#include "GUI/FileDialog.hxx"
+#include "simgear/debug/debug_types.h"
+
+// Apple
+#include <UniformTypeIdentifiers/UniformTypeIdentifiers.h>
+
+#include "GUI/FileDialog.hxx"
+#include "simgear/debug/debug_types.h"
+#include "simgear/math/sg_types.hxx"
 
 #include <AppKit/NSSavePanel.h>
 #include <AppKit/NSOpenPanel.h>
@@ -34,26 +27,32 @@
 #include <Main/fg_props.hxx>
 #include <Viewer/renderer.hxx>
 
-// 10.6 compiler won't accept block-scoped locals in Objective-C++,
-// so making these globals.
-static NSString* completion_path = nil;
-static SGPath completion_sgpath;
-
 class CocoaFileDialog::CocoaFileDialogPrivate
 {
 public:
-    CocoaFileDialogPrivate() :
-        panel(nil)
-    {
+  CocoaFileDialogPrivate() = default;
 
+  ~CocoaFileDialogPrivate() { [panel release]; }
+
+  NSSavePanel *panel = nil;
+
+  NSArray<UTType *> *buildContentTypes(const string_list &patterns) {
+    // important that we return an empty array (and not `nil`) if
+    // patterns is empty, since that corresponds to 'all types allowed'
+    auto *types = [[NSMutableArray<UTType *> new] autorelease];
+    for (auto p : patterns) {
+      if (!simgear::strutils::starts_with(p, "*.")) {
+        SG_LOG(SG_GUI, SG_ALERT, "can't use pattern on Cocoa:" << p);
+        continue;
+      }
+
+      auto cocoaFileExtension = stdStringToCocoa(p.substr(2));
+      auto *utType = [UTType typeWithFilenameExtension:cocoaFileExtension];
+      [types addObject:utType];
     }
 
-    ~CocoaFileDialogPrivate()
-    {
-        [panel release];
-    }
-
-    NSSavePanel* panel;
+    return types;
+  }
 };
 
 CocoaFileDialog::CocoaFileDialog(FGFileDialog::Usage use) :
@@ -85,7 +84,7 @@ void CocoaFileDialog::exec()
 // it window-modal.
     NSWindow* cocoaWindow = nil;
     std::vector<osgViewer::GraphicsWindow*> windows;
-    globals->get_renderer()->getViewer()->getWindows(windows);
+    globals->get_renderer()->getViewerBase()->getWindows(windows);
 
     for (auto gw : windows) {
         // OSG doesn't use RTTI, so no dynamic cast. Let's check the class type
@@ -105,37 +104,24 @@ void CocoaFileDialog::exec()
         [d->panel setNameFieldStringValue:stdStringToCocoa(_placeholder)];
     }
 
-    if (_filterPatterns.empty()) {
-        [d->panel setAllowedFileTypes:nil];
-    } else {
-        NSMutableArray* extensions = [NSMutableArray arrayWithCapacity:0];
-        for (const auto& ext : _filterPatterns) {
-            if (!simgear::strutils::starts_with(ext, "*.")) {
-                SG_LOG(SG_GENERAL, SG_INFO, "can't use pattern on Cococa:" << ext);
-                continue;
-            }
-            [extensions addObject:stdStringToCocoa(ext.substr(2))];
-        }
-
-        [d->panel setAllowedFileTypes:extensions];
-    }
-
+    // if _filterPatterns is empty, we want an empty content types array to
+    // indicate 'any file type' as per the NSSavePanel docs
+    [d->panel setAllowedContentTypes:d->buildContentTypes(_filterPatterns)];
     [d->panel setTitle:stdStringToCocoa(_title)];
+
     if (_showHidden) {
         [d->panel setShowsHiddenFiles:YES];
     }
 
     [d->panel setDirectoryURL: pathToNSURL(_initialPath)];
 
-    [d->panel beginSheetModalForWindow:cocoaWindow completionHandler:^(NSInteger result)
-    {
-        if (result == NSFileHandlingPanelOKButton) {
-            completion_path = [[d->panel URL] path];
-            //NSLog(@"the URL is: %@", d->panel URL]);
-            completion_sgpath = SGPath::fromUtf8([completion_path UTF8String]);
-            _callback->onFileDialogDone(this, completion_sgpath);
-        }
-    }];
+    NSInteger result = [d->panel runModal];
+    if (result == NSModalResponseOK) {
+        NSString *nspath = [[d->panel URL] path];
+        // NSLog(@"the URL is: %@", d->panel URL]);
+        auto p = SGPath::fromUtf8([nspath UTF8String]);
+        handleSelectedPath(p);
+    }
 }
 
 void CocoaFileDialog::close()
