@@ -31,7 +31,6 @@
 #include <QSettings>
 #include <QDebug>
 
-#include <Include/version.h>
 #include <Main/globals.hxx>
 #include <Main/locale.hxx>
 #include <Main/sentryIntegration.hxx>
@@ -40,6 +39,8 @@
 #include <simgear/props/props_io.hxx>
 #include <simgear/structure/exception.hxx>
 
+#include "SettingsWrapper.hxx"
+
 static quint32 CACHE_VERSION = 13;
 
 const std::vector<QByteArray> static_localizedStringTags = {"name", "desc"};
@@ -78,7 +79,7 @@ bool AircraftItem::initFromFile(QDir dir, QString filePath)
     LocalizedStrings ls;
     ls.locale = "en";
     ls.strings["name"] = QString::fromStdString(sim->getStringValue("description")).trimmed();
-    authors = sim->getStringValue("author");
+    authors = QString::fromStdString(sim->getStringValue("author"));
 
     if (sim->hasChild("rating")) {
         SGPropertyNode_ptr ratingsNode = sim->getNode("rating");
@@ -89,11 +90,11 @@ bool AircraftItem::initFromFile(QDir dir, QString filePath)
 
     if (sim->hasChild("long-description")) {
         // clean up any XML whitspace in the text.
-        ls.strings["desc"] = QString(sim->getStringValue("long-description")).simplified();
+        ls.strings["desc"] = QString::fromStdString(sim->getStringValue("long-description")).simplified();
     }
 
     if (sim->hasChild("variant-of")) {
-        variantOf = sim->getStringValue("variant-of");
+        variantOf = QString::fromStdString(sim->getStringValue("variant-of"));
     } else {
         isPrimary = true;
     }
@@ -107,16 +108,16 @@ bool AircraftItem::initFromFile(QDir dir, QString filePath)
         int nChildren = tagsNode->nChildren();
         for (int i = 0; i < nChildren; i++) {
             const SGPropertyNode* c = tagsNode->getChild(i);
-            if (strcmp(c->getName(), "tag") == 0) {
-                const char* tagName = c->getStringValue();
-                usesHeliports |= (strcmp(tagName, "helicopter") == 0);
+            if (c->getNameString() == "tag") {
+                std::string tagName = c->getStringValue();
+                usesHeliports |= (tagName == "helicopter");
                 // could also consider vtol tag?
-                usesSeaports |= (strcmp(tagName, "seaplane") == 0);
-                usesSeaports |= (strcmp(tagName, "floats") == 0);
-                needsMaintenance |= (strcmp(tagName, "needs-maintenance") == 0);
+                usesSeaports |= (tagName == "seaplane");
+                usesSeaports |= (tagName == "floats");
+                needsMaintenance |= (tagName == "needs-maintenance");
 
                 // and actually store the tags
-                tags.push_back(QString::fromUtf8(tagName));
+                tags.push_back(QString::fromStdString(tagName));
             }
         } // of tags iteration
     } // of set-xml has tags
@@ -149,13 +150,13 @@ bool AircraftItem::initFromFile(QDir dir, QString filePath)
     }
 
     if (sim->hasChild("thumbnail")) {
-        thumbnailPath = sim->getStringValue("thumbnail");
+        thumbnailPath = QString::fromStdString(sim->getStringValue("thumbnail"));
     } else {
         thumbnailPath = "thumbnail.jpg";
     }
 
     if (sim->hasChild("minimum-fg-version")) {
-        minFGVersion = sim->getStringValue("minimum-fg-version");
+        minFGVersion = QString::fromStdString(sim->getStringValue("minimum-fg-version"));
     }
 
     homepageUrl = QUrl(QString::fromStdString(sim->getStringValue("urls/home-page")));
@@ -227,7 +228,7 @@ QString AircraftItem::description() const
 QString AircraftItem::baseName() const
 {
     QString fn = QFileInfo(path).fileName();
-    fn.truncate(fn.count() - 8);
+    fn.truncate(fn.size() - 8);
     return fn;
 }
 
@@ -436,7 +437,7 @@ protected:
 private:
     void readCache()
     {
-        QSettings settings;
+        auto settings = flightgear::getQSettings();
         QByteArray cacheData = settings.value("aircraft-cache").toByteArray();
         if (!cacheData.isEmpty()) {
             QDataStream ds(cacheData);
@@ -463,7 +464,7 @@ private:
 
     void writeCache()
     {
-        QSettings settings;
+        auto settings = flightgear::getQSettings();
         QByteArray cacheData;
         {
             QDataStream ds(&cacheData, QIODevice::WriteOnly);
