@@ -21,6 +21,8 @@
 #include <simgear/compiler.h>
 #include <atomic>
 
+#include <QOpenGLContext>
+
 #include "QQuickDrawable.hxx"
 
 #include <QQmlComponent>
@@ -30,7 +32,6 @@
 
 #include <QTimer>
 #include <QCoreApplication>
-#include <QOpenGLContext>
 #include <QOpenGLFunctions>
 #include <QQuickItem>
 #include <QQuickWindow>
@@ -60,7 +61,7 @@
 #include <GUI/FGQmlPropertyNode.hxx>
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
-#include <Viewer/OSGQtAdaption.hxx>
+#include <GUI/OSGQtAdaption.hxx>
 #include <simgear/structure/commands.hxx>
 
 #if defined(HAVE_PUI)
@@ -172,8 +173,8 @@ public:
     QOpenGLContext* qtContext = nullptr;
     osg::GraphicsContext* osgContext = nullptr;
     
-    std::atomic_bool renderControlInited;
-    std::atomic_bool syncPending;
+    std::atomic_int renderControlInited;
+    std::atomic_int syncPending;
     
     void frameEvent()
     {
@@ -226,13 +227,7 @@ public slots:
     void initRenderControl()
     {
         qtContext = flightgear::qtContextFromOSG(osgContext);
-                
-    #if QT_VERSION < 0x050600
-        SG_LOG(SG_GUI, SG_ALERT, "Qt < 5.6 was used to build FlightGear, multi-threading of QtQuick is not safe");
-    #else
-        renderControl->prepareThread(QThread::currentThread());
-    #endif
-                
+        renderControl->prepareThread(QThread::currentThread());                
         QOpenGLContextPrivate::setCurrentContext(qtContext);
         QOpenGLContextPrivate::get(qtContext)->surface = foreignOSGWindow;
         renderControl->initialize(qtContext);
@@ -579,13 +574,18 @@ void QQuickDrawable::drawImplementation(osg::RenderInfo& renderInfo) const
     d->quickWindow->resetOpenGLState();
 }
 
-void QQuickDrawable::reload(QUrl url)
+void QQuickDrawable::reload(const QUrl& url)
 {
     d->qmlEngine->clearComponentCache();
     setSource(url);
 }
 
-void QQuickDrawable::setSource(QUrl url)
+void QQuickDrawable::setSourcePath(const std::string& path)
+{
+    setSource(QUrl::fromLocalFile(QString::fromStdString(path)));
+}
+
+void QQuickDrawable::setSource(const QUrl& url)
 {
     if (d->rootItem)
         delete d->rootItem;
