@@ -43,6 +43,10 @@
 #include <QUrl>
 #include <QtGlobal>
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 1, 0))
+#include <QNetworkInformation>
+#endif
+
 // Simgear
 #include <simgear/timing/timestamp.hxx>
 #include <simgear/props/props_io.hxx>
@@ -66,6 +70,7 @@
 #include <Navaids/navrecord.hxx>
 
 
+#include <Main/MultipleInstanceLock.hxx>
 #include <Main/fg_init.hxx>
 #include <Main/locale.hxx>
 #include <Main/options.hxx>
@@ -76,16 +81,22 @@
 #include "LauncherMainWindow.hxx"
 #include "LocalAircraftCache.hxx"
 #include "PathListModel.hxx"
+#include "SettingsWrapper.hxx"
+#include "SetupRootDialog.hxx"
 #include "UnitsModel.hxx"
-#include "GettingStartedTip.hxx"
 
 #if defined(SG_MAC)
 #include <GUI/CocoaHelpers.h>
 #endif
 
+#if defined(Q_OS_WIN)
+#include <windows.h> // for GetAsyncKeyState and VK_ constants
+#endif
+
 using namespace flightgear;
 using namespace simgear::pkg;
 using std::string;
+using namespace std::chrono_literals;
 
 namespace { // anonymous namespace
 
@@ -109,47 +120,18 @@ bool initNavCache()
 
     const auto wflags = Qt::Dialog | Qt::CustomizeWindowHint | Qt::WindowTitleHint | Qt::WindowSystemMenuHint | Qt::MSWindowsFixedSizeDialogHint;
 
-    if (NavDataCache::isAnotherProcessRebuilding()) {
-        const char* waitForOtherMsg = QT_TRANSLATE_NOOP("initNavCache", "Another copy of FlightGear is creating the navigation database. Waiting for it to finish.");
-        QString m = qApp->translate("initNavCache", waitForOtherMsg);
-
-        addSentryBreadcrumb("Launcher: showing wait for other process NavCache rebuild dialog", "info");
-        QProgressDialog waitForRebuild(m,
-                                       QString() /* cancel text */,
-                                       0, 0, Q_NULLPTR,
-                                       wflags);
-        waitForRebuild.setWindowModality(Qt::WindowModal);
-        waitForRebuild.setMinimumWidth(600);
-        waitForRebuild.setAutoReset(false);
-        waitForRebuild.setAutoClose(false);
-        waitForRebuild.show();
-
-        QTimer updateTimer;
-        updateTimer.setInterval(500);
-        bool rebuildIsDone = false;
-
-        QObject::connect(&updateTimer, &QTimer::timeout, [&waitForRebuild, &rebuildIsDone]() {
-            if (!NavDataCache::isAnotherProcessRebuilding()) {
-                waitForRebuild.done(0);
-                rebuildIsDone = true;
-                return;
-            }
-        });
-
-        updateTimer.start(); // timer won't actually run until we process events
-        waitForRebuild.exec();
-        updateTimer.stop();
-
-        if (!rebuildIsDone) {
-            flightgear::addSentryBreadcrumb("Launcher wait on other process nav-cache rebuild abandoned by user", "info");
+    if (flightgear::ExclusiveInstanceLock::isLocked()) {
+        bool ok = flightgear::ExclusiveInstanceLock::showWaitDialog();
+        if (!ok) {
             return false;
         }
-
-        addSentryBreadcrumb("Launcher: done waiting for other process NavCache rebuild dialog", "info");
     }
 
     NavDataCache* cache = NavDataCache::createInstance();
     if (cache->isRebuildRequired()) {
+        // start the rebuild right now, before showing the dialog
+        cache->rebuild();
+
         QProgressDialog rebuildProgress(baseLabel,
                                         QString() /* cancel text */,
                                         0, 100, Q_NULLPTR,
@@ -203,6 +185,10 @@ bool initNavCache()
         flightgear::addSentryBreadcrumb("Launcher nav-cache rebuild complete", "info");
     }
 
+
+    // if using the launcher, this is when we're done with our exclusive section
+    // from now on, multiple copies can access FG_HOME / the nav-cache
+    flightgear::ExclusiveInstanceLock::destroyInstance();
     return true;
 }
 
@@ -211,8 +197,7 @@ class NaturalEarthDataLoaderThread : public QThread
     Q_OBJECT
 public:
 
-    NaturalEarthDataLoaderThread() :
-        m_lineInsertCount(0)
+    NaturalEarthDataLoaderThread()
     {
         connect(this, &QThread::finished, this, &NaturalEarthDataLoaderThread::onFinished);
     }
@@ -224,10 +209,26 @@ public:
 protected:
     void run() override
     {
-        loadNaturalEarthFile("ne_10m_coastline.shp", flightgear::PolyLine::COASTLINE, false);
-        loadNaturalEarthFile("ne_10m_rivers_lake_centerlines.shp", flightgear::PolyLine::RIVER, false);
-        loadNaturalEarthFile("ne_10m_lakes.shp", flightgear::PolyLine::LAKE, true);
-        loadNaturalEarthFile("ne_10m_urban_areas.shp", flightgear::PolyLine::URBAN, true);
+        struct FileAndType {
+            std::string file;
+            flightgear::PolyLine::Type type;
+            bool closed = false;
+        };
+
+        const std::initializer_list<FileAndType> files = {
+            {"ne_10m_coastline.shp", flightgear::PolyLine::COASTLINE, false},
+            {"ne_10m_rivers_lake_centerlines.shp", flightgear::PolyLine::RIVER, false},
+            {"ne_10m_lakes.shp", flightgear::PolyLine::LAKE, true},
+            {"ne_10m_urban_areas.shp", flightgear::PolyLine::URBAN, true}
+        };
+
+        for (const auto& d : files) {
+            if (m_abandoned) {
+                break;
+            }
+
+            loadNaturalEarthFile(d.file, d.type, d.closed);
+        }
     }
 
 private:
@@ -255,7 +256,6 @@ private:
     }
 
     flightgear::PolyLineList m_parsedLines;
-    unsigned int m_lineInsertCount;
     bool m_abandoned = false;
 };
 
@@ -301,11 +301,6 @@ OpenGLStatus checkForWorkingOpenGL()
                 flightgear::addSentryBreadcrumb("Detected Intel < 2.1 renderer", "info");
                 result = OpenGLStatus::Intel14;
             }
-        } else if (simgear::strutils::starts_with(renderer, "S3 Graphics")) {
-            if (ctx.format().majorVersion() < 2) {
-                flightgear::addSentryBreadcrumb("Detected S2 < 2.1 renderer", "info");
-                result = OpenGLStatus::Unknown;
-            }
         }
 
         // ensure the context is no longer current on the offscreen
@@ -331,9 +326,7 @@ static void simgearMessageOutput(QtMsgType type, const QMessageLogContext &conte
     sgDebugPriority mappedPriority = SG_WARN;
     switch (type) {
     case QtDebugMsg:    mappedPriority = SG_DEBUG; break;
-#if QT_VERSION >= 0x050500
     case QtInfoMsg:     mappedPriority = SG_INFO; break;
-#endif
     case QtWarningMsg:  mappedPriority = SG_WARN; break;
     case QtCriticalMsg: mappedPriority = SG_ALERT; break;
     case QtFatalMsg:    mappedPriority = SG_POPUP; break;
@@ -343,7 +336,7 @@ static void simgearMessageOutput(QtMsgType type, const QMessageLogContext &conte
     const char* file = context.file ? context.file : nullFile;
     const auto s = msg.toStdString();
     // important we copy the file name here, since QMessageLogContext doesn't
-    sglog().logCopyingFilename(SG_GUI, mappedPriority, file, context.line, s);
+    sglog().logCopyingFilename(SG_GUI, mappedPriority, file, context.line, "" /*function*/, s);
     if (type == QtFatalMsg) {
         // if we abort() here, we get a thread crash which prevents
         // us actually seeing the error
@@ -372,7 +365,7 @@ void selectUITranslation()
     QStringList uiLanguages = QLocale::system().uiLanguages();
     //qWarning() << "UI languages:" << uiLanguages;
 
-    for (QString locale : qAsConst(uiLanguages)) {
+    for (QString locale : std::as_const(uiLanguages)) {
         // remove script if it exists, eg zh-Hans-CN -> zh-CN
         locale = QLocale(locale).name();
         locale.replace('-', '_');
@@ -399,11 +392,6 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
     if (!qtInitDone) {
         qtInitDone = true;
 
-        // Disable Qt 5.15 warnings about obsolete Connections/onFoo: syntax
-        // we cannot use the new syntax
-        // as long as we have to support Qt 5.9
-        qputenv("QT_LOGGING_RULES", "qt.qml.connections.warning=false");
-        
         initQtResources(); // can't be called from a namespace
 
         s_argc = argc; // QApplication only stores a reference to argc,
@@ -424,29 +412,38 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
 		// leave things unset here, so users can use env var
 		// QT_AUTO_SCREEN_SCALE_FACTOR=1 to enable it at runtime
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #if !defined (SG_WINDOWS)
-    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif
 #endif
         static_qApp.reset(new QApplication(s_argc, argv));
+        // This processEvents() call prevents a crash when FG is started
+        // under X11 without the built-in launcher and a Qt dialog is
+        // opened (SIGPIPE received as Qt calls IceProcessMessages()).
+        // See https://stackoverflow.com/a/56687321 and
+        // https://bugreports.qt.io/browse/QTBUG-58709 for more info.
+        static_qApp->processEvents();
         static_qApp->setOrganizationName("FlightGear");
         static_qApp->setApplicationName("FlightGear");
         static_qApp->setOrganizationDomain("flightgear.org");
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
         static_qApp->setDesktopFileName(
           QStringLiteral("org.flightgear.FlightGear.desktop"));
-#endif
+
         QTranslator* fallbackTranslator = new QTranslator(static_qApp.get());
         if (!fallbackTranslator->load(QLatin1String(":/FlightGear_en_US.qm"))) {
-            qWarning() << "Failed to load default (en) translations";
+            qWarning() << "Failed to load the built-in launcher fallback"
+                       << "translation (English). If you compiled FlightGear"
+                       << "yourself, you may want to pass -DFG_DATA_DIR to"
+                       << "CMake so as to allow the FlightGear build system"
+                       << "to find FGData.";
             delete fallbackTranslator;
         } else {
             static_qApp->installTranslator(fallbackTranslator);
         }
 
-        // check for --langauge=xx option and prefer that over QLocale
+        // check for --language=xx option and prefer that over QLocale
         // detection of the locale if it exists
         auto lang = simgear::strutils::replace(
             Options::getArgValue(argc, argv, "--language"),
@@ -459,22 +456,13 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
                 qInfo() << "Loaded translations based on --language from:" << localeFile;
                 static_qApp->installTranslator(translator);
             } else {
-                qInfo() << "--langauge was set, but no translations found at:" << localeFile;
+                qInfo() << "--language was set, but no translations found at:" << localeFile;
                 delete translator;
             }
         } else {
             selectUITranslation();
         }
 
-        // temporary code: only enable getting-started tips for English, to give translators time to
-        // catch up
-        if (!lang.empty()) {
-            GettingStartedTip::setGlobalTipsEnabled(simgear::strutils::starts_with(lang, "en"));
-        } else {
-            QLocale currentLocale;
-            GettingStartedTip::setGlobalTipsEnabled(currentLocale.name().startsWith("en"));
-        }
-
         // reset numeric / collation locales as described at:
         // http://doc.qt.io/qt-5/qcoreapplication.html#details
         ::setlocale(LC_NUMERIC, "C");
@@ -495,6 +483,15 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
 #endif
     }
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 3, 0))
+    QNetworkInformation::loadDefaultBackend();
+#elif (QT_VERSION >= QT_VERSION_CHECK(6,1,0))
+    bool ok = QNetworkInformation::load(QNetworkInformation::Feature::Reachability);
+    if (!ok) {
+        qInfo() << "network information plugins:" << QNetworkInformation::availableBackends();
+    }
+#endif
+
     if (doInitQSettings) {
         initQSettings();
     }
@@ -515,8 +512,9 @@ void initQSettings()
 
     if (!qSettingsInitDone) {
         qRegisterMetaType<QuantityValue>();
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         qRegisterMetaTypeStreamOperators<QuantityValue>("QuantityValue");
-
+#endif
 
         qSettingsInitDone = true;
         string fgHome = globals->get_fg_home().utf8Str();
@@ -527,6 +525,28 @@ void initQSettings()
     }
 }
 
+QSettings getQSettings()
+{
+#if defined(SG_MAC)
+    return QSettings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationDomain(),
+                     "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#else
+    return QSettings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(),
+                     "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#endif
+}
+
+std::unique_ptr<QSettings> createQSettings()
+{
+#if defined(SG_MAC)
+    return std::make_unique<QSettings>(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationDomain(),
+                                       "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#else
+    return std::make_unique<QSettings>(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(),
+                                       "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#endif
+}
+
 bool checkKeyboardModifiersForSettingFGRoot()
 {
     initQSettings();
@@ -611,7 +631,7 @@ void launcherSetSceneryPaths()
 
 // mimic what options.cxx does, so we can find airport data for parking
 // positions
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     // append explicit scenery paths
     Q_FOREACH(QString path, PathListModel::readEnabledPaths("scenery-paths-v2")) {
         globals->append_fg_scenery(path.toStdString());
@@ -663,7 +683,7 @@ bool runLauncherDialog()
         // launcher GUI. We'll disable the UI.
         LaunchConfig::setEnableDownloadDirUI(false);
     } else {
-        QSettings settings;
+        auto settings = flightgear::getQSettings();
         QString downloadDir = settings.value("download-dir").toString();
         if (!downloadDir.isEmpty()) {
             options->setOption("download-dir", downloadDir.toStdString());
@@ -684,13 +704,6 @@ bool runLauncherDialog()
     globals->get_locale()->selectLanguage(lang);
     globals->packageRoot()->setLocale(globals->get_locale()->getPreferredLanguage());
 
-    // startup the HTTP system now since packages needs it
-    FGHTTPClient* http = globals->add_new_subsystem<FGHTTPClient>();
-
-    // we guard against re-init in the global phase; bind and postinit
-    // will happen as normal
-    http->init();
-
     QPointer<NaturalEarthDataLoaderThread> naturalEarthLoader = new NaturalEarthDataLoaderThread;
     naturalEarthLoader->start();
 
@@ -698,11 +711,15 @@ bool runLauncherDialog()
     // try to initialise various Cocoa structures.
     flightgear::WindowBuilder::setPoseAsStandaloneApp(false);
 
+
+    FGHTTPClient::getOrCreate();
+
     LauncherMainWindow dlg(false);
-    if (options->isOptionSet("enable-fullscreen")) {
+
+    if (options->isOptionSet("fullscreen")) {
         dlg.showFullScreen();
     } else {
-        dlg.show();
+        dlg.setVisible(true);
     }
 
     int appResult = qApp->exec();
@@ -763,12 +780,21 @@ LockFileDialogResult showLockFileDialog()
     mb.setIconPixmap(QPixmap(":/app-icon-large"));
     mb.setWindowTitle(title);
     mb.setText(text);
-    mb.setInformativeText(infoText);
     mb.addButton(QMessageBox::Ok);
     mb.setDefaultButton(QMessageBox::Ok);
+#if !defined(SG_WINDOWS)
     mb.addButton(QMessageBox::Reset);
+    mb.setInformativeText(infoText);
+#endif
     mb.addButton(QMessageBox::Close);
 
+    // no lock file on Windows, so don't show the reset text/button. Enabling it
+    // causes more errors due to people clicking it erroneously
+#if !defined(SG_WINDOWS)
+    mb.setInformativeText(infoText);
+    mb.addButton(QMessageBox::Reset);
+#endif
+
     int r = mb.exec();
     if (r == QMessageBox::Reset)
         return LockFileReset;
@@ -777,6 +803,68 @@ LockFileDialogResult showLockFileDialog()
     return LockFileContinue;
 }
 
+bool showSetupRootDialog(bool usingDefaultRoot)
+{
+    return SetupRootDialog::runDialog(usingDefaultRoot);
+}
+
+bool showUpdateRootDialog(bool usingDefaultRoot)
+{
+    return SetupRootDialog::runUpdateDialog(usingDefaultRoot);
+}
+
+
+SetupRootResult restoreUserSelectedRoot(SGPath& path)
+{
+    return SetupRootDialog::restoreUserSelectedRoot(path);
+}
+
+// defined here since it uses QWidgets
+bool ExclusiveInstanceLock::showWaitDialog()
+{
+    const char* waitForOtherMsg = QT_TRANSLATE_NOOP("initNavCache", "Another copy of FlightGear is preparing data files. Waiting for it to finish.");
+    QString m = qApp->translate("initNavCache", waitForOtherMsg);
+
+    const auto wflags = Qt::Dialog | Qt::CustomizeWindowHint | Qt::WindowTitleHint | Qt::WindowSystemMenuHint | Qt::MSWindowsFixedSizeDialogHint;
+
+    addSentryBreadcrumb("showing wait for other process dialog", "info");
+
+    QProgressDialog waitDialog(m,
+                               QString() /* cancel text */,
+                               0, 0, Q_NULLPTR,
+                               wflags);
+    waitDialog.setWindowModality(Qt::WindowModal);
+    waitDialog.setMinimumWidth(600);
+    waitDialog.setAutoReset(false);
+    waitDialog.setAutoClose(false);
+    waitDialog.show();
+
+    QTimer updateTimer;
+    updateTimer.setInterval(500);
+    bool done = false;
+
+    QObject::connect(&updateTimer, &QTimer::timeout, [&waitDialog, &done]() {
+        auto lockedReason = isLocked();
+        if (!lockedReason.has_value()) {
+            waitDialog.done(0);
+            done = true;
+            return;
+        }
+    });
+
+    updateTimer.start(); // timer won't actually run until we process events
+    waitDialog.exec();
+    updateTimer.stop();
+
+    if (!done) {
+        addSentryBreadcrumb("wait on other process abandoned by user", "info");
+        return false;
+    }
+
+    addSentryBreadcrumb("done waiting for other process lock dialog", "info");
+    return true;
+}
+
 } // of namespace flightgear
 
 #include "QtLauncher.moc"
