@@ -53,6 +53,7 @@
 #include "QtLauncher.hxx"
 #include "SettingsWrapper.hxx"
 #include "UpdateDownloadedFGData.hxx"
+#include "DownloadTerrasyncSharedData.hxx"
 #include <GUI/QtDNSClient.hxx>
 #include <Main/MultipleInstanceLock.hxx>
 
@@ -62,7 +63,6 @@
 
 using namespace std::chrono_literals;
 
-const quint32 static_basePackagePatchLevel = 3;
 const qint64 fourMB = 4 * 1024 * 1024;
 
 namespace {
@@ -184,6 +184,12 @@ public:
             return 0;
         }
 
+        // open the file in read-write mode
+        if (!m_resumeData.open(QIODevice::ReadWrite)) {
+            qWarning() << "Failed to open download resume file in read-write mode";
+            return 0;
+        }
+
         req.setRawHeader("Range", QString("bytes=%1-").arg(resumeBytes).toUtf8());
         m_readResumeFile = true;
     
@@ -219,14 +225,14 @@ public:
             }
         }
 
-        QString templateUrl = m_servers.front() + QStringLiteral("/release-%1/FlightGear-%2.%3-data.txz");
+        QString templateUrl = m_servers.front() + QStringLiteral("/release-%1/FlightGear-%2-data.txz");
         if (templateUrl.startsWith("https://sourceforge.net/")) {
             // deal with different SF syntax
             templateUrl += QStringLiteral("/download");
         }
 
         QString majorMinorVersion = QString(FLIGHTGEAR_MAJOR_MINOR_VERSION);
-        m_downloadUrl = QUrl(templateUrl.arg(majorMinorVersion).arg(majorMinorVersion).arg(static_basePackagePatchLevel));
+        m_downloadUrl = QUrl(templateUrl.arg(majorMinorVersion).arg(FLIGHTGEAR_VERSION));
 
         qInfo() << "Download URI:" << m_downloadUrl;
 
@@ -239,20 +245,15 @@ public:
         // check if we can resume an existing download, returns the
         // number of overlap bytes or zero for no resume.
         m_resumeOverlapBytes = resumeDownload(req);
+        if (!m_readResumeFile) {
+            // if we're not resuming, empty the file and open it write-only
+            m_resumeData.open(QIODevice::WriteOnly | QIODevice::Truncate);
+        }
 
         m_download = m_networkManager->get(req);
         m_download->setReadBufferSize(64 * 1024 * 1024);
 
-        if (!m_readResumeFile) {
-            // if we're not resuming, just write to the resume file
-            m_resumeData.open(QIODevice::WriteOnly | QIODevice::Truncate);
-        }
-
         connect(m_download, &QNetworkReply::downloadProgress, this, &InstallFGDataThread::onDownloadProgress);
-
-        // lambda slot, but scoped to an object living on this thread.
-        // this means the extraction work is done asynchronously with the
-        // download
         connect(m_download, &QNetworkReply::readyRead, this, &InstallFGDataThread::processBytes);
         connect(m_download, &QNetworkReply::finished, this, &InstallFGDataThread::onReplyFinished);
         connect(m_download, &QNetworkReply::metaDataChanged, this, &InstallFGDataThread::onMetaDataChanged);
@@ -307,7 +308,7 @@ public:
                 }
                 m_archive->extractBytes((const uint8_t*)resumeBytes.constData(), resumeBytes.size());
                 m_extractedBytes += resumeBytes.size();
-                if (m_resumeData.pos() >= m_resumedBytesSize) {
+                if (m_resumeData.pos() >= static_cast<qint64>(m_resumedBytesSize)) {
                     qInfo() << "done reading resume file bytes";
                     m_readResumeFile = false;
                 }
@@ -543,7 +544,8 @@ SetupRootDialog::SetupRootDialog(PromptState prompt, const SGPath& checked) :
     m_promptState(prompt),
     m_checkedPath(checked)
 {
-    flightgear::ExclusiveInstanceLock::instance()->updateReason("setup-fgdata");
+    auto exLock = flightgear::ExclusiveInstanceLock::instance();
+    exLock->updateReason("setup-fgdata");
 
     m_ui.reset(new Ui::SetupRootDialog);
     m_ui->setupUi(this);
@@ -554,6 +556,8 @@ SetupRootDialog::SetupRootDialog(PromptState prompt, const SGPath& checked) :
             this, &SetupRootDialog::onDownload);
     connect(m_ui->changeDownloadLocation, &QPushButton::clicked,
             this, &SetupRootDialog::onSelectDownloadDir);
+    connect(m_ui->defaultDownloadLocation, &QPushButton::clicked,
+            this, &SetupRootDialog::onUseDefaultDownloadDir);
     connect(m_ui->buttonBox, &QDialogButtonBox::rejected,
             this, &QDialog::reject);
 
@@ -561,8 +565,9 @@ SetupRootDialog::SetupRootDialog(PromptState prompt, const SGPath& checked) :
     if (options->isOptionSet("download-dir")) {
         // if download dir is set on the command line, don't allow changing it here
         m_ui->changeDownloadLocation->setEnabled(false);
+        m_ui->defaultDownloadLocation->hide();
     }
-
+    
     m_ui->versionLabel->setText(tr("<h1>FlightGear %1</h1>").arg(FLIGHTGEAR_VERSION));
     m_ui->bigIcon->setPixmap(QPixmap(":/app-icon-large"));
     m_ui->contentsPages->setCurrentIndex(0);
@@ -571,7 +576,8 @@ SetupRootDialog::SetupRootDialog(PromptState prompt, const SGPath& checked) :
 
     if (prompt == NeedToUpdateDownloadedData) {
         m_ui->downloadButton->setText(tr("Update"));
-        m_ui->changeDownloadLocation->setEnabled(false);
+        m_ui->changeDownloadLocation->hide();
+        m_ui->defaultDownloadLocation->hide();
     }
 
     m_networkManager = new QNetworkAccessManager(this);
@@ -613,14 +619,13 @@ bool SetupRootDialog::runDialog(PromptState prompt, const SGPath& checkedPath)
 flightgear::SetupRootResult SetupRootDialog::restoreUserSelectedRoot(SGPath& sgpath)
 {
     const auto readOnly = fgGetBool("/sim/fghome-readonly", false);
-
+    auto options = flightgear::Options::sharedInstance();
     auto settings = flightgear::getQSettings();
     QString path = settings.value(rootPathKey()).toString();
     const bool ask = flightgear::checkKeyboardModifiersForSettingFGRoot();
 
     QString downloadDir = settings.value("download-dir").toString();
     if (!downloadDir.isEmpty()) {
-        auto options = flightgear::Options::sharedInstance();
         options->setCustomDownloadDir(SGPath::fromUtf8(downloadDir.toStdString()));
     }
 
@@ -631,7 +636,7 @@ flightgear::SetupRootResult SetupRootDialog::restoreUserSelectedRoot(SGPath& sgp
             return flightgear::SetupRootResult::UserExit;
         }
 
-        bool ok = runDialog(ManualChoiceRequested);
+        bool ok = runDialog(ManualChoiceRequested, SGPath{});
         if (!ok) {
             return flightgear::SetupRootResult::UserExit;
         }
@@ -649,13 +654,23 @@ flightgear::SetupRootResult SetupRootDialog::restoreUserSelectedRoot(SGPath& sgp
 
     if (path.isEmpty()) {
         if (downloadedDataExistsButStale()) {
-            bool ok = runDialog(NeedToUpdateDownloadedData, flightgear::Options::sharedInstance()->downloadedDataRoot());
+            bool ok = runDialog(NeedToUpdateDownloadedData, options->downloadedDataRoot());
             if (!ok) {
                 return flightgear::SetupRootResult::UserExit;
             }
 
             // assume update worked, fall through
-        } 
+        }
+        
+        const auto pkgData = options->platformDefaultRoot();
+        if (flightgear::Options::isFGData(pkgData)) {
+            const auto pkgDataQt = QString::fromStdString(pkgData.utf8Str());
+            if (validateVersion(pkgDataQt)) {
+                return flightgear::SetupRootResult::UseDefault;
+            }
+
+            qWarning() << pkgDataQt << "contains FGData, but version doesn't match required:" << FLIGHTGEAR_VERSION;
+        }
     }
 
 // to give better feedback, we need to record which path we tried,
@@ -667,7 +682,7 @@ flightgear::SetupRootResult SetupRootDialog::restoreUserSelectedRoot(SGPath& sgp
             return flightgear::SetupRootResult::RestoredOk;
         }
 
-        // path semed good, but version failed, so this is the one to report
+        // path seemed good, but version failed, so this is the one to report
         checkedPath = SGPath::fromUtf8(path.toStdString());
     }
 
@@ -721,8 +736,7 @@ bool SetupRootDialog::validatePath(QString path)
  */
 bool SetupRootDialog::validateVersion(QString path)
 {
-    std::string minBasePackageVersion = std::to_string(FLIGHTGEAR_MAJOR_VERSION) + "." + std::to_string(FLIGHTGEAR_MINOR_VERSION) + "." + std::to_string(static_basePackagePatchLevel);
-
+    const std::string minBasePackageVersion{FLIGHTGEAR_VERSION};
     std::string ver = fgBasePackageVersion(SGPath::fromUtf8(path.toStdString()));
 
     // ensure major & minor fields match exactly
@@ -730,6 +744,7 @@ bool SetupRootDialog::validateVersion(QString path)
         return false;
     }
 
+    // patch level must be higher
     return simgear::strutils::compare_versions(minBasePackageVersion, ver) <= 0;
 }
 
@@ -748,7 +763,7 @@ bool SetupRootDialog::downloadedDataExistsButStale()
         return false;
     }
 
-    std::string minBasePackageVersion = std::to_string(FLIGHTGEAR_MAJOR_VERSION) + "." + std::to_string(FLIGHTGEAR_MINOR_VERSION) + "." + std::to_string(static_basePackagePatchLevel);
+    const std::string minBasePackageVersion{FLIGHTGEAR_VERSION};
     std::string ver = fgBasePackageVersion(r);
 
     // major or minor mismatch, we can't use it
@@ -775,7 +790,7 @@ bool SetupRootDialog::downloadedDataExistsButStale()
         return true;
     }
 
-    // update needed if the on-disk base package version is *lower* than static_basePackagePatchLevel
+    // update needed if the on-disk base package version is *lower* than our patch version
     return simgear::strutils::compare_versions(ver, minBasePackageVersion) < 0;
 }
 
@@ -809,6 +824,18 @@ void SetupRootDialog::onBrowse()
     accept(); // we're done
 }
 
+bool SetupRootDialog::locationIsWritable(QString path)
+{
+    // we don't use QFileInfo::isWriteable here because of complexity around
+    // NTFS ACL checks (needs Qt 6.6 for QNtfsPermissionCheckGuard)
+    QFile f(path + "/_check_write");
+    if (!f.open(QIODevice::NewOnly | QIODevice::WriteOnly)) {
+        return false;
+    }
+    f.remove(); // closes
+    return true;
+}
+
 void SetupRootDialog::onSelectDownloadDir()
 {
     auto settings = flightgear::getQSettings();
@@ -822,11 +849,27 @@ void SetupRootDialog::onSelectDownloadDir()
         return;
     }
 
+    if (!locationIsWritable(downloadDir)) {
+        m_browsedPath = downloadDir;
+        m_promptState = ChoseInvalidDownloadLocation;
+        updatePromptText();
+        return;
+    }
+
+    m_promptState = ManualChoiceRequested;
     settings.setValue("download-dir", downloadDir);
     flightgear::Options::sharedInstance()->setOption("download-dir", downloadDir.toStdString());
     updatePromptText();
 }
 
+void SetupRootDialog::onUseDefaultDownloadDir()
+{
+    auto settings = flightgear::getQSettings();
+    settings.remove("download-dir");
+    flightgear::Options::sharedInstance()->clearOption("download-dir");
+    updatePromptText();
+}
+
 void SetupRootDialog::onDownload()
 {
     // clear !ask value or custom root
@@ -839,6 +882,7 @@ void SetupRootDialog::onDownload()
     }
 
     m_promptState = DownloadingExtractingArchive;
+    m_archiveName = tr("base data package");
     updatePromptText();
 
     m_ui->contentsPages->setCurrentIndex(1);
@@ -859,7 +903,6 @@ void SetupRootDialog::onDownload()
     connect(installThread, &InstallFGDataThread::installProgress, this, [this](QString s, int percent) {
         m_ui->installText->setText(tr("Installation %1% complete.\nExtracting %2").arg(percent).arg(s));
         m_ui->installProgress->setValue(percent);
-        // m_ui->installProgress->setMaximum(total);
     });
 
     connect(installThread, &InstallFGDataThread::failed, this, [this](QString s) {
@@ -875,13 +918,49 @@ void SetupRootDialog::onDownload()
             updatePromptText();
             m_ui->contentsPages->setCurrentIndex(0);
         } else {
-            accept();
+            startSharedDataDownload();
         }
     });
 
     installThread->start();
 }
 
+void SetupRootDialog::startSharedDataDownload()
+{
+    auto dlThread = new DownloadTerrasyncSharedData(this, m_networkManager);
+    connect(dlThread, &DownloadTerrasyncSharedData::downloadProgress, this, [this](quint64 current, quint64 total) {
+        m_ui->downloadProgress->setValue(current);
+        m_ui->downloadProgress->setMaximum(total);
+
+        const quint64 currentMb = current / (1024 * 1024);
+        const quint64 totalMb = total / (1024 * 1024);
+
+        const int percent = calculateProgressPercentage(current, total);
+
+        m_ui->downloadText->setText(tr("Downloading %1 of %2 MB (%3%)").arg(currentMb).arg(totalMb).arg(percent));
+    });
+
+    connect(dlThread, &DownloadTerrasyncSharedData::beginArchive, this, [this](QString s) {
+        m_archiveName = s;
+        updatePromptText();
+    });
+
+    connect(dlThread, &DownloadTerrasyncSharedData::installProgress, this, [this](QString s, int percent) {
+        m_ui->installText->setText(tr("Installation %1% complete.\nExtracting %2").arg(percent).arg(s));
+        m_ui->installProgress->setValue(percent);
+    });
+
+    connect(dlThread, &DownloadTerrasyncSharedData::failed, this, [this](QString s) {
+        accept();
+    });
+
+    connect(dlThread, &DownloadTerrasyncSharedData::finished, this, [this]() {
+        accept();
+    });
+
+    dlThread->start();
+}
+
 void SetupRootDialog::onUpdate()
 {
     m_promptState = UpdatingViaTerrasync;
@@ -966,11 +1045,13 @@ void SetupRootDialog::updatePromptText()
         t = tr("The chosen file (%1) is not a valid compressed archive.").arg(m_browsedPath);
         break;
 
-
-    case DownloadingExtractingArchive:
-        t = tr("Please wait while the data files are downloaded, extracted and verified.");
+    case ChoseInvalidDownloadLocation:
+        t = tr("The chosen download location (%1) is not writable. Please select another location.").arg(m_browsedPath);
         break;
 
+    case DownloadingExtractingArchive:
+        t = tr("Please wait while the data files are downloaded, extracted and verified.\nCurrent archive: %1").arg(m_archiveName);
+        break;
 
     case UpdatingViaTerrasync:
         t = tr("Please wait while the data files are updated and verified.");
@@ -995,6 +1076,8 @@ void SetupRootDialog::updatePromptText()
     auto dd = flightgear::Options::sharedInstance()->actualDownloadDir();
     const auto dlp = QString::fromStdString(dd.utf8Str());
     m_ui->downloadLocationLabel->setText(tr("Data files will be downloaded to: %1").arg(dlp));
+
+    m_ui->defaultDownloadLocation->setEnabled(flightgear::Options::sharedInstance()->isOptionSet("download-dir"));
 }
 
 #include "SetupRootDialog.moc"
