@@ -20,11 +20,11 @@ namespace {
 HWND getMainViewerHWND()
 {
 	osgViewer::Viewer::Windows windows;
-	if (!globals->get_renderer() || !globals->get_renderer()->getViewer()) {
+	if (!globals->get_renderer() || !globals->get_renderer()->getViewerBase()) {
 		return 0;
 	}
 
-    globals->get_renderer()->getViewer()->getWindows(windows);
+    globals->get_renderer()->getViewerBase()->getWindows(windows);
     osgViewer::Viewer::Windows::const_iterator it = windows.begin();
     for(; it != windows.end(); ++it) {
         if (strcmp((*it)->className(), "GraphicsWindowWin32")) {
@@ -67,14 +67,14 @@ WindowsFileDialog::~WindowsFileDialog()
 void WindowsFileDialog::exec()
 {
     char Filestring[MAX_PATH] = "\0";
-    OPENFILENAME opf={0};
+    OPENFILENAMEA opf = {0};
     opf.lStructSize = sizeof(OPENFILENAME);
     opf.lpstrFile = Filestring;
     opf.lpstrTitle = const_cast<char *>(_title.c_str());
     opf.nMaxFile = MAX_PATH;
 
     std::string extensions;
-    size_t extensionsLen;
+    size_t extensionsLen=0;
     if (!_filterPatterns.empty()) {
         for (const auto& ext : _filterPatterns) {
             if (!simgear::strutils::starts_with(ext, "*.")) {
@@ -98,14 +98,14 @@ void WindowsFileDialog::exec()
     if (_usage == USE_SAVE_FILE) {
         if (GetSaveFileNameA(&opf)) {
             std::string stringPath(opf.lpstrFile);
-            _callback->onFileDialogDone(this, stringPath);
+            handleSelectedPath(SGPath::fromUtf8(stringPath));
         }
     } else if (_usage == USE_CHOOSE_DIR) {
         chooseDir();
     } else {
         if (GetOpenFileNameA(&opf)) {
             std::string stringPath(opf.lpstrFile);
-            _callback->onFileDialogDone(this, stringPath);
+            handleSelectedPath(SGPath::fromUtf8(stringPath));
         }
     }
 }
@@ -122,9 +122,9 @@ void WindowsFileDialog::chooseDir()
 
 	char pathBuf[MAX_PATH] = "\0";
 
-	BROWSEINFO binfo;
-	memset(&binfo, 0, sizeof(BROWSEINFO));
-	binfo.hwndOwner = getMainViewerHWND();
+    BROWSEINFOA binfo;
+    memset(&binfo, 0, sizeof(BROWSEINFOA));
+    binfo.hwndOwner = getMainViewerHWND();
 	binfo.ulFlags = BIF_USENEWUI | BIF_RETURNONLYFSDIRS | BIF_EDITBOX;
 
 	binfo.pidlRoot = NULL; // can browse anywhere
@@ -132,14 +132,14 @@ void WindowsFileDialog::chooseDir()
 	binfo.lpfn = BrowseFolderCallback;
 	binfo.lParam = reinterpret_cast<LPARAM>(this);
 
-	PIDLIST_ABSOLUTE results = SHBrowseForFolder(&binfo);
-	if (results == NULL) {
+    PIDLIST_ABSOLUTE results = SHBrowseForFolderA(&binfo);
+    if (results == NULL) {
 		// user cancelled
 		return;
 	}
 
-	SHGetPathFromIDList(results, pathBuf);
-	CoTaskMemFree(results);
+    SHGetPathFromIDListA(results, pathBuf);
+    CoTaskMemFree(results);
 
-	_callback->onFileDialogDone(this, SGPath(pathBuf));
+    handleSelectedPath(SGPath(pathBuf));
 }
