@@ -1,8 +1,10 @@
 // new_gui.cxx: implementation of XML-configurable GUI support.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+/*
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
+
+#include <config.h>
 
 #include "new_gui.hxx"
 
@@ -12,18 +14,20 @@
 #include <sys/types.h>
 
 #include <simgear/compiler.h>
-#include <simgear/structure/exception.hxx>
-#include <simgear/props/props_io.hxx>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/misc/sg_dir.hxx>
+#include <simgear/props/props_io.hxx>
+#include <simgear/structure/exception.hxx>
 
 #include <Add-ons/AddonManager.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/sentryIntegration.hxx>
+#include <Scripting/NasalSys.hxx>
 
 #if defined(SG_UNIX) && !defined(SG_MAC) 
 #include "GL/glx.h"
 #endif
 
-#include "FGPUIMenuBar.hxx"
 
 #if defined(SG_MAC)
 #include "FGCocoaMenuBar.hxx"
@@ -33,10 +37,22 @@
 #include "FGWindowsMenuBar.hxx"
 #endif
 
+// ensure we include this before puAux.h, so that
+// #define _PU_H_ 1 has been done, and hence we don't
+// include the un-modified system pu.h
+#include "FlightGear_pu.h"
+
+#include <plib/puAux.h>
+
+
 #include "FGPUIDialog.hxx"
+#include "FGPUIMenuBar.hxx"
+
 #include "FGFontCache.hxx"
 #include "FGColor.hxx"
 
+#include "Highlight.hxx"
+
 // ignore the word Navaid here, it's a DataCache
 #include <Navaids/NavDataCache.hxx>
 
@@ -50,9 +66,7 @@ extern void puCleanUpJunk(void);
 ////////////////////////////////////////////////////////////////////////
 
 
-
-NewGUI::NewGUI () :
-  _active_dialog(0)
+NewGUI::NewGUI()
 {
 }
 
@@ -62,6 +76,47 @@ NewGUI::~NewGUI ()
         delete it->second;
 }
 
+// Recursively finds all nodes called <leaf> and appends the string values of
+// these nodes to <out>.
+//
+static void findAllLeafValues(SGPropertyNode* node, const std::string& leaf, std::vector<std::string>& out)
+{
+    string name = node->getNameString();
+    if (name == leaf) {
+        out.push_back(node->getStringValue());
+    }
+    for (int i=0; i<node->nChildren(); ++i) {
+        findAllLeafValues(node->getChild(i), leaf, out);
+    }
+}
+
+
+/* Registers menu-dialog associations with Highlight::add_menu_dialog(). */
+static void scanMenus()
+{
+    /* We find all nodes called 'dialog-name' in
+    sim/menubar/default/menu[]/item[]. */
+    SGPropertyNode* menubar = globals->get_props()->getNode("sim/menubar/default");
+    assert(menubar);
+    auto highlight = globals->get_subsystem<Highlight>();
+    if (!highlight) {
+        return;
+    }
+    for (int menu_p=0; menu_p<menubar->nChildren(); ++menu_p) {
+        SGPropertyNode* menu = menubar->getChild(menu_p);
+        if (menu->getNameString() != "menu") continue;
+        for (int item_p=0; item_p<menu->nChildren(); ++item_p) {
+            SGPropertyNode* item = menu->getChild(item_p);
+            if (item->getNameString() != "item") continue;
+            std::vector<std::string> dialog_names;
+            findAllLeafValues(item, "dialog-name", dialog_names);
+            for (auto dialog_name: dialog_names) {
+                highlight->addMenuDialog(HighlightMenu(menu->getIndex(), item->getIndex()), dialog_name);
+            }
+        }
+    }
+}
+
 void
 NewGUI::init ()
 {
@@ -76,40 +131,43 @@ NewGUI::init ()
     SGPath p(globals->get_fg_root(), "gui/dialogs");
     readDir(p);
     
-    SGPath aircraftDialogDir(string(fgGetString("/sim/aircraft-dir")), "gui/dialogs");
-    if (aircraftDialogDir.exists()) {
-        readDir(aircraftDialogDir);
-    }
+    if (fgGetBool("/sim/gui/startup") == false) {
+        SGPath aircraftDialogDir(fgGetString("/sim/aircraft-dir"), "gui/dialogs");
+        if (aircraftDialogDir.exists()) {
+            readDir(aircraftDialogDir);
+        }
 
-    // Read XML dialogs made available by registered add-ons
-    const auto& addonManager = flightgear::addons::AddonManager::instance();
-    for (const auto& addon: addonManager->registeredAddons()) {
-        SGPath addonDialogDir = addon->getBasePath() / "gui/dialogs";
+        // Read XML dialogs made available by registered add-ons
+        const auto& addonManager = flightgear::addons::AddonManager::instance();
+        for (const auto& addon: addonManager->registeredAddons()) {
+            SGPath addonDialogDir = addon->getBasePath() / "gui/dialogs";
 
-        if (addonDialogDir.exists()) {
-            readDir(addonDialogDir);
+            if (addonDialogDir.exists()) {
+                readDir(addonDialogDir);
+            }
         }
     }
 
     // Fix for http://code.google.com/p/flightgear-bugs/issues/detail?id=947
     fgGetNode("sim/menubar")->setAttribute(SGPropertyNode::PRESERVE, true);
     _menubar->init();
+    scanMenus();
 }
 
 void
 NewGUI::shutdown()
 {
-    DialogDict::iterator it = _active_dialogs.begin();
-    for (; it != _active_dialogs.end(); ++it) {
-        delete it->second;
-    }
     _active_dialogs.clear();
-    
+    _active_dialog.clear();
+
     fgUntie("/sim/menubar/visibility");
+    fgUntie("/sim/menubar/overlap-hide");
     _menubar.reset();
     _dialog_props.clear();
 
+#if defined(HAVE_PUI)
     puCleanUpJunk();
+#endif
 }
 
 void
@@ -140,7 +198,7 @@ NewGUI::createMenuBarImplementation()
    //     _menubar.reset(new FGWindowsMenuBar);
     }
 #endif
-    if (!_menubar.get()) {
+    if (!_menubar) {
         _menubar.reset(new FGPUIMenuBar);
     }
 }
@@ -187,11 +245,16 @@ NewGUI::unbind ()
 {
 }
 
+void NewGUI::postinit()
+{
+    _menubar->postinit();
+}
+
 void
 NewGUI::update (double delta_time_sec)
 {
     SG_UNUSED(delta_time_sec);
-    map<string,FGDialog *>::iterator iter = _active_dialogs.begin();
+    auto iter = _active_dialogs.begin();
     for(/**/; iter != _active_dialogs.end(); iter++)
         iter->second->update();
 }
@@ -199,6 +262,11 @@ NewGUI::update (double delta_time_sec)
 bool
 NewGUI::showDialog (const string &name)
 {
+    if (name.empty()) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "showDialog: no dialog name provided");
+        return false;
+    }
+
     // first, check if it's already shown
     if (_active_dialogs.find(name) != _active_dialogs.end()){
         _active_dialogs[name]->bringToFront();
@@ -207,14 +275,39 @@ NewGUI::showDialog (const string &name)
   
     // check we know about the dialog by name
     if (_dialog_names.find(name) == _dialog_names.end()) {
-        SG_LOG(SG_GENERAL, SG_ALERT, "Dialog " << name << " not defined");
+        simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::GUIDialog, "Dialog not found:" + name);
         return false;
     }
-    _active_dialogs[name] = new FGPUIDialog(getDialogProperties(name));
-    fgSetString("/sim/gui/dialogs/current-dialog", name);
 
-//    setActiveDialog(new FGPUIDialog(getDialogProperties(name)));
-    return true;
+    // log dialog show/close except these noisy ones (screen_window & screen_display)
+    if (name.find("__screen_") != 0) {
+        flightgear::addSentryBreadcrumb("showing GUI dialog:" + name, "info");
+    }
+
+    try {
+        _active_dialogs[name] = new FGPUIDialog(getDialogProperties(name));
+
+        fgSetString("/sim/gui/dialogs/current-dialog", name);
+
+        //    setActiveDialog(new FGPUIDialog(getDialogProperties(name)));
+        return true;
+    } catch (sg_exception& e) {
+        simgear::reportFailure(simgear::LoadFailure::Misconfigured, simgear::ErrorCode::GUIDialog, "Dialog failed to show:" + name + ":" + e.getFormattedMessage(), e.getLocation());
+        return false;
+    }
+}
+
+bool
+NewGUI::toggleDialog (const string &name)
+{
+    if (_active_dialogs.find(name) == _active_dialogs.end()) {
+        showDialog(name);
+        return true;
+    }
+    else {
+        closeDialog(name);
+        return false;
+    }
 }
 
 bool
@@ -223,22 +316,27 @@ NewGUI::closeActiveDialog ()
     if (_active_dialog == 0)
         return false;
 
+    // TODO support a request-close callback here, optionally
+    // many places in code assume this code-path does an
+    // immediate close, but for some UI paths it would be nice to
+    // allow some dialogs the chance to inervene
+
     // Kill any entries in _active_dialogs...  Is there an STL
     // algorithm to do (delete map entries by value, not key)?  I hate
     // the STL :) -Andy
-    map<string,FGDialog *>::iterator iter = _active_dialogs.begin();
+    auto iter = _active_dialogs.begin();
     for(/**/; iter != _active_dialogs.end(); iter++) {
         if(iter->second == _active_dialog) {
+            _active_dialog->close();
+
             _active_dialogs.erase(iter);
             // iter is no longer valid
             break;
         }
     }
 
-    delete _active_dialog;
     _active_dialog = 0;
-    if (_active_dialogs.size())
-    {
+    if (!_active_dialogs.empty()) {
         fgSetString("/sim/gui/dialogs/current-dialog", _active_dialogs.begin()->second->getName());
     }
     return true;
@@ -248,9 +346,13 @@ bool
 NewGUI::closeDialog (const string& name)
 {
     if(_active_dialogs.find(name) != _active_dialogs.end()) {
+        if (name.find("__screen_") != 0) {
+            flightgear::addSentryBreadcrumb("closing GUI dialog:" + name, "info");
+        }
+
         if(_active_dialog == _active_dialogs[name])
-            _active_dialog = 0;
-        delete _active_dialogs[name];
+            _active_dialog.clear();
+
         _active_dialogs.erase(name);
         return true;
     }
@@ -283,8 +385,8 @@ NewGUI::getDialogProperties (const string &name)
     return it->second;
 }
 
-FGDialog *
-NewGUI::getDialog (const string &name)
+NewGUI::FGDialogRef
+NewGUI::getDialog(const string& name)
 {
     if(_active_dialogs.find(name) != _active_dialogs.end())
         return _active_dialogs[name];
@@ -302,8 +404,8 @@ NewGUI::setActiveDialog (FGDialog * dialog)
     _active_dialog = dialog;
 }
 
-FGDialog *
-NewGUI::getActiveDialog ()
+NewGUI::FGDialogRef
+NewGUI::getActiveDialog()
 {
     return _active_dialog;
 }
@@ -344,12 +446,11 @@ NewGUI::setMenuBarOverlapHide(bool hide)
 void
 NewGUI::newDialog (SGPropertyNode* props)
 {
-    const char* cname = props->getStringValue("name");
-    if(!cname) {
+    string name = props->getStringValue("name", "");
+    if(name.empty()) {
         SG_LOG(SG_GENERAL, SG_ALERT, "New dialog has no <name> property");
         return;
     }
-    string name = cname;
   
     if(_active_dialogs.find(name) == _active_dialogs.end()) {
         _dialog_props[name] = props;
@@ -358,6 +459,7 @@ NewGUI::newDialog (SGPropertyNode* props)
     }
 }
 
+
 void
 NewGUI::readDir (const SGPath& path)
 {
@@ -370,7 +472,42 @@ NewGUI::readDir (const SGPath& path)
 
     flightgear::NavDataCache* cache = flightgear::NavDataCache::instance();
     flightgear::NavDataCache::Transaction txn(cache);
+    auto highlight = globals->get_subsystem<Highlight>();
     for (SGPath xmlPath : dir.children(simgear::Dir::TYPE_FILE, ".xml")) {
+
+      SGPropertyNode_ptr props = new SGPropertyNode;
+      SGPropertyNode *nameprop = nullptr;
+      std::string name;
+
+      /* Always parse the dialog even if cache->isCachedFileModified() says
+      it is cached, so that we can register property-dialog associations with
+      Highlight::add_property_dialog(). */
+      try {
+        readProperties(xmlPath, props);
+      } catch (const sg_exception &) {
+        SG_LOG(SG_INPUT, SG_ALERT, "Error parsing dialog " << xmlPath);
+        props.reset();
+      }
+      if (props) {
+        nameprop = props->getNode("name");
+        if (nameprop) {
+          name = nameprop->getStringValue();
+          // Look for 'property' nodes within the dialog. (could
+          // also look for "dialog-name" to catch things like
+          // dialog-show.)
+          std::vector<std::string> property_paths;
+          findAllLeafValues(props, "property", property_paths);
+          for (auto property_path: property_paths) {
+            // We could maybe hoist this test for hightlight to avoid reaching
+            // here if it is null, but that's difficult to test, so we do it
+            // here instead.
+            if (highlight) {
+              highlight->addPropertyDialog(property_path, name);
+            }
+          }
+        }
+      }
+        
       if (!cache->isCachedFileModified(xmlPath)) {
         // cached, easy
         string name = cache->readStringProperty(xmlPath.utf8Str());
@@ -378,32 +515,27 @@ NewGUI::readDir (const SGPath& path)
         continue;
       }
       
-    // we need to parse the actual XML
-      SGPropertyNode_ptr props = new SGPropertyNode;
-      try {
-        readProperties(xmlPath, props);
-      } catch (const sg_exception &) {
+      // we need to parse the actual XML
+      if (!props) {
         SG_LOG(SG_INPUT, SG_ALERT, "Error parsing dialog " << xmlPath);
         continue;
       }
       
-      SGPropertyNode *nameprop = props->getNode("name");
       if (!nameprop) {
         SG_LOG(SG_INPUT, SG_WARN, "dialog " << xmlPath << " has no name; skipping.");
         continue;
       }
       
-      string name = nameprop->getStringValue();
       _dialog_names[name] = xmlPath;
-    // update cached values
-        if (!cache->isReadOnly()) {
-            cache->stampCacheFile(xmlPath);
-            cache->writeStringProperty(xmlPath.utf8Str(), name);
-        }
+      // update cached values
+      if (!cache->isReadOnly()) {
+        cache->stampCacheFile(xmlPath);
+        cache->writeStringProperty(xmlPath.utf8Str(), name);
+      }
     } // of directory children iteration
   
     txn.commit();
-}
+}
 ////////////////////////////////////////////////////////////////////////
 // Style handling.
 ////////////////////////////////////////////////////////////////////////
@@ -427,6 +559,23 @@ NewGUI::setStyle (void)
 
     //puSetDefaultStyle();
 
+    
+    if (0) {
+        // Re-read gui/style/*.xml files so that one can edit them and see the
+        // results without restarting flightgear.
+        SGPath  p(globals->get_fg_root(), "gui/styles");
+        SGPropertyNode* sim_gui = globals->get_props()->getNode("sim/gui/", true);
+        simgear::Dir dir(p);
+        int i = 0;
+        for (SGPath xml: dir.children(simgear::Dir::TYPE_FILE, ".xml")) {
+            SGPropertyNode_ptr node = sim_gui->getChild("style", i, true);
+            node->removeAllChildren();
+            SG_LOG(SG_GENERAL, SG_WARN, "reading from " << xml << " into " << node->getPath());
+            readProperties(xml, node);
+            i += 1;
+        }
+    }
+    
     int which = fgGetInt("/sim/gui/current-style", 0);
     SGPropertyNode *sim = globals->get_props()->getNode("sim/gui", true);
     SGPropertyNode *n = sim->getChild("style", which);
@@ -438,7 +587,7 @@ NewGUI::setStyle (void)
     // n->copy() doesn't delete existing nodes, so need to clear them all
     // first.
     selected_style->removeAllChildren();
-    n->copy(selected_style);
+    copyProperties(n, selected_style);
 
     //if (selected_style && n)
     //    n->alias(selected_style);
@@ -448,19 +597,23 @@ NewGUI::setStyle (void)
 
     for (int i = 0; i < n->nChildren(); i++) {
         SGPropertyNode *child = n->getChild(i);
-        _colors[child->getName()] = new FGColor(child);
+        _colors[child->getNameString()] = new FGColor(child);
     }
 
     FGColor *c = _colors["background"];
+#if defined(HAVE_PUI)
     puSetDefaultColourScheme(c->red(), c->green(), c->blue(), c->alpha());
+#endif
 }
 
 
 void
 NewGUI::setupFont (SGPropertyNode *node)
 {
+#if defined(HAVE_PUI)
     _font = FGFontCache::instance()->get(node);
     puSetDefaultFonts(*_font, *_font);
+#endif
     return;
 }
 
