@@ -1,226 +1,67 @@
 // FGHIDEventInput.cxx -- handle event driven input devices via HIDAPI
 //
-// Written by James Turner
-//
-// Copyright (C) 2017, James Turner <zakalawe@mac.com>
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
+// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-FileCopyrightText: 2017 James Turner <james@flightgear.org>
 
 #include "config.h"
+#include "simgear/debug/debug_types.h"
 
 #include "FGHIDEventInput.hxx"
 
-#include <cstdlib>
-#include <cassert>
 #include <algorithm>
+#include <cassert>
+#include <cstdlib>
+#include <unordered_set>
+#include <string>
 
+#include <Main/fg_props.hxx>
 #include <hidapi/hidapi.h>
 #include <hidapi/hidparse.h>
 
-#include <simgear/structure/exception.hxx>
-#include <simgear/sg_inlines.h>
-#include <simgear/misc/strutils.hxx>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/io/lowlevel.hxx>
+#include <simgear/misc/strutils.hxx>
+#include <simgear/sg_inlines.h>
+#include <simgear/structure/exception.hxx>
 
-const char* hexTable = "0123456789ABCDEF";
+#include "FGHIDUsage.hxx"
 
-namespace HID
+using simgear::strutils::encodeHex;
+
+namespace HID {
+
+bool shouldPrefixWithAbs(uint32_t usagePage, uint32_t usage)
 {
-    enum class UsagePage
-    {
-        Undefined = 0,
-        GenericDesktop,
-        Simulation,
-        VR,
-        Sport,
-        Game,
-        GenericDevice,
-        Keyboard,
-        LEDs,
-        Button,
-        Ordinal,
-        Telephony,
-        Consumer,
-        Digitizer,
-        // reserved 0x0E
-        // PID 0x0f
-        Unicode = 0x10,
-        AlphanumericDisplay = 0x14,
-
-        VendorDefinedStart = 0xFF00
-    };
-
-    enum GenericDesktopUsage
-    {
-        // generic desktop section
-        GD_Joystick = 0x04,
-        GD_GamePad = 0x05,
-        GD_Keyboard = 0x06,
-        GD_Keypad = 0x07,
-        GD_MultiAxisController = 0x08,
-        GD_X = 0x30,
-        GD_Y,
-        GD_Z,
-        GD_Rx,
-        GD_Ry,
-        GD_Rz,
-        GD_Slider,
-        GD_Dial,
-        GD_Wheel,
-        GD_Hatswitch,
-        GD_DpadUp = 0x90,
-        GD_DpadDown,
-        GD_DpadRight,
-        GD_DpadLeft
-    };
-
-    enum LEDUsage
-    {
-        LED_Undefined = 0,
-        LED_Play = 0x36,
-        LED_Pause = 0x37,
-        LED_GenericIndicator = 0x4B
-    };
-
-    enum AlphanumericUsage
-    {
-        AD_AlphanumericDisplay = 0x01,
-        AD_BitmappedDisplay = 0x2,
-        AD_DisplayControlReport = 0x24,
-        AD_ClearDisplay = 0x25,
-        AD_CharacterReport = 0x2B,
-        AD_DisplayData = 0x2C,
-        AD_DisplayStatus = 0x2D,
-        AD_Rows = 0x35,
-        AD_Columns = 0x36,
-        AD_7SegmentDirectMap = 0x43,
-        AD_14SegmentDirectMap = 0x45,
-        AD_DisplayBrightness = 0x46,
-        AD_DisplayContrast = 0x47
-    };
-
-    enum class ReportType
-    {
-        Invalid = 0,
-        In = 0x08,
-        Out = 0x09,
-        Feature = 0x0B
-    };
-
-    std::string nameForUsage(uint32_t usagePage, uint32_t usage)
-    {
-        const auto enumUsage = static_cast<UsagePage>(usagePage);
-        if (enumUsage == UsagePage::Undefined) {
-          std::stringstream os;
-          os << "undefined-" << usage;
-          return os.str();
+    const auto enumUsage = static_cast<UsagePage>(usagePage);
+    if (enumUsage == UsagePage::GenericDesktop) {
+        switch (usage) {
+        case GD_Wheel:
+        case GD_Dial:
+        case GD_Hatswitch:
+        case GD_Slider:
+        case GD_Rx:
+        case GD_Ry:
+        case GD_Rz:
+        case GD_X:
+        case GD_Y:
+        case GD_Z:
+            return true;
+        default:
+            break;
         }
-
-        if (enumUsage == UsagePage::GenericDesktop) {
-            switch (usage) {
-                case GD_Joystick:          return "joystick";
-                case GD_Wheel:             return "wheel";
-                case GD_Dial:              return "dial";
-                case GD_Hatswitch:         return "hat";
-                case GD_Slider:            return "slider";
-                case GD_Rx:                return "x-rotate";
-                case GD_Ry:                return "y-rotate";
-                case GD_Rz:                return "z-rotate";
-                case GD_X:                 return "x-translate";
-                case GD_Y:                 return "y-translate";
-                case GD_Z:                 return "z-translate";
-                default:
-                    SG_LOG(SG_INPUT, SG_WARN, "Unhandled HID generic desktop usage:" << usage);
-            }
-        } else if (enumUsage == UsagePage::Simulation) {
-            switch (usage) {
-                default:
-                    SG_LOG(SG_INPUT, SG_WARN, "Unhandled HID simulation usage:" << usage);
-            }
-        } else if (enumUsage == UsagePage::Consumer) {
-            switch (usage) {
-                default:
-                    SG_LOG(SG_INPUT, SG_WARN, "Unhandled HID consumer usage:" << usage);
-            }
-        } else if (enumUsage == UsagePage::AlphanumericDisplay) {
-            switch (usage) {
-                case AD_AlphanumericDisplay:       return "alphanumeric";
-                case AD_CharacterReport:           return "character-report";
-                case AD_DisplayData:               return "display-data";
-                case AD_DisplayBrightness:         return "display-brightness";
-                case AD_7SegmentDirectMap:          return "seven-segment-direct";
-                case AD_14SegmentDirectMap:         return "fourteen-segment-direct";
-
-                default:
-                    SG_LOG(SG_INPUT, SG_WARN, "Unhandled HID alphanumeric usage:" << usage);
-            }
-        } else if (enumUsage == UsagePage::LEDs) {
-            switch (usage) {
-                case LED_GenericIndicator:         return "led-misc";
-                case LED_Pause:                    return "led-pause";
-                default:
-                    SG_LOG(SG_INPUT, SG_WARN, "Unhandled HID LED usage:" << usage);
-
-            }
-        } else if (enumUsage == UsagePage::Button) {
-            std::stringstream os;
-            os << "button-" << usage;
-            return os.str();
-        } else if (enumUsage >= UsagePage::VendorDefinedStart) {
-            return "vendor";
-        } else {
-            SG_LOG(SG_INPUT, SG_WARN, "Unhandled HID usage page:" << std::hex << usagePage
-                   << " with usage " << std::hex << usage);
-        }
-
-        return "unknown";
     }
 
-    bool shouldPrefixWithAbs(uint32_t usagePage, uint32_t usage)
-    {
-        const auto enumUsage = static_cast<UsagePage>(usagePage);
-        if (enumUsage == UsagePage::GenericDesktop) {
-            switch (usage) {
-            case GD_Wheel:
-            case GD_Dial:
-            case GD_Hatswitch:
-            case GD_Slider:
-            case GD_Rx:
-            case GD_Ry:
-            case GD_Rz:
-            case GD_X:
-            case GD_Y:
-            case GD_Z:
-                return true;
-            default:
-                break;
-            }
-        }
+    return false;
+}
 
-        return false;
-    }
-
-    ReportType reportTypeFromString(const std::string& s)
-    {
-        if (s == "input") return ReportType::In;
-        if (s == "output") return ReportType::Out;
-        if (s == "feature") return ReportType::Feature;
-        return ReportType::Invalid;
-    }
-} // of namespace
+ReportType reportTypeFromString(const std::string& s)
+{
+    if (s == "input") return ReportType::In;
+    if (s == "output") return ReportType::Out;
+    if (s == "feature") return ReportType::Feature;
+    return ReportType::Invalid;
+}
+} // namespace HID
 
 class FGHIDEventInput::FGHIDEventInputPrivate
 {
@@ -231,13 +72,13 @@ public:
 };
 
 // anonymous namespace to define our device subclass
-namespace
-{
+namespace {
 
-class FGHIDDevice : public FGInputDevice {
+class FGHIDDevice : public FGInputDevice
+{
 public:
     FGHIDDevice(hid_device_info* devInfo,
-        FGHIDEventInput* subsys);
+                FGHIDEventInput* subsys);
 
     virtual ~FGHIDDevice();
 
@@ -246,18 +87,19 @@ public:
     void Configure(SGPropertyNode_ptr node) override;
 
     void update(double dt) override;
-    const char *TranslateEventName(FGEventData &eventData) override;
-    void Send( const char * eventName, double value ) override;
-    void SendFeatureReport(unsigned int reportId, const std::string& data) override;
+    const char* TranslateEventName(FGEventData& eventData) override;
+    void Send(const char* eventName, double value) override;
+    void SendFeatureReport(unsigned int reportId, const simgear::UInt8Vector& data) override;
+    void SendOutputReport(unsigned int reportId, const simgear::UInt8Vector& data) override;
 
     class Item
     {
     public:
-        Item(const std::string& n, uint32_t offset, uint8_t size) :
-            name(n),
-            bitOffset(offset),
-            bitSize(size)
-        {}
+        Item(const std::string& n, uint32_t offset, uint8_t size) : name(n),
+                                                                    bitOffset(offset),
+                                                                    bitSize(size)
+        {
+        }
 
         std::string name;
         uint32_t bitOffset = 0; // form the start of the report
@@ -270,6 +112,7 @@ public:
         // hopefully this doesn't need to be a list
         FGInputEvent_ptr event;
     };
+
 private:
     class Report
     {
@@ -308,6 +151,8 @@ private:
 
     void defineReport(SGPropertyNode_ptr reportNode);
 
+    void dumpRawBytes(const std::string& b) const;
+
     std::vector<Report*> _reports;
     std::string _hidPath;
     hid_device* _device = nullptr;
@@ -320,7 +165,7 @@ private:
     bool _haveLocalDescriptor = false;
 
     /// allow specifying the descriptor as hex bytes in XML
-    std::vector<uint8_t>_rawXMLDescriptor;
+    simgear::UInt8Vector _rawXMLDescriptor;
 
     // all sets which will be send on the next update() call.
     std::set<Report*> _dirtyReports;
@@ -330,9 +175,8 @@ class HIDEventData : public FGEventData
 {
 public:
     // item, value, dt, keyModifiers
-    HIDEventData(FGHIDDevice::Item* it, int value, double dt, int keyMods) :
-        FGEventData(value, dt, keyMods),
-        item(it)
+    HIDEventData(FGHIDDevice::Item* it, int value, double dt, int keyMods) : FGEventData(value, dt, keyMods),
+                                                                             item(it)
     {
         assert(item);
     }
@@ -340,8 +184,9 @@ public:
     FGHIDDevice::Item* item = nullptr;
 };
 
-FGHIDDevice::FGHIDDevice(hid_device_info *devInfo, FGHIDEventInput *)
+FGHIDDevice::FGHIDDevice(hid_device_info* devInfo, FGHIDEventInput*)
 {
+    class_id = "FGHIDDevice";
     _hidPath = devInfo->path;
 
     std::wstring manufacturerName, productName;
@@ -351,7 +196,7 @@ FGHIDDevice::FGHIDDevice(hid_device_info *devInfo, FGHIDEventInput *)
     if (devInfo->manufacturer_string) {
         manufacturerName = std::wstring(devInfo->manufacturer_string);
         SetName(simgear::strutils::convertWStringToUtf8(manufacturerName) + " " +
-            simgear::strutils::convertWStringToUtf8(productName));
+                simgear::strutils::convertWStringToUtf8(productName));
     } else {
         SetName(simgear::strutils::convertWStringToUtf8(productName));
     }
@@ -363,7 +208,15 @@ FGHIDDevice::FGHIDDevice(hid_device_info *devInfo, FGHIDEventInput *)
         SetSerialNumber(simgear::strutils::convertWStringToUtf8(serial));
     }
 
-    SG_LOG(SG_INPUT, SG_DEBUG, "HID device:" << GetName() << " at path " << _hidPath);
+    std::string _usage = HID::nameForUsage(devInfo->usage_page, devInfo->usage);
+    SG_LOG(SG_INPUT, SG_DEBUG, "HID device " << _hidPath << " "
+        << "0x" << std::hex << devInfo->vendor_id << ":0x" << std::hex << devInfo->product_id << " "
+        << "release " << devInfo->release_number << " "
+        << "usage " << _usage << "(0x" << std::hex << devInfo->usage_page 
+        << ":0x" << std::hex << devInfo->usage << ") "
+        << "ifn " << devInfo->interface_number << " "
+        << GetName() );
+
 }
 
 FGHIDDevice::~FGHIDDevice()
@@ -403,6 +256,7 @@ void FGHIDDevice::Configure(SGPropertyNode_ptr node)
 
 bool FGHIDDevice::Open()
 {
+    SG_LOG(SG_INPUT, SG_INFO, "HID open " << GetUniqueName());
     _device = hid_open_path(_hidPath.c_str());
     if (_device == nullptr) {
         SG_LOG(SG_INPUT, SG_WARN, GetUniqueName() << ": HID: Failed to open:" << _hidPath);
@@ -410,18 +264,16 @@ bool FGHIDDevice::Open()
         return false;
     }
 
-#if !defined(SG_WINDOWS)
     if (_rawXMLDescriptor.empty()) {
         _rawXMLDescriptor.resize(2048);
-        int descriptorSize = hid_get_descriptor(_device, _rawXMLDescriptor.data(), _rawXMLDescriptor.size());
+        int descriptorSize = hid_get_report_descriptor(_device, _rawXMLDescriptor.data(), _rawXMLDescriptor.size());
         if (descriptorSize <= 0) {
-           SG_LOG(SG_INPUT, SG_WARN, "HID: " << GetUniqueName() << " failed to read HID descriptor");
-           return false;
+            SG_LOG(SG_INPUT, SG_WARN, "HID: " << GetUniqueName() << " failed to read HID descriptor");
+            return false;
         }
 
         _rawXMLDescriptor.resize(descriptorSize);
     }
-#endif
 
     if (!_haveLocalDescriptor) {
         bool ok = parseUSBHIDDescriptor();
@@ -452,24 +304,14 @@ bool FGHIDDevice::parseUSBHIDDescriptor()
 #if defined(SG_WINDOWS)
     if (_rawXMLDescriptor.empty()) {
         SG_LOG(SG_INPUT, SG_ALERT, GetUniqueName() << ": on Windows, there is no way to extract the UDB-HID report descriptor. "
-               << "\nPlease supply the report descriptor in the device XML configuration.");
+                                                   << "\nPlease supply the report descriptor in the device XML configuration.");
         SG_LOG(SG_INPUT, SG_ALERT, "See this page:<> for information on extracting the report descriptor on Windows");
         return false;
     }
 #endif
 
     if (_debugRaw) {
-        SG_LOG(SG_INPUT, SG_INFO, "\nHID: descriptor for:" << GetUniqueName());
-        {
-            std::ostringstream byteString;
-
-            for (auto i=0; i<_rawXMLDescriptor.size(); ++i) {
-                byteString << hexTable[_rawXMLDescriptor[i] >> 4];
-                byteString << hexTable[_rawXMLDescriptor[i] & 0x0f];
-                byteString << " ";
-            }
-            SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << byteString.str());
-        }
+        SG_LOG(SG_INPUT, SG_INFO, "\nHID: descriptor for:" << GetUniqueName() << "\n\t" << encodeHex(_rawXMLDescriptor, ':'));
     }
 
     hid_item* rootItem = nullptr;
@@ -592,8 +434,7 @@ void FGHIDDevice::parseItem(hid_item* item)
     uint32_t bitOffset = report->currentBitSize();
 
     if (debugEvents) {
-        SG_LOG(SG_INPUT, SG_INFO, GetUniqueName() << ": add:" << name << ", bits: " << bitOffset << ":" << (int) item->report_size
-            << ", report=" << (int) item->report_id);
+        SG_LOG(SG_INPUT, SG_INFO, GetUniqueName() << ": add:" << name << ", bits: " << bitOffset << ":" << (int)item->report_size << ", report=" << (int)item->report_id);
     }
 
     Item* itemObject = new Item{name, bitOffset, item->report_size};
@@ -629,11 +470,10 @@ void FGHIDDevice::update(double dt)
         const uint8_t reportNumber = _haveNumberedReports ? reportBuf[0] : 0;
         auto inputReport = getReport(HID::ReportType::In, reportNumber, false);
         if (!inputReport) {
-            SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: Unknown input report number:" <<
-                static_cast<int>(reportNumber));
+            SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: Unknown input report number:" << static_cast<int>(reportNumber));
         } else {
             uint8_t* reportBytes = _haveNumberedReports ? reportBuf + 1 : reportBuf;
-            size_t reportSize = _haveNumberedReports ? readCount -  1 : readCount;
+            size_t reportSize = _haveNumberedReports ? readCount - 1 : readCount;
             processInputReport(inputReport, reportBytes, reportSize, dt, modifiers);
         }
     }
@@ -658,7 +498,7 @@ void FGHIDDevice::sendReport(Report* report) const
     memset(reportBytes, 0, sizeof(reportBytes));
     reportBytes[0] = report->number;
 
-// fill in valid data
+    // fill in valid data
     for (auto item : report->items) {
         reportLength += item->bitSize;
         if (item->lastValue == 0) {
@@ -671,17 +511,10 @@ void FGHIDDevice::sendReport(Report* report) const
     reportLength /= 8;
 
     if (_debugRaw) {
-        std::ostringstream byteString;
-        for (size_t i=0; i<reportLength; ++i) {
-            byteString << hexTable[reportBytes[i] >> 4];
-            byteString << hexTable[reportBytes[i] & 0x0f];
-            byteString << " ";
-        }
-        SG_LOG(SG_INPUT, SG_INFO, "sending bytes: " << byteString.str());
+        SG_LOG(SG_INPUT, SG_INFO, "sending bytes: " << encodeHex(reportBytes, ':'));
     }
 
-
-// send the data, based on the report type
+    // send the data, based on the report type
     if (report->type == HID::ReportType::Feature) {
         hid_send_feature_report(_device, reportBytes, reportLength + 1);
     } else {
@@ -700,15 +533,9 @@ void FGHIDDevice::processInputReport(Report* report, unsigned char* data,
                                      double dt, int keyModifiers)
 {
     if (_debugRaw) {
-        SG_LOG(SG_INPUT, SG_INFO, GetName() << " FGHIDDeivce received input report:" << (int) report->number << ", len=" << length);
+        SG_LOG(SG_INPUT, SG_INFO, GetName() << " FGHIDDeivce received input report:" << (int)report->number << ", len=" << length);
         {
-            std::ostringstream byteString;
-            for (size_t i=0; i<length; ++i) {
-                byteString << hexTable[data[i] >> 4];
-                byteString << hexTable[data[i] & 0x0f];
-                byteString << " ";
-            }
-            SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << byteString.str());
+            SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << encodeHex(data, length, ':'));
         }
     }
 
@@ -719,12 +546,12 @@ void FGHIDDevice::processInputReport(Report* report, unsigned char* data,
 
         // suppress events for values that aren't changing
         if (item->isRelative) {
-            // supress spurious 0-valued relative events
+            // suppress spurious 0-valued relative events
             if (value == 0) {
                 continue;
             }
         } else {
-            // supress no-change events for absolute items
+            // suppress no-change events for absolute items
             if (value == item->lastValue) {
                 continue;
             }
@@ -743,45 +570,38 @@ void FGHIDDevice::processInputReport(Report* report, unsigned char* data,
     }
 }
 
-void FGHIDDevice::SendFeatureReport(unsigned int reportId, const std::string& data)
+void FGHIDDevice::SendFeatureReport(unsigned int reportId, const simgear::UInt8Vector& data)
 {
     if (!_device) {
         return;
     }
 
     if (_debugRaw) {
-        SG_LOG(SG_INPUT, SG_INFO, GetName() << ": FGHIDDevice: Sending feature report:" << (int) reportId << ", len=" << data.size());
-        {
-            std::ostringstream byteString;
-
-            for (unsigned int i=0; i<data.size(); ++i) {
-                byteString << hexTable[data[i] >> 4];
-                byteString << hexTable[data[i] & 0x0f];
-                byteString << " ";
-            }
-            SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << byteString.str());
-        }
+        SG_LOG(SG_INPUT, SG_INFO, GetName() << ": FGHIDDevice: Sending feature report:" << (int)reportId << ", len=" << data.size());
+        SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << encodeHex(data, ':'));
     }
 
     uint8_t buf[65];
-    size_t len = std::min(data.length() + 1, sizeof(buf));
+    size_t len = std::min(data.size() + 1, sizeof(buf));
     buf[0] = reportId;
     memcpy(buf + 1, data.data(), len - 1);
     size_t r = hid_send_feature_report(_device, buf, len);
     if (r < 0) {
         SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: Sending feature report failed, error-string is:\n"
-               << simgear::strutils::error_string(errno));
+                                            << simgear::strutils::error_string(errno));
     }
 }
 
-const char *FGHIDDevice::TranslateEventName(FGEventData &eventData)
+const char* FGHIDDevice::TranslateEventName(FGEventData& eventData)
 {
     HIDEventData& hidEvent = static_cast<HIDEventData&>(eventData);
     return hidEvent.item->name.c_str();
 }
 
-void FGHIDDevice::Send(const char *eventName, double value)
+void FGHIDDevice::Send(const char* eventName, double value)
 {
+    // even though this called 'Send' it's really 'mark value for sending when we update', to
+    // avoid generating multiple output reports in a single update frame.
     auto item = itemWithName(eventName);
     if (item.second == nullptr) {
         SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice:unknown item name:" << eventName);
@@ -793,19 +613,51 @@ void FGHIDDevice::Send(const char *eventName, double value)
         return; // not actually changing
     }
 
+    lastEventName->setStringValue(eventName);
+    lastEventValue->setDoubleValue(value);
+
     // update the stored value prior to sending
     item.second->lastValue = intValue;
     _dirtyReports.insert(item.first);
 }
 
+void FGHIDDevice::SendOutputReport(unsigned int reportId, const simgear::UInt8Vector& data)
+{
+    if (!_device) {
+        return;
+    }
+
+    auto r = getReport(HID::ReportType::Out, reportId);
+    if (!r) {
+        SG_LOG(SG_INPUT, SG_DEV_ALERT, "HID device does not define an output report with ID:" << reportId);
+        return;
+    }
+
+
+    if (_debugRaw) {
+        SG_LOG(SG_INPUT, SG_INFO, GetName() << ": FGHIDDevice: output feature:" << (int)reportId << ", len=" << data.size());
+        SG_LOG(SG_INPUT, SG_INFO, "\tbytes: " << encodeHex(data, ':'));
+    }
+
+    uint8_t buf[65];
+    size_t len = std::min(data.size() + 1, sizeof(buf));
+    buf[0] = reportId;
+    memcpy(buf + 1, data.data(), len - 1);
+
+    auto result = hid_write(_device, buf, len);
+    if (result < 0) {
+        SG_LOG(SG_INPUT, SG_DEV_ALERT, GetName() << ": FGHIDDevice: Sending outpit report failed, error-string is:\n"
+                                                 << simgear::strutils::error_string(errno));
+    }
+}
+
 void FGHIDDevice::defineReport(SGPropertyNode_ptr reportNode)
 {
     const int nChildren = reportNode->nChildren();
     uint32_t bitCount = 0;
     const auto rty = HID::reportTypeFromString(reportNode->getStringValue("type"));
     if (rty == HID::ReportType::Invalid) {
-        SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: invalid report type:" <<
-               reportNode->getStringValue("type"));
+        SG_LOG(SG_INPUT, SG_WARN, GetName() << ": FGHIDDevice: invalid report type:" << reportNode->getStringValue("type"));
         return;
     }
 
@@ -817,29 +669,29 @@ void FGHIDDevice::defineReport(SGPropertyNode_ptr reportNode)
     auto report = new Report(rty, id);
     _reports.push_back(report);
 
-    for (int c=0; c < nChildren; ++c) {
-        const auto nd = reportNode->getChild(c);
-        const int size = nd->getIntValue("size", 1); // default to a single bit
-        if (!strcmp(nd->getName(), "unused-bits")) {
+    for (int c = 0; c < nChildren; ++c) {
+        const auto n = reportNode->getChild(c);
+        const int size = n->getIntValue("size", 1); // default to a single bit
+        if (n->getNameString() == "unused-bits") {
             bitCount += size;
             continue;
         }
 
-        if (!strcmp(nd->getName(), "type") || !strcmp(nd->getName(), "id")) {
+        if (n->getNameString() == "type" || n->getNameString() == "id") {
             continue; // already handled above
         }
 
         // allow repeating items
-        uint8_t count = nd->getIntValue("count", 1);
-        std::string name = nd->getNameString();
+        uint8_t count = n->getIntValue("count", 1);
+        std::string name = n->getNameString();
         const auto lastHypen = name.rfind("-");
         std::string baseName = name.substr(0, lastHypen + 1);
         int baseIndex = std::stoi(name.substr(lastHypen + 1));
 
         const bool isRelative = (name.find("rel-") == 0);
-        const bool isSigned = nd->getBoolValue("is-signed", false);
+        const bool isSigned = n->getBoolValue("is-signed", false);
 
-        for (uint8_t i=0; i < count; ++i) {
+        for (uint8_t i = 0; i < count; ++i) {
             std::ostringstream oss;
             oss << baseName << (baseIndex + i);
             Item* itemObject = new Item{oss.str(), bitCount, static_cast<uint8_t>(size)};
@@ -852,7 +704,7 @@ void FGHIDDevice::defineReport(SGPropertyNode_ptr reportNode)
 }
 
 
-} // of anonymous namespace
+} // namespace
 
 
 int extractBits(uint8_t* bytes, size_t lengthInBytes, size_t bitOffset, size_t bitSize)
@@ -864,7 +716,7 @@ int extractBits(uint8_t* bytes, size_t lengthInBytes, size_t bitOffset, size_t b
     const size_t bytesToCopy = std::min(sizeof(uint32_t), (offsetInByte + bitSize + 7) / 8);
     uint32_t v = 0;
     // this goes from byte alignment to word alignment safely
-    memcpy((void*) &v, bytes + wholeBytesToSkip, bytesToCopy);
+    memcpy((void*)&v, bytes + wholeBytesToSkip, bytesToCopy);
 
     // shift down so lowest bit is aligned
     v = v >> offsetInByte;
@@ -898,9 +750,8 @@ void writeBits(uint8_t* bytes, size_t bitOffset, size_t bitSize, int value)
     }
 }
 
-FGHIDEventInput::FGHIDEventInput() :
-    FGEventInput(),
-    d(new FGHIDEventInputPrivate)
+FGHIDEventInput::FGHIDEventInput() : FGEventInput("Input/HID", "/input/hid"),
+                                     d(new FGHIDEventInputPrivate)
 {
     d->p = this; // store back pointer to outer object on pimpl
 }
@@ -909,13 +760,6 @@ FGHIDEventInput::~FGHIDEventInput()
 {
 }
 
-void FGHIDEventInput::init()
-{
-    FGEventInput::init();
-    // have to wait until postinit since loading config files
-    // requires Nasal to be running
-}
-
 void FGHIDEventInput::reinit()
 {
     SG_LOG(SG_INPUT, SG_INFO, "Re-Initializing HID input bindings");
@@ -932,10 +776,22 @@ void FGHIDEventInput::postinit()
 
     hid_device_info* devices = hid_enumerate(0 /* vendor ID */, 0 /* product ID */);
 
+    // open each device path only once, it may appear multiple times with different usage value
+    std::unordered_set<std::string> seenPaths;    
     for (hid_device_info* curDev = devices; curDev != nullptr; curDev = curDev->next) {
-        d->evaluateDevice(curDev);
+        if (curDev->path) {
+            std::string pathStr(curDev->path);
+            if (seenPaths.find(pathStr) == seenPaths.end()) {
+                seenPaths.insert(pathStr);
+                d->evaluateDevice(curDev);
+            } else {
+                std::string _usage = HID::nameForUsage(curDev->usage_page, curDev->usage);
+                SG_LOG(SG_INPUT, SG_DEBUG, "Skipping duplicate path " << pathStr << " "
+                    << "usage " << _usage << "(0x" << std::hex << curDev->usage_page << ":0x"
+                    << std::hex << curDev->usage << ")");
+            }            
+        }
     }
-
     hid_free_enumeration(devices);
 }
 
