@@ -50,10 +50,10 @@ AirspeedIndicator::init ()
     std::string branch;
     branch = "/instrumentation/" + _name;
 
-    SGPropertyNode *node = fgGetNode(branch.c_str(), _num, true );
+    SGPropertyNode *node = fgGetNode(branch, _num, true );
     _serviceable_node = node->getChild("serviceable", 0, true);
-    _total_pressure_node = fgGetNode(_total_pressure.c_str(), true);
-    _static_pressure_node = fgGetNode(_static_pressure.c_str(), true);
+    _total_pressure_node = fgGetNode(_total_pressure, true);
+    _static_pressure_node = fgGetNode(_static_pressure, true);
     _density_node = fgGetNode("/environment/density-slugft3", true);
     _speed_node = node->getChild("indicated-speed-kt", 0, true);
     _tas_node = node->getChild("true-speed-kt", 0, true);
@@ -78,10 +78,10 @@ AirspeedIndicator::init ()
         }
 
         _airspeed_limit = node->getChild("airspeed-limit-kt", 0, true);
-        _pressure_alt = fgGetNode(_pressure_alt_source.c_str(), true);
+        _pressure_alt = fgGetNode(_pressure_alt_source, true);
     }
     
-    _environmentManager = (FGEnvironmentMgr*) globals->get_subsystem("environment");
+    _environmentManager = globals->get_subsystem<FGEnvironmentMgr>();
 }
 
 void
@@ -137,9 +137,9 @@ AirspeedIndicator::computeMach()
     return;
   }
   
-  FGEnvironment env(_environmentManager->getEnvironment());
+    const auto env = _environmentManager->getAircraftEnvironment();
   
-  double oatK = env.get_temperature_degc() + SG_T0_K - 15.0 ;         // OAT in Kelvin
+  double oatK = env->get_temperature_degc() + SG_T0_K - 15.0 ;         // OAT in Kelvin
   oatK = std::max( oatK , 0.001 );                                // should never happen, but just in case someone flies into space...
   double c = sqrt(SG_gamma * SG_R_m2_p_s2_p_K * oatK);                 // speed-of-sound in m/s at aircraft position
   double pt = _total_pressure_node->getDoubleValue() * SG_INHG_TO_PA;  // total pressure in Pa
