@@ -3,9 +3,7 @@
 //
 // This file is in the Public Domain and comes with no warranty.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include "gps.hxx"
 
@@ -28,7 +26,7 @@
 #include "Airports/runways.hxx"
 #include "Autopilot/route_mgr.hxx"
 
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/sg_inlines.h>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/structure/exception.hxx>
@@ -43,16 +41,19 @@ static const char* makeTTWString(double TTW)
   if ((TTW <= 0.0) || (TTW >= 356400.5)) { // 99 hours
     return "--:--:--";
   }
-      
-  unsigned int TTW_seconds = (int) (TTW + 0.5);
-  unsigned int TTW_minutes = 0;
-  unsigned int TTW_hours   = 0;
-  static char TTW_str[9];
+
+  unsigned TTW_seconds = (unsigned) (TTW + 0.5);
+  unsigned TTW_minutes = 0;
+  unsigned TTW_hours   = 0;
+
   TTW_hours   = TTW_seconds / 3600;
   TTW_minutes = (TTW_seconds / 60) % 60;
   TTW_seconds = TTW_seconds % 60;
-  snprintf(TTW_str, 9, "%02d:%02d:%02d",
+
+  static char TTW_str[16];
+  snprintf(TTW_str, 16, "%02u:%02u:%02u",
     TTW_hours, TTW_minutes, TTW_seconds);
+
   return TTW_str;
 }
 
@@ -88,6 +89,14 @@ void GPS::Config::bind(GPS* aOwner, SGPropertyNode* aCfg)
   aOwner->tie(aCfg, "over-flight-arm-distance-nm", SGRawValuePointer<double>(&_overflightArmDistance));
   aOwner->tie(aCfg, "over-flight-arm-angle-deg", SGRawValuePointer<double>(&_overflightArmAngle));
   aOwner->tie(aCfg, "delegate-sequencing", SGRawValuePointer<bool>(&_delegateSequencing));
+  aOwner->tie(aCfg, "max-fly-by-turn-angle-deg", SGRawValueMethods<GPS, double>(*aOwner, &GPS::maxFlyByTurnAngleDeg, &GPS::setFlyByMaxTurnAngle));
+}
+
+void GPS::setFlyByMaxTurnAngle(double maxAngle)
+{
+    _config.setMaxFlyByTurnAngle(maxAngle);
+    // keep the FlightPlan in sync, so RoutePath matches
+    _route->setMaxFlyByTurnAngle(maxAngle);
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -107,7 +116,7 @@ GPS::GPS ( SGPropertyNode *node, bool defaultGPSMode) :
                                                         fgGetNode("/autopilot/route-manager/active", true)))
 {
   string branch = "/instrumentation/" + _name;
-  _gpsNode = fgGetNode(branch.c_str(), _num, true );
+  _gpsNode = fgGetNode(branch, _num, true );
   _scratchNode = _gpsNode->getChild("scratch", 0, true);
   
   SGPropertyNode *wp_node = _gpsNode->getChild("wp", 0, true);
@@ -144,7 +153,7 @@ GPS::init ()
 // waypoints
   // for compatibility, alias selected course down to wp/wp[1]/desired-course-deg
   SGPropertyNode* wp1Crs = _currentWayptNode->getChild("desired-course-deg", 0, true);
-  wp1Crs->alias(_gpsNode->getChild("desired-course-deg", 0, true));
+  wp1Crs->alias(_gpsNode->getChild("desired-course-deg", 0, true), true);
 
   _tracking_bug_node = _gpsNode->getChild("tracking-bug", 0, true);
     
@@ -156,12 +165,12 @@ GPS::init ()
     
 // navradio slaving properties
   SGPropertyNode* toFlag = _gpsNode->getChild("to-flag", 0, true);
-  toFlag->alias(_currentWayptNode->getChild("to-flag"));
-  
+  toFlag->alias(_currentWayptNode->getChild("to-flag"), true);
+
   SGPropertyNode* fromFlag = _gpsNode->getChild("from-flag", 0, true);
-  fromFlag->alias(_currentWayptNode->getChild("from-flag"));
-    
-// autopilot drive properties
+  fromFlag->alias(_currentWayptNode->getChild("from-flag"), true);
+
+  // autopilot drive properties
   _apDrivingFlag = fgGetNode("/autopilot/settings/gps-driving-true-heading", true);
   _apTrueHeading = fgGetNode("/autopilot/settings/true-heading-deg",true);
 
@@ -261,13 +270,25 @@ GPS::bind()
 // navradio slaving properties  
   tie(_gpsNode, "cdi-deflection", SGRawValueMethods<GPS,double>
     (*this, &GPS::getCDIDeflection));
+
+    _currentWpLatNode = _currentWayptNode->getNode("latitude-deg", true);
+    _currentWpLonNode = _currentWayptNode->getNode("longitude-deg", true);
+    _currentWpAltNode = _currentWayptNode->getNode("altitude-ft", true);
 }
 
 void
 GPS::unbind()
 {
-  _tiedProperties.Untie();
-  _gpsNode.clear();
+    // avoid shutdown crash during Untie(), due to invoking the getters to capture the
+    // current value(s)
+    _currentWaypt = _prevWaypt = nullptr;
+
+    _tiedProperties.Untie();
+    _gpsNode.clear();
+    _currentWayptNode.clear();
+    _currentWpLatNode.clear();
+    _currentWpLonNode.clear();
+    _currentWpAltNode.clear();
 }
 
 void
@@ -320,11 +341,7 @@ GPS::update (double delta_time_sec)
   if (_dataValid) {
     if (_wayptController.get()) {
       _wayptController->update(delta_time_sec);
-      SGGeod p(_wayptController->position());
-      _currentWayptNode->setDoubleValue("longitude-deg", p.getLongitudeDeg());
-      _currentWayptNode->setDoubleValue("latitude-deg", p.getLatitudeDeg());
-      _currentWayptNode->setDoubleValue("altitude-ft", p.getElevationFt());
-      
+        updateCurrentWpNode(_wayptController->position());
       _desiredCourse = getLegMagCourse();
       
       _gpsNode->setStringValue("rnav-controller-status", _wayptController->status());
@@ -343,7 +360,7 @@ GPS::update (double delta_time_sec)
   if (_dataValid && (_mode == "init")) {
     // will select LEG mode if the route is active
     routeManagerFlightPlanChanged(nullptr);
-    FGRouteMgr* routeMgr = globals->get_subsystem<FGRouteMgr>();
+    auto routeMgr = globals->get_subsystem<FGRouteMgr>();
       
     if (!routeMgr || !routeMgr->isRouteActive()) {
       // initialise in OBS mode, with waypt set to the nearest airport.
@@ -373,6 +390,7 @@ void GPS::shutdown()
         _route->removeDelegate(this);
         _route = nullptr;
     }
+    _wayptController.reset();
 }
 
 void GPS::routeManagerFlightPlanChanged(SGPropertyNode*)
@@ -382,7 +400,7 @@ void GPS::routeManagerFlightPlanChanged(SGPropertyNode*)
   }
   
   SG_LOG(SG_INSTR, SG_DEBUG, "GPS saw route-manager flight-plan replaced.");
-  FGRouteMgr* routeMgr = globals->get_subsystem<FGRouteMgr>();
+  auto routeMgr = globals->get_subsystem<FGRouteMgr>();
   if (!routeMgr) {
     return;
   }
@@ -474,6 +492,11 @@ double GPS::selectedMagCourse()
   return _selectedCourse;
 }
 
+double GPS::maxFlyByTurnAngleDeg() const
+{
+    return _config.maxFlyByTurnAngleDeg();
+}
+
 simgear::optional<double> GPS::nextLegTrack()
 {
     auto next = _route->nextLeg();
@@ -797,15 +820,19 @@ void GPS::wp1Changed()
     _legDistanceNm = _wayptController->distanceToWayptM() * SG_METER_TO_NM;
     
     // synchronise these properties immediately
-    SGGeod p(_wayptController->position());
-    _currentWayptNode->setDoubleValue("longitude-deg", p.getLongitudeDeg());
-    _currentWayptNode->setDoubleValue("latitude-deg", p.getLatitudeDeg());
-    _currentWayptNode->setDoubleValue("altitude-ft", p.getElevationFt());
+      updateCurrentWpNode(_wayptController->position());
     
     _desiredCourse = getLegMagCourse();
   }
 }
 
+void GPS::updateCurrentWpNode(const SGGeod& p)
+{
+    _currentWpLatNode->setDoubleValue(p.getLatitudeDeg());
+    _currentWpLonNode->setDoubleValue(p.getLongitudeDeg());
+    _currentWpAltNode->setDoubleValue(p.getElevationFt());
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // property getter/setters
 
@@ -1427,7 +1454,7 @@ void GPS::addAirportToScratch(FGAirport* aAirport)
     for (unsigned int r=0; r<aAirport->numRunways(); ++r) {
         SGPropertyNode* rwyNd = _scratchNode->getChild("runways", r, true);
         FGRunway* rwy = aAirport->getRunwayByIndex(r);
-        // TODO - filter out unsuitable runways in the future
+        // TODO: filter out unsuitable runways in the future
         // based on config again
         
         rwyNd->setStringValue("id", rwy->ident().c_str());
@@ -1435,7 +1462,7 @@ void GPS::addAirportToScratch(FGAirport* aAirport)
         rwyNd->setIntValue("width-ft", rwy->widthFt());
         rwyNd->setIntValue("heading-deg", rwy->headingDeg());
         // map surface code to a string
-        // TODO - lighting information
+        // TODO: lighting information
         
         if (rwy->ILS()) {
             rwyNd->setDoubleValue("ils-frequency-mhz", rwy->ILS()->get_freq() / 100.0);
@@ -1495,7 +1522,7 @@ void GPS::defineWaypoint()
     }
     
     SG_LOG(SG_INSTR, SG_INFO, "GPS:defineWaypoint: creating waypoint:" << ident);
-    FGPositionedRef wpt = FGPositioned::createUserWaypoint(ident, _scratchPos);
+    FGPositionedRef wpt = FGPositioned::createWaypoint(FGPositioned::WAYPOINT, ident, _scratchPos, false);
     _searchResults.clear();
     _searchResults.push_back(wpt);
     setScratchFromPositioned(wpt.get(), -1);
