@@ -492,10 +492,11 @@ private:
           const SGPath dir(fgGetString("/sim/aircraft-dir"));
           if (dir.file() != expectedDirNode->getStringValue()) {
               flightgear::fatalMessageBoxThenExit("Aircraft folder named incorrectly",
-                                          "The folder of the selected aircraft must be named '" + expectedDir +
-                                          "' (instead of '" + dir.file() + "') to work correctly. If you downloaded it yourself, " +
-                                          "please ensure the folder is called '" +
-                                          expectedDir + "' and re-name if necessary.");
+                                                  "The folder of the selected aircraft must be named '" + expectedDir +
+                                                      "' (instead of '" + dir.file() + "') to work correctly. If you downloaded it yourself, " +
+                                                      "please ensure the folder is called '" +
+                                                      expectedDir + "' and re-name if necessary.",
+                                                  {}, EXIT_FAILURE, false /* don't report to Sentry */);
               return false;
           }
       } else {
@@ -547,7 +548,7 @@ SGPath fgHomePath()
     return SGPath::fromEnv("FG_HOME", platformDefaultDataPath());
 }
 
-InitHomeResult fgInitHome()
+InitHomeResult fgInitHome(int argc, char** argv)
 {
   SGPath dataPath = fgHomePath();
   globals->set_fg_home(dataPath);
@@ -564,10 +565,12 @@ InitHomeResult fgInitHome()
           dataPath.utf8Str() + "'.");
         return InitHomeAbort;
     }
-    
-    if (fgGetBool("/sim/fghome-readonly", false)) {
+
+    const auto explicitReadOnly = flightgear::Options::checkForArgEnable(argc, argv, "read-only");
+    if (explicitReadOnly) {
         // user / config forced us into readonly mode, fine
-        SG_LOG(SG_GENERAL, SG_INFO, "Running with FG_HOME readonly");
+        SG_LOG(SG_GENERAL, SG_MANDATORY_INFO, "Running with FG_HOME readonly");
+        fgSetBool("/sim/fghome-readonly", true);
         return InitHomeExplicitReadOnly;
     }
 
@@ -1300,7 +1303,7 @@ void fgStartNewReset()
 {
     flightgear::updateSentryTag("have-reset", "yes");
 
-    // save user settings now, so that USERARCIVE-d values changes since the
+    // save user settings now, so that USERARCHIVE-d values changes since the
     // last init are recorded and hence re-loaded when we fgInitConfig down
     // later in this function. Otherwise all such settings are lost.
     globals->saveUserSettings();
