@@ -42,7 +42,7 @@ using namespace std;
 using namespace simgear;
 
 // text color
-#if defined(__linux__) || defined(__sun) || defined(__CYGWIN__) || defined( __FreeBSD__ ) || defined ( sgi )
+#if defined(__linux__) || defined(__sun) || defined(__CYGWIN__) || defined( __FreeBSD__ ) || defined( __OpenBSD__ ) || defined ( sgi )
 #	define R "\033[31;1m"		// red
 #	define G "\033[32;1m"		// green
 #	define Y "\033[33;1m"		// yellow
@@ -121,206 +121,7 @@ ostream& operator<<(ostream& s, const SGMetarVisibility& v)
 
 void printReport(SGMetar *m)
 {
-#define NaN SGMetarNaN
-	const char *s;
-	char buf[256];
-	double d;
-	int i, lineno;
-
-	if ((i = m->getReportType()) == SGMetar::AUTO)
-		s = "\t\t(automatically generated)";
-	else if (i == SGMetar::COR)
-		s = "\t\t(manually corrected)";
-	else if (i == SGMetar::RTD)
-		s = "\t\t(routine delayed)";
-	else
-		s = "";
-
-	cout << "METAR Report" << s << endl;
-	cout << "============" << endl;
-	cout << "Airport-Id:\t\t" << m->getId() << endl;
-
-
-	// date/time
-	int year = m->getYear();
-	int month = m->getMonth();
-	cout << "Report time:\t\t" << year << '/' << month << '/' << m->getDay();
-	cout << ' ' << m->getHour() << ':';
-	cout << setw(2) << setfill('0') << m->getMinute() << " UTC" << endl;
-
-
-	// visibility
-	SGMetarVisibility minvis = m->getMinVisibility();
-	SGMetarVisibility maxvis = m->getMaxVisibility();
-	double min = minvis.getVisibility_m();
-	double max = maxvis.getVisibility_m();
-	if (min != NaN) {
-		if (max != NaN) {
-			cout << "min. Visibility:\t" << minvis << endl;
-			cout << "max. Visibility:\t" << maxvis << endl;
-		} else
-			cout << "Visibility:\t\t" << minvis << endl;
-	}
-
-
-	// directed visibility
-	const SGMetarVisibility *dirvis = m->getDirVisibility();
-	for (i = 0; i < 8; i++, dirvis++)
-		if (dirvis->getVisibility_m() != NaN)
-			cout << "\t\t\t" << *dirvis << endl;
-
-
-	// vertical visibility
-	SGMetarVisibility vertvis = m->getVertVisibility();
-	if ((d = vertvis.getVisibility_ft()) != NaN)
-		cout << "Vert. visibility:\t" << vertvis << endl;
-	else if (vertvis.getModifier() == SGMetarVisibility::NOGO)
-		cout << "Vert. visibility:\timpossible to determine" << endl;
-
-
-	// wind
-	d = m->getWindSpeed_kmh();
-	cout << "Wind:\t\t\t";
-	if (d < .1)
-		cout << "none" << endl;
-	else {
-		if ((i = m->getWindDir()) == -1)
-			cout << "from variable directions";
-		else
-			cout << "from the " << azimuthName(i) << " (" << i << "°)";
-		cout << " at " << rnd(d, -1) << " km/h";
-
-		cout << "\t\t" << rnd(m->getWindSpeed_kt(), -1) << " kt";
-		cout << " = " << rnd(m->getWindSpeed_mph(), -1) << " mph";
-		cout << " = " << rnd(m->getWindSpeed_mps(), -1) << " m/s";
-		cout << endl;
-
-		if ((d = m->getGustSpeed_kmh()) != NaN) {
-			cout << "\t\t\twith gusts at " << rnd(d, -1) << " km/h";
-			cout << "\t\t\t" << rnd(m->getGustSpeed_kt(), -1) << " kt";
-			cout << " = " << rnd(m->getGustSpeed_mph(), -1) << " mph";
-			cout << " = " << rnd(m->getGustSpeed_mps(), -1) << " m/s";
-			cout << endl;
-		}
-
-		int from = m->getWindRangeFrom();
-		int to = m->getWindRangeTo();
-		if (from != to) {
-			cout << "\t\t\tvariable from " << azimuthName(from);
-			cout << " to " << azimuthName(to);
-			cout << " (" << from << "°--" << to << "°)" << endl;
-		}
-	}
-
-
-	// temperature/humidity/air pressure
-	if ((d = m->getTemperature_C()) != NaN) {
-		cout << "Temperature:\t\t" << d << "°C\t\t\t\t\t";
-		cout << rnd(m->getTemperature_F(), -1) << "°F" << endl;
-
-		if ((d = m->getDewpoint_C()) != NaN) {
-			cout << "Dewpoint:\t\t" << d << "°C\t\t\t\t\t";
-			cout << rnd(m->getDewpoint_F(), -1) << "°F"  << endl;
-			cout << "Rel. Humidity:\t\t" << rnd(m->getRelHumidity()) << "%" << endl;
-		}
-	}
-	if ((d = m->getPressure_hPa()) != NaN) {
-		cout << "Pressure:\t\t" << rnd(d) << " hPa\t\t\t\t";
-		cout << rnd(m->getPressure_inHg(), -2) << " in. Hg" << endl;
-	}
-
-
-	// weather phenomena
-	vector<string> wv = m->getWeather();
-	vector<string>::iterator weather;
-	for (i = 0, weather = wv.begin(); weather != wv.end(); weather++, i++) {
-		cout << (i ? ", " : "Weather:\t\t") << weather->c_str();
-	}
-	if (i)
-		cout << endl;
-
-
-	// cloud layers
-	const char *coverage_string[5] = {
-		"clear skies", "few clouds", "scattered clouds", "broken clouds", "sky overcast"
-	};
-	vector<SGMetarCloud> cv = m->getClouds();
-	vector<SGMetarCloud>::iterator cloud;
-	for (lineno = 0, cloud = cv.begin(); cloud != cv.end(); cloud++, lineno++) {
-		cout << (lineno ? "\t\t\t" : "Sky condition:\t\t");
-
-		if ((i = cloud->getCoverage()) != -1)
-			cout << coverage_string[i];
-		if ((d = cloud->getAltitude_ft()) != NaN)
-			cout << " at " << rnd(d, 1) << " ft";
-		if ((s = cloud->getTypeLongString()))
-			cout << " (" << s << ')';
-		if (d != NaN)
-			cout << "\t\t\t" << rnd(cloud->getAltitude_m(), 1) << " m";
-		cout << endl;
-	}
-
-
-	// runways
-	map<string, SGMetarRunway> rm = m->getRunways();
-	map<string, SGMetarRunway>::iterator runway;
-	for (runway = rm.begin(); runway != rm.end(); runway++) {
-		lineno = 0;
-		if (!strcmp(runway->first.c_str(), "ALL"))
-			cout << "All runways:\t\t";
-		else
-			cout << "Runway " << runway->first << ":\t\t";
-		SGMetarRunway rwy = runway->second;
-
-		// assemble surface string
-		vector<string> surface;
-		if ((s = rwy.getDepositString()) && strlen(s))
-			surface.push_back(s);
-		if ((s = rwy.getExtentString()) && strlen(s))
-			surface.push_back(s);
-		if ((d = rwy.getDepth()) != NaN) {
-			sprintf(buf, "%.1lf mm", d * 1000.0);
-			surface.push_back(buf);
-		}
-		if ((s = rwy.getFrictionString()) && strlen(s))
-			surface.push_back(s);
-		if ((d = rwy.getFriction()) != NaN) {
-			sprintf(buf, "friction: %.2lf", d);
-			surface.push_back(buf);
-		}
-
-		if (! surface.empty()) {
-			vector<string>::iterator rwysurf = surface.begin();
-			for (i = 0; rwysurf != surface.end(); rwysurf++, i++) {
-				if (i)
-					cout << ", ";
-				cout << *rwysurf;
-			}
-			lineno++;
-		}
-
-		// assemble visibility string
-		SGMetarVisibility minvis = rwy.getMinVisibility();
-		SGMetarVisibility maxvis = rwy.getMaxVisibility();
-		if ((d = minvis.getVisibility_m()) != NaN) {
-			if (lineno++)
-				cout << endl << "\t\t\t";
-			cout << minvis;
-		}
-		if (maxvis.getVisibility_m() != d) {
-			cout << endl << "\t\t\t" << maxvis << endl;
-			lineno++;
-		}
-
-		if (rwy.getWindShear()) {
-			if (lineno++)
-				cout << endl << "\t\t\t";
-			cout << "critical wind shear" << endl;
-		}
-		cout << endl;
-	}
-	cout << endl;
-#undef NaN
+    cout << m->getDescription(0);
 }
 
 
@@ -481,13 +282,14 @@ void usage()
 		"       -c|--command-line    print command line\n"
 		"       -e E|--elevation E   set airport elevation to E meters\n"
 		"                            (added to cloud bases in command line mode)\n"
+		"       -s|--string <METAR string>\n"
 		"Environment:\n"
 		"       http_proxy           set proxy in the form \"http://host:port/\"\n"
 		"\n"
 		"Examples:\n"
 		"       $ metar ksfo koak\n"
 		"       $ metar -c ksfo -r ksfo\n"
-		"       $ metar \"LOWL 161500Z 19004KT 160V240 9999 FEW035 SCT300 29/23 Q1006 NOSIG\"\n"
+		"       $ metar -s \"LOWL 161500Z 19004KT 160V240 9999 FEW035 SCT300 29/23 Q1006 NOSIG\"\n"
 		"       $ fgfs  `metar -e 183 -c loww`\n"
 		"       $ http_proxy=http://localhost:3128/ metar ksfo\n"
 		"\n"
@@ -528,7 +330,17 @@ int main(int argc, char *argv[])
 				return 1;
 			}
 			elevation = strtod(argv[i], 0);
-		} else {
+        }
+        else if (!strcmp(argv[i], "-s") || !strcmp(argv[i], "--string")) {
+            if (++i >= argc) {
+                cerr << "-s option used with out string\n";
+                return 1;
+            }
+            const char* string = argv[i];
+            SGMetar metar(string);
+            printReport(&metar);
+        }
+		else {
 			static bool shown = false;
 			if (verbose && !shown) {
 				cerr << "Proxy host: '" << proxy_host << "'" << endl;
@@ -573,10 +385,10 @@ int main(int argc, char *argv[])
 				//SGMetar *m = new SGMetar("2004/01/11 01:20\nLOWG 110120Z AUTO VRB01KT 0050 1600N R35/0600 FG M06/M06 Q1019 88//////\n");
 
 				if (verbose) {
-					cerr << G "INPUT: " << m->getData() << "" N << endl;
+					cerr << G "INPUT: " << m->getDataString() << "" N << endl;
 
-					const char *unused = m->getUnusedData();
-					if (*unused)
+					const auto unused = m->getUnparsedData();
+					if (!unused.empty())
 						cerr << R "UNUSED: " << unused << "" N << endl;
 				}
 
