@@ -18,12 +18,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+#include "config.h"
+
 #include "HTTPClient.hxx"
 
 #include <cassert>
 
 #include <Main/fg_props.hxx>
-#include <Include/version.h>
 
 #include <simgear/sg_inlines.h>
 
@@ -47,10 +48,10 @@ typedef nasal::Ghost<pkg::PackageRef> NasalPackage;
 typedef nasal::Ghost<pkg::CatalogRef> NasalCatalog;
 typedef nasal::Ghost<pkg::InstallRef> NasalInstall;
 
-static const char* OFFICIAL_CATALOG_ID = "org.flightgear.fgaddon.stable_2020";
+static const char* OFFICIAL_CATALOG_ID = "org.flightgear.fgaddon.stable_2024";
 
 // fallback URL is used when looking up a version-specific catalog fails
-static const char* FALLBACK_CATALOG_URL = "http://mirrors.ibiblio.org/flightgear/ftp/Aircraft-2020/catalog.xml";
+static const char* FALLBACK_CATALOG_URL = "https://mirrors.ibiblio.org/flightgear/ftp/Aircraft-2024/catalog.xml";
 
 namespace {
 
@@ -71,7 +72,7 @@ namespace {
     std::string _getDefaultCatalogUrl()
     {
         return fgGetString("/sim/package-system/default-catalog/url",
-                    "http://mirrors.ibiblio.org/flightgear/ftp/Aircraft-2020/catalog.xml");
+                           "https://mirrors.ibiblio.org/flightgear/ftp/Aircraft-2024/catalog.xml");
     }
 } // of anonymous namespace
 
@@ -85,6 +86,19 @@ FGHTTPClient::~FGHTTPClient()
 {
 }
 
+FGHTTPClient* FGHTTPClient::getOrCreate()
+{
+    auto ext = globals->get_subsystem<FGHTTPClient>();
+    if (ext) {
+        return ext;
+    }
+
+    ext = globals->get_subsystem_mgr()->add<FGHTTPClient>();
+    ext->init();
+    return ext;
+}
+
+
 void FGHTTPClient::init()
 {
     // launcher may need to setup HTTP access abnormally early, so
@@ -107,12 +121,6 @@ void FGHTTPClient::init()
   if (packageRoot) {
     // package system needs access to the HTTP engine too
     packageRoot->setHTTPClient(_http.get());
-
-    // start a refresh now
-    // setting 'force' true to work around the problem where a slightly stale
-    // catalog exists, but aircraft are modified - this causes an MD5 sum
-    // mismatch
-    packageRoot->refresh(true);
   }
 
     _inited = true;
@@ -209,6 +217,21 @@ static naRef f_catalog_search(pkg::Catalog& cat, const nasal::CallContext& ctx)
     return ctx.to_nasal(result);
 }
 
+static naRef f_catalog_packages(pkg::Catalog& cat, const nasal::CallContext& ctx)
+{
+    // TODO: support package types
+    pkg::PackageList result = cat.packages();
+    return ctx.to_nasal(result);
+}
+
+
+static naRef f_catalog_installedPackages(pkg::Catalog& cat, naContext c)
+{
+    // TODO: support package types
+    pkg::PackageList result = cat.installedPackages();
+    return nasal::to_nasal(c, result);
+}
+
 static naRef f_package_variants(pkg::Package& pack, naContext c)
 {
     nasal::Hash h(c);
@@ -236,11 +259,11 @@ void FGHTTPClient::postinit()
   .member("id", &pkg::Catalog::id)
   .member("url", &pkg::Catalog::url)
   .member("description", &pkg::Catalog::description)
-  .method("packages", &pkg::Catalog::packages)
+  .method("packages", &f_catalog_packages)
   .method("packageById", &pkg::Catalog::getPackageById)
   .method("refresh", &pkg::Catalog::refresh)
   .method("needingUpdate", &pkg::Catalog::packagesNeedingUpdate)
-  .member("installed", &pkg::Catalog::installedPackages)
+  .member("installed", &f_catalog_installedPackages)
   .method("search", &f_catalog_search)
   .member("enabled", &pkg::Catalog::isEnabled);
 
@@ -277,7 +300,7 @@ void FGHTTPClient::postinit()
   
   pkg::Root* packageRoot = globals->packageRoot();
   if (packageRoot) {
-    FGNasalSys* nasalSys = globals->get_subsystem<FGNasalSys>();
+    auto nasalSys = globals->get_subsystem<FGNasalSys>();
     nasal::Hash nasalGlobals = nasalSys->getGlobals();
     nasal::Hash nasalPkg = nasalGlobals.createHash("pkg"); // module
     nasalPkg.set("root", packageRoot);
