@@ -69,6 +69,12 @@ static naGhostType TaxiwayGhostType = { positionedGhostDestroy, "taxiway", runwa
 static const char* fixGhostGetMember(naContext c, void* g, naRef field, naRef* out);
 static naGhostType FixGhostType = { positionedGhostDestroy, "fix", fixGhostGetMember, nullptr };
 
+static const char* commGhostGetMember(naContext c, void* g, naRef field, naRef* out);
+static naGhostType CommGhostType = {positionedGhostDestroy, "comm", commGhostGetMember, nullptr};
+
+static const char* poiGhostGetMember(naContext c, void* g, naRef field, naRef* out);
+static naGhostType POIGhostType = {positionedGhostDestroy, "poi", poiGhostGetMember, nullptr};
+
 static void hashset(naContext c, naRef hash, const char* key, naRef val)
 {
   naRef s = naNewString(c);
@@ -88,8 +94,9 @@ FGPositioned* positionedGhost(naRef r)
     if ((naGhost_type(r) == &AirportGhostType) ||
         (naGhost_type(r) == &NavaidGhostType) ||
         (naGhost_type(r) == &RunwayGhostType) ||
-        (naGhost_type(r) == &FixGhostType))
-    {
+        (naGhost_type(r) == &FixGhostType) ||
+        (naGhost_type(r) == &FixGhostType) ||
+        (naGhost_type(r) == &POIGhostType)) {
         return (FGPositioned*) naGhost_ptr(r);
     }
 
@@ -131,6 +138,22 @@ static FGFix* fixGhost(naRef r)
   return 0;
 }
 
+#if 0
+static flightgear::CommStation* commGhost(naRef r)
+{
+  if (naGhost_type(r) == &CommGhostType)
+    return (flightgear::CommStation*)naGhost_ptr(r);
+  return nullptr;
+}
+#endif
+
+static POI* poiGhost(naRef r)
+{
+    if (naGhost_type(r) == &POIGhostType)
+        return (POI*)naGhost_ptr(r);
+    return nullptr;
+}
+
 
 static void positionedGhostDestroy(void* g)
 {
@@ -142,6 +165,7 @@ static void positionedGhostDestroy(void* g)
 
 static naRef airportPrototype;
 static naRef geoCoordClass;
+static naRef waypointPrototype;
 
 naRef ghostForAirport(naContext c, const FGAirport* apt)
 {
@@ -203,6 +227,26 @@ naRef ghostForFix(naContext c, const FGFix* r)
   return naNewGhost2(c, &FixGhostType, (void*) r);
 }
 
+naRef ghostForComm(naContext c, const flightgear::CommStation* comm)
+{
+  if (!c) {
+    return naNil();
+  }
+
+  FGPositioned::get(comm); // take a ref
+  return naNewGhost2(c, &CommGhostType, (void*)comm);
+}
+
+naRef ghostForPOI(naContext c, const POI* r)
+{
+    if (!r) {
+        return naNil();
+    }
+
+    FGPositioned::get(r); // take a ref
+    return naNewGhost2(c, &POIGhostType, (void*)r);
+}
+
 naRef ghostForPositioned(naContext c, FGPositionedRef pos)
 {
     if (!pos) {
@@ -222,6 +266,24 @@ naRef ghostForPositioned(naContext c, FGPositionedRef pos)
         return ghostForHelipad(c, fgpositioned_cast<FGHelipad>(pos));
     case FGPositioned::RUNWAY:
         return ghostForRunway(c, fgpositioned_cast<FGRunway>(pos));
+    case FGPositioned::FREQ_APP_DEP:
+    case FGPositioned::FREQ_AWOS:
+    case FGPositioned::FREQ_GROUND:
+    case FGPositioned::FREQ_TOWER:
+    case FGPositioned::FREQ_ATIS:
+    case FGPositioned::FREQ_CLEARANCE:
+    case FGPositioned::FREQ_UNICOM:
+    case FGPositioned::FREQ_ENROUTE:
+        return ghostForComm(c, fgpositioned_cast<flightgear::CommStation>(pos));
+
+    case FGPositioned::WAYPOINT:
+    case FGPositioned::CITY:
+    case FGPositioned::COUNTRY:
+    case FGPositioned::TOWN:
+    case FGPositioned::VISUAL_REPORTING_POINT:
+    case FGPositioned::VILLAGE:
+        return ghostForPOI(c, fgpositioned_cast<POI>(pos));
+
     default:
         SG_LOG(SG_NASAL, SG_DEV_ALERT, "Type lacks Nasal ghost mapping:" << pos->typeString());
         return naNil();
@@ -292,9 +354,13 @@ static const char* runwayGhostGetMember(naContext c, void* g, naRef field, naRef
   else if (!strcmp(fieldName, "lat")) *out = naNum(base->latitude());
   else if (!strcmp(fieldName, "lon")) *out = naNum(base->longitude());
   else if (!strcmp(fieldName, "heading")) *out = naNum(base->headingDeg());
+  else if (!strcmp(fieldName, "magnetic_heading"))
+      *out = naNum(base->magneticHeadingDeg());
   else if (!strcmp(fieldName, "length")) *out = naNum(base->lengthM());
   else if (!strcmp(fieldName, "width")) *out = naNum(base->widthM());
   else if (!strcmp(fieldName, "surface")) *out = naNum(base->surface());
+  else if (!strcmp(fieldName, "airport"))
+      *out = ghostForAirport(c, base->airport());
   else if (base->type() == FGRunwayBase::RUNWAY) {
     FGRunway* rwy = (FGRunway*) g;
     if (!strcmp(fieldName, "threshold")) *out = naNum(rwy->displacedThresholdM());
@@ -362,8 +428,16 @@ static const char* navaidGhostGetMember(naContext c, void* g, naRef field, naRef
     }
   } else if (!strcmp(fieldName, "guid")) {
       *out = naNum(nav->guid());
+  } else if (!strcmp(fieldName, "runway")) {
+      *out = ghostForRunway(c, nav->runway());
+  } else if (!strcmp(fieldName, "airport")) {
+      if (nav->runway()) {
+          *out = ghostForAirport(c, nav->runway()->airport());
+      } else {
+          *out = naNil(); // not associated with an airport / runway
+      }
   } else {
-    return 0;
+      return 0;
   }
 
   return "";
@@ -380,6 +454,8 @@ static const char* fixGhostGetMember(naContext c, void* g, naRef field, naRef* o
     // for homogenity with other values returned by navinfo()
   else if (!strcmp(fieldName, "type")) *out = stringToNasal(c, "fix");
   else if (!strcmp(fieldName, "name")) *out = stringToNasal(c, fix->ident());
+  else if (!strcmp(fieldName, "guid"))
+      *out = naNum(fix->guid());
   else {
     return 0;
   }
@@ -387,6 +463,64 @@ static const char* fixGhostGetMember(naContext c, void* g, naRef field, naRef* o
   return "";
 }
 
+static const char* commGhostGetMember(naContext c, void* g, naRef field, naRef* out)
+{
+  const char* fieldName = naStr_data(field);
+  auto comm = static_cast<flightgear::CommStation*>(g);
+
+  if (!strcmp(fieldName, "id"))
+    *out = stringToNasal(c, comm->ident());
+  else if (!strcmp(fieldName, "lat"))
+    *out = naNum(comm->latitude());
+  else if (!strcmp(fieldName, "lon"))
+    *out = naNum(comm->longitude());
+  else if (!strcmp(fieldName, "airport"))
+      *out = ghostForAirport(c, comm->airport());
+  else if (!strcmp(fieldName, "range_nm"))
+      *out = naNum(comm->rangeNm());
+  else if (!strcmp(fieldName, "type")) {
+    *out = stringToNasal(c, comm->nameForType(comm->type()));
+  } else if (!strcmp(fieldName, "name"))
+    *out = stringToNasal(c, comm->name());
+  else if (!strcmp(fieldName, "frequency")) {
+    *out = naNum(comm->freqMHz());
+  } else if (!strcmp(fieldName, "guid")) {
+      *out = naNum(comm->guid());
+  } else {
+      return 0;
+  }
+
+  return "";
+}
+
+static const char* poiGhostGetMember(naContext c, void* g, naRef field, naRef* out)
+{
+    const char* fieldName = naStr_data(field);
+    POI* wpt = (POI*)g;
+
+    if (!strcmp(fieldName, "parents")) {
+      *out = naNewVector(c);
+      naVec_append(*out, waypointPrototype);
+    } else if (!strcmp(fieldName, "id"))
+        *out = stringToNasal(c, wpt->ident());
+    else if (!strcmp(fieldName, "lat"))
+        *out = naNum(wpt->latitude());
+    else if (!strcmp(fieldName, "lon"))
+        *out = naNum(wpt->longitude());
+    else if (!strcmp(fieldName, "type"))
+        *out = stringToNasal(c, wpt->nameForType(wpt->type()));
+    else if (!strcmp(fieldName, "name"))
+        *out = stringToNasal(c, wpt->name());
+    else if (!strcmp(fieldName, "guid"))
+        *out = naNum(wpt->guid());
+    else {
+        return 0;
+    }
+
+    return "";
+}
+
+
 static bool hashIsCoord(naRef h)
 {
   naRef parents = naHash_cget(h, (char*) "parents");
@@ -455,7 +589,7 @@ bool geodFromHash(naRef ref, SGGeod& result)
 
 int geodFromArgs(naRef* args, int offset, int argc, SGGeod& result)
 {
-  if (offset >= argc) {
+  if (offset >= argc || !args) {
     return 0;
   }
 
@@ -486,6 +620,11 @@ int geodFromArgs(naRef* args, int offset, int argc, SGGeod& result)
       return 1;
     }
 
+    if (gt == &POIGhostType) {
+        result = poiGhost(args[offset])->geod();
+        return 1;
+    }
+
     auto wp = wayptGhost(args[offset]);
     if (wp) {
       result = wp->position();
@@ -676,6 +815,9 @@ static naRef f_geodinfo(naContext c, naRef me, int argc, naRef* args)
   const simgear::BVHMaterial *material;
   SGGeod geod = SGGeod::fromDegM(lon, lat, elev);
 
+  if (globals == nullptr)
+    return naNil();
+
   const auto scenery = globals->get_scenery();
   if (scenery == nullptr)
     return naNil();
@@ -704,7 +846,20 @@ static naRef f_geodinfo(naContext c, naRef me, int argc, naRef* args)
     HASHSET("load_resistance", 15, naNum(mat->get_load_resistance()));
     HASHSET("bumpiness", 9, naNum(mat->get_bumpiness()));
     HASHSET("light_coverage", 14, naNum(mat->get_light_coverage()));
+  } else if (material) {
+    matdata = naNewHash(c);
+    naRef names = naNewVector(c);
+    naVec_append(names, stringToNasal(c, ""));
+
+    HASHSET("region", 6, stringToNasal(c, ""));
+    HASHSET("names", 5, names);
+    HASHSET("solid", 5, naNum(material->get_solid()));
+    HASHSET("friction_factor", 15, naNum(material->get_friction_factor()));
+    HASHSET("rolling_friction", 16, naNum(material->get_rolling_friction()));
+    HASHSET("load_resistance", 15, naNum(material->get_load_resistance()));
+    HASHSET("bumpiness", 9, naNum(material->get_bumpiness()));
   }
+
   naVec_append(vec, matdata);
   return vec;
 #undef HASHSET
@@ -1341,13 +1496,67 @@ static naRef f_findNavaidsByFrequency(naContext c, naRef me, int argc, naRef* ar
   naRef r = naNewVector(c);
   FGNavList::TypeFilter filter(type);
   auto navs = FGNavList::findAllByFreq(freqMhz, pos, &filter);
-  for (nav_rec_ptr a : navs) {
-    naVec_append(r, ghostForNavaid(c, a.ptr()));
+  for (auto a : navs) {
+      naVec_append(r, ghostForNavaid(c, a.ptr()));
   }
 
   return r;
 }
 
+static naRef f_findCommByFrequency(naContext c, naRef me, int argc, naRef* args)
+{
+  int argOffset = 0;
+  SGGeod pos = globals->get_aircraft_position();
+  argOffset += geodFromArgs(args, 0, argc, pos);
+
+  if (!naIsNum(args[argOffset])) {
+    naRuntimeError(c, "findCommByFrequencyMhz expectes frequency (in Mhz) as arg %d", argOffset);
+  }
+
+  // initial filter is all comm types
+  FGPositioned::TypeFilter filter(FGPositioned::FREQ_GROUND, FGPositioned::FREQ_UNICOM);
+  double freqMhz = args[argOffset++].num;
+  if (argOffset < argc) {
+    // allow specifying an explicitly type by name
+    filter = FGPositioned::TypeFilter::fromString(naStr_data(args[argOffset]));
+  }
+
+  auto ref = NavDataCache::instance()->findCommByFreq(static_cast<int>(freqMhz * 1000), pos, &filter);
+  if (!ref) {
+    return naNil();
+  }
+
+  auto comm = fgpositioned_cast<flightgear::CommStation>(ref);
+  return ghostForComm(c, comm);
+}
+
+static naRef f_findCommsByFrequency(naContext c, naRef me, int argc, naRef* args)
+{
+    int argOffset = 0;
+    SGGeod pos = globals->get_aircraft_position();
+    argOffset += geodFromArgs(args, 0, argc, pos);
+
+    if (!naIsNum(args[argOffset])) {
+        naRuntimeError(c, "f_findCommsByFrequency expectes frequency (in Mhz) as arg %d", argOffset);
+    }
+
+    FGPositioned::TypeFilter filter(FGPositioned::FREQ_GROUND, FGPositioned::FREQ_UNICOM);
+    double freqMhz = args[argOffset++].num;
+    if (argOffset < argc) {
+        // allow specifying an explicitly type by name
+        filter = FGPositioned::TypeFilter::fromString(naStr_data(args[argOffset]));
+    }
+
+    naRef r = naNewVector(c);
+    auto stations = NavDataCache::instance()->findCommsByFreq(static_cast<int>(freqMhz * 1000), pos, &filter);
+    for (auto id : stations) {
+        auto sta = FGPositioned::loadById<CommStation>(id);
+        naVec_append(r, ghostForComm(c, sta.ptr()));
+    }
+
+    return r;
+}
+
 static naRef f_findNavaidsByIdent(naContext c, naRef me, int argc, naRef* args)
 {
   int argOffset = 0;
@@ -1550,6 +1759,79 @@ static naRef f_tileIndex(naContext c, naRef me, int argc, naRef* args)
   return naNum(b.gen_index());
 }
 
+static naRef f_createWaypoint(naContext c, naRef me, int argc, naRef* args)
+{
+    if ((argc < 3) || !naIsString(args[0]) || !naIsString(args[1])) {
+        naRuntimeError(c, "createWaypoint: expects type and ident as first two args");
+    }
+
+    std::string typeSpec(naStr_data(args[0]));
+    std::string ident(naStr_data(args[1]));
+    std::string name;
+
+    FGPositioned::Type wptType = FGPositioned::typeFromName(typeSpec);
+
+    SGGeod pos;
+    int argOffset = geodFromArgs(args, 2, argc, pos) + 2;
+    if ((argc > argOffset) && naIsString(args[argOffset])) {
+        name = naStr_data(args[argOffset]);
+        ++argOffset;
+    }
+
+    bool isTemporary = false;
+    if ((argc > argOffset) && naIsNum(args[argOffset])) {
+        isTemporary = (args[argOffset].num > 0);
+    }
+
+    try {
+        auto wpt = FGPositioned::createWaypoint(wptType, ident, pos, isTemporary, name);
+        if (wptType == FGPositioned::FIX) {
+            return ghostForFix(c, fgpositioned_cast<FGFix>(wpt));
+        }
+
+        auto poi = fgpositioned_cast<POI>(wpt);
+        return ghostForPOI(c, poi);
+    } catch (std::exception& e) {
+        naRuntimeError(c, "Failed to create waypoint: %s", e.what());
+    }
+
+    return naNil();
+}
+
+
+static naRef f_deleteWaypoint(naContext c, naRef me, int argc, naRef* args)
+{
+    if (argc < 1) {
+        naRuntimeError(c, "deleteWaypoint: missing argument");
+    }
+
+    FGPositioned* wpt = poiGhost(args[0]);
+    if (!wpt && naIsNum(args[0])) {
+        PositionedID guid = static_cast<PositionedID>(args[0].num);
+        wpt = NavDataCache::instance()->loadById(guid);
+        if (!POI::isType(wpt->type())) {
+          naRuntimeError(c, "deleteWaypoint: only waypoints can be deleted for now");
+        }
+    }
+
+    FGPositioned::deleteWaypoint(wpt);
+    // note wpt is dead here, don't access
+    return naNil();
+}
+
+static naRef f_waypoint_move(naContext c, naRef me, int argc, naRef* args)
+{
+  POI* wpt = poiGhost(me);
+  if (!wpt) {
+    naRuntimeError(c, "waypoint.move called on non-POI object");
+  }
+
+  SGGeod pos;
+  geodFromArgs(args, 0, argc, pos);
+
+  NavDataCache::instance()->updatePosition(wpt->guid(), pos);
+  return naNil();
+}
 
 void shutdownNasalPositioned()
 {
@@ -1584,33 +1866,39 @@ FGPositionedRef positionedFromArg(naRef ref)
 }
 
 // Table of extension functions.  Terminate with zeros.
-static struct { const char* name; naCFunction func; } funcs[] = {
-  { "carttogeod", f_carttogeod },
-  { "geodtocart", f_geodtocart },
-  { "geodinfo", f_geodinfo },
-  { "formatLatLon", f_formatLatLon },
-  { "parseStringAsLatLonValue", f_parseStringAsLatLonValue},
-  { "get_cart_ground_intersection", f_get_cart_ground_intersection },
-  { "aircraftToCart", f_aircraftToCart },
-  { "airportinfo", f_airportinfo },
-  { "findAirportsWithinRange", f_findAirportsWithinRange },
-  { "findAirportsByICAO", f_findAirportsByICAO },
-  { "navinfo", f_navinfo },
-  { "findNavaidsWithinRange", f_findNavaidsWithinRange },
-  { "findNDBByFrequencyKHz", f_findNDBByFrequency },
-  { "findNDBsByFrequencyKHz", f_findNDBsByFrequency },
-  { "findNavaidByFrequencyMHz", f_findNavaidByFrequency },
-  { "findNavaidsByFrequencyMHz", f_findNavaidsByFrequency },
-  { "findNavaidsByID", f_findNavaidsByIdent },
-  { "findFixesByID", f_findFixesByIdent },
-  { "findByIdent", f_findByIdent },
-  { "magvar", f_magvar },
-  { "courseAndDistance", f_courseAndDistance },
-  { "greatCircleMove", f_greatCircleMove },
-  { "tileIndex", f_tileIndex },
-  { "tilePath", f_tilePath },
-  { 0, 0 }
-};
+static struct {
+  const char* name;
+  naCFunction func;
+} funcs[] = {
+    {"carttogeod", f_carttogeod},
+    {"geodtocart", f_geodtocart},
+    {"geodinfo", f_geodinfo},
+    {"formatLatLon", f_formatLatLon},
+    {"parseStringAsLatLonValue", f_parseStringAsLatLonValue},
+    {"get_cart_ground_intersection", f_get_cart_ground_intersection},
+    {"aircraftToCart", f_aircraftToCart},
+    {"airportinfo", f_airportinfo},
+    {"findAirportsWithinRange", f_findAirportsWithinRange},
+    {"findAirportsByICAO", f_findAirportsByICAO},
+    {"navinfo", f_navinfo},
+    {"findNavaidsWithinRange", f_findNavaidsWithinRange},
+    {"findNDBByFrequencyKHz", f_findNDBByFrequency},
+    {"findNDBsByFrequencyKHz", f_findNDBsByFrequency},
+    {"findNavaidByFrequencyMHz", f_findNavaidByFrequency},
+    {"findNavaidsByFrequencyMHz", f_findNavaidsByFrequency},
+    {"findCommByFrequencyMHz", f_findCommByFrequency},
+    {"findCommsByFrequencyMHz", f_findCommsByFrequency},
+    {"findNavaidsByID", f_findNavaidsByIdent},
+    {"findFixesByID", f_findFixesByIdent},
+    {"findByIdent", f_findByIdent},
+    {"magvar", f_magvar},
+    {"courseAndDistance", f_courseAndDistance},
+    {"greatCircleMove", f_greatCircleMove},
+    {"tileIndex", f_tileIndex},
+    {"tilePath", f_tilePath},
+    {"createWaypoint", f_createWaypoint},
+    {"deleteWaypoint", f_deleteWaypoint},
+    {0, 0}};
 
 
 naRef initNasalPositioned(naRef globals, naContext c)
@@ -1639,6 +1927,10 @@ naRef initNasalPositioned(naRef globals, naContext c)
     hashset(c, airportPrototype, "findBestRunwayForPos", naNewFunc(c, naNewCCode(c, f_airport_findBestRunway)));
     hashset(c, airportPrototype, "tostring", naNewFunc(c, naNewCCode(c, f_airport_toString)));
 
+    waypointPrototype = naNewHash(c);
+    naSave(c, waypointPrototype);
+    hashset(c, waypointPrototype, "move", naNewFunc(c, naNewCCode(c, f_waypoint_move)));  
+
     for(int i=0; funcs[i].name; i++) {
       hashset(c, globals, funcs[i].name,
       naNewFunc(c, naNewCCode(c, funcs[i].func)));
