@@ -21,6 +21,7 @@
 //
 // $Id$
 
+#include "simgear/structure/exception.hxx"
 #ifdef _MSC_VER
 #pragma warning (disable: 4786)
 #endif
@@ -31,14 +32,16 @@
 
 #include "fg_fx.hxx"
 
-#include <algorithm>
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
+#include <Main/sentryIntegration.hxx>
 #include <Sound/soundmanager.hxx>
+#include <algorithm>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/misc/sg_path.hxx>
 #include <simgear/sound/xmlsound.hxx>
 
 FGFX::FGFX ( const std::string &refname, SGPropertyNode *props ) :
@@ -66,6 +69,9 @@ FGFX::FGFX ( const std::string &refname, SGPropertyNode *props ) :
     _atc_volume = _props->getNode("sim/sound/atc/volume", true);
     _atc_ext = _props->getNode("sim/sound/atc/external-view", true);
 
+    _machwave_active = _props->getNode("sim/sound/machwave/active", true);
+    _machwave_volume = _props->getNode("sim/sound/machwave/offset-m", true);
+
     _smgr = globals->get_subsystem<FGSoundManager>();
     if (!_smgr) {
       return;
@@ -85,21 +91,16 @@ FGFX::FGFX ( const std::string &refname, SGPropertyNode *props ) :
     }
 }
 
-void FGFX::unbind()
+void FGFX::shutdown()
 {
-    if (_smgr)
-    {
-        _smgr->remove(_refname);
-    }
-
-    // because SGXmlSound has an owning ref back to us, we need to
-    // clear these here, or we will never get destroyed
-    std::for_each(_sound.begin(), _sound.end(), [](const SGXmlSound* snd) { delete snd; });
-    _sound.clear();
+    _smgr->remove(_refname);
+    _xmlSounds.clear();
 }
 
 FGFX::~FGFX ()
 {
+    // check that shutdown has been called
+    assert(!_smgr->exists(_refname));
 }
 
 void
@@ -120,54 +121,47 @@ FGFX::init()
     SGPath path = globals->resolve_aircraft_path(path_str);
     if (path.isNull())
     {
+        simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AudioFX,
+                               "Failed to find FX XML file:" + path_str, sg_location{path_str});
         SG_LOG(SG_SOUND, SG_ALERT,
                "File not found: '" << path_str);
         return;
     }
-    SG_LOG(SG_SOUND, SG_INFO, "Reading sound " << node->getName()
+    SG_LOG(SG_SOUND, SG_INFO, "Reading sound " << node->getNameString()
            << " from " << path);
 
     SGPropertyNode root;
     try {
         readProperties(path, &root);
-    } catch (const sg_exception &) {
-        SG_LOG(SG_SOUND, SG_ALERT,
-               "Error reading file '" << path << '\'');
+    } catch (const sg_exception& e) {
+        simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AudioFX,
+                               "Failure loading FX XML:" + e.getFormattedMessage(), e.getLocation());
         return;
     }
 
     node = root.getNode("fx");
     if(node) {
         for (int i = 0; i < node->nChildren(); ++i) {
-            std::unique_ptr<SGXmlSound> soundfx{new SGXmlSound};
-  
+            SGXmlSoundRef soundfx{new SGXmlSound};
+            auto child = node->getChild(i);
             try {
-                bool ok = soundfx->init( _props, node->getChild(i), this, _avionics,
-                               path.dir() );
+                bool ok = soundfx->init(_props, child, this, _avionics, path.dir());
                 if (ok) {
-                    // take the pointer out of the unique ptr so it's not deleted
-                    _sound.push_back( soundfx.release() );
+                    _xmlSounds.push_back(soundfx);
                 }
-            } catch ( sg_exception &e ) {
-                SG_LOG(SG_SOUND, SG_ALERT, e.getFormattedMessage());
+            } catch (const sg_exception& e) {
+                simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AudioFX,
+                                       "Failure creating Audio FX:" + e.getFormattedMessage(), sg_location{child});
             }
         }
     }
 }
 
-
-void
-FGFX::reinit()
-{
-    std::for_each(_sound.begin(), _sound.end(), [](const SGXmlSound* snd) { delete snd; });
-    _sound.clear();
-    init();
-}
-
-
 void
 FGFX::update (double dt)
 {
+    // FGFX::update is called via the SGSoundManager update(), which
+    // calls update on all its sample-groups.
     if (!_smgr) {
         return;
     }
@@ -175,8 +169,8 @@ FGFX::update (double dt)
     if (!_active && _smgr->is_active())
     {
         _active = true;
-        for ( unsigned int i = 0; i < _sound.size(); i++ ) {
-            _sound[i]->start();
+        for (auto s : _xmlSounds) {
+            s->start();
         }
     }
 
@@ -206,12 +200,15 @@ FGFX::update (double dt)
                 _atc->suspend();
         }
 
+        _machwave_active->setBoolValue(_mInCone);
+        _machwave_volume->setFloatValue(_mOffset_m);
+
         set_volume( _volume->getDoubleValue() );
         resume();
 
         // update sound effects if not paused
-        for ( unsigned int i = 0; i < _sound.size(); i++ ) {
-            _sound[i]->update(dt);
+        for (auto xs : _xmlSounds) {
+            xs->update(dt);
         }
 
         SGSampleGroup::update(dt);
