@@ -1,24 +1,9 @@
-// electrical.cxx - a flexible, generic electrical system model.
-//
-// Written by Curtis Olson, started September 2002.
-//
-// Copyright (C) 2002  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileName: electrical.cxx
+ * SPDX-FileComment: a flexible, generic electrical system model
+ * SPDX-FileCopyrightText: Copyright (C) 2002  Curtis L. Olson  - http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -26,6 +11,7 @@
 
 #include <cstdlib>
 #include <cstring>
+#include <algorithm>
 
 #include <simgear/structure/exception.hxx>
 #include <simgear/misc/sg_path.hxx>
@@ -65,7 +51,7 @@ FGElectricalSupplier::FGElectricalSupplier ( SGPropertyNode *node ) {
 
     // cout << "Creating a supplier" << endl;
     name = node->getStringValue("name");
-    string _model = node->getStringValue("kind");
+    std::string _model = node->getStringValue("kind");
     // cout << "_model = " << _model << endl;
     if ( _model == "battery" ) {
         model = FG_BATTERY;
@@ -89,8 +75,8 @@ FGElectricalSupplier::FGElectricalSupplier ( SGPropertyNode *node ) {
     for ( i = 0; i < node->nChildren(); ++i ) {
         SGPropertyNode *child = node->getChild(i);
         // cout << " scanning: " << child->getName() << endl;
-        if ( !strcmp(child->getName(), "prop") ) {
-            string prop = child->getStringValue();
+        if ( child->getNameString() == "prop" ) {
+            std::string prop = child->getStringValue();
             // cout << "  Adding prop = " << prop << endl;
             add_prop( prop );
             fgSetFloat( prop.c_str(), ideal_amps );
@@ -208,8 +194,8 @@ FGElectricalBus::FGElectricalBus ( SGPropertyNode *node ) {
     int i;
     for ( i = 0; i < node->nChildren(); ++i ) {
         SGPropertyNode *child = node->getChild(i);
-        if ( !strcmp(child->getName(), "prop") ) {
-            string prop = child->getStringValue();
+        if ( child->getNameString() == "prop" ) {
+            std::string prop = child->getStringValue();
             add_prop( prop );
         }
     }
@@ -230,8 +216,8 @@ FGElectricalOutput::FGElectricalOutput ( SGPropertyNode *node ) {
     int i;
     for ( i = 0; i < node->nChildren(); ++i ) {
         SGPropertyNode *child = node->getChild(i);
-        if ( !strcmp(child->getName(), "prop") ) {
-            string prop = child->getStringValue();
+        if ( child->getNameString() == "prop" ) {
+            std::string prop = child->getStringValue();
             add_prop( prop );
         }
     }
@@ -247,8 +233,8 @@ FGElectricalSwitch::FGElectricalSwitch( SGPropertyNode *node ) :
     int i;
     for ( i = 0; i < node->nChildren(); ++i ) {
         SGPropertyNode *child = node->getChild(i);
-        string cname = child->getName();
-        string cval = child->getStringValue();
+        std::string cname = child->getNameString();
+        std::string cval = child->getStringValue();
         if ( cname == "prop" ) {
             switch_node = fgGetNode( cval.c_str(), true );
             // cout << "switch node = " << cval << endl;
@@ -276,8 +262,8 @@ FGElectricalConnector::FGElectricalConnector ( SGPropertyNode *node,
     int i;
     for ( i = 0; i < node->nChildren(); ++i ) {
         SGPropertyNode *child = node->getChild(i);
-        string cname = child->getName();
-        string cval = child->getStringValue();
+        std::string cname = child->getNameString();
+        std::string cval = child->getStringValue();
         // cout << "  " << cname << " = " << cval << endl;
         if ( cname == "input" ) {
             FGElectricalComponent *s = es->find( child->getStringValue() );
@@ -359,7 +345,9 @@ FGElectricalSystem::FGElectricalSystem ( SGPropertyNode *node ) :
 }
 
 
-FGElectricalSystem::~FGElectricalSystem () {
+FGElectricalSystem::~FGElectricalSystem()
+{
+    SG_LOG(SG_SYSTEMS, SG_INFO, "Destroying elec system");
 }
 
 
@@ -388,6 +376,11 @@ void FGElectricalSystem::init () {
 
     if ( path.length() ) {
         SGPath config = globals->resolve_aircraft_path(path);
+        if (!config.exists()) {
+            SG_LOG( SG_SYSTEMS, SG_ALERT,  "Failed to find electrical system model: " << config );
+            return;
+        }
+
         // load an obsolete xml configuration
         SG_LOG( SG_SYSTEMS, SG_DEV_WARN,
                 "Reading deprecated xml electrical system model from\n    "
@@ -426,8 +419,27 @@ void FGElectricalSystem::bind ()
 void FGElectricalSystem::unbind ()
 {
     _serviceable_node.reset();
+    _volts_out.reset();
+    _amps_out.reset();
 }
 
+void FGElectricalSystem::deleteComponents(comp_list& comps)
+{
+    std::for_each(comps.begin(), comps.end(),
+                  [](FGElectricalComponent* comp) {
+                      delete comp;
+                  });
+
+    comps.clear();
+}
+
+void FGElectricalSystem::shutdown()
+{
+    deleteComponents(suppliers);
+    deleteComponents(buses);
+    deleteComponents(outputs);
+    deleteComponents(connectors);
+}
 
 void FGElectricalSystem::update (double dt)
 {
@@ -573,7 +585,7 @@ bool FGElectricalSystem::build (SGPropertyNode* config_props) {
     int count = config_props->nChildren();
     for ( i = 0; i < count; ++i ) {
         node = config_props->getChild(i);
-        string name = node->getName();
+        std::string name = node->getNameString();
         // cout << name << endl;
         if ( name == "supplier" ) {
             FGElectricalSupplier *s =
@@ -606,7 +618,7 @@ bool FGElectricalSystem::build (SGPropertyNode* config_props) {
 // total current drawn by the children of this node.
 float FGElectricalSystem::propagate( FGElectricalComponent *node, double dt,
                                      float input_volts, float input_amps,
-                                     string s ) {
+                                     std::string s ) {
     s += " ";
 
     float total_load = 0.0;
@@ -698,9 +710,9 @@ float FGElectricalSystem::propagate( FGElectricalComponent *node, double dt,
 
 
 // search for the named component and return a pointer to it, NULL otherwise
-FGElectricalComponent *FGElectricalSystem::find ( const string &name ) {
+FGElectricalComponent *FGElectricalSystem::find ( const std::string &name ) {
     unsigned int i;
-    string s;
+    std::string s;
 
     // search suppliers
     for ( i = 0; i < suppliers.size(); ++i ) {
