@@ -53,12 +53,13 @@
 #include <algorithm>
 
 #include <simgear/compiler.h>
-#include <simgear/misc/sg_path.hxx>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_dir.hxx>
+#include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
-#include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/structure/exception.hxx>
+#include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/threads/SGThread.hxx>
 #include <simgear/timing/sg_time.hxx>
 
@@ -90,7 +91,7 @@ using std::vector;
 class ScheduleParseThread : public SGThread, public XMLVisitor
 {
 public:
-  ScheduleParseThread(FGTrafficManager* traffic) :
+    explicit ScheduleParseThread(FGTrafficManager* traffic) :
     _trafficManager(traffic),
     _isFinished(false),
     _cancelThread(false),
@@ -130,7 +131,6 @@ public:
 
   void run() override
   {
-      flightgear::sentryThreadReportXMLErrors(false);
       for (const auto& p : _trafficDirPaths) {
           parseTrafficDir(p);
           if (_cancelThread) {
@@ -227,15 +227,15 @@ public:
         else if (!strcmp(name, "flight")) {
             // We have loaded and parsed all the information belonging to this flight
             // so we temporarily store it.
-            //cerr << "Pusing back flight " << callsign << endl;
+            //cerr << "Pushing back flight " << callsign << endl;
             //cerr << callsign  <<  " " << fltrules     << " "<< departurePort << " " <<  arrivalPort << " "
             //   << cruiseAlt <<  " " << departureTime<< " "<< arrivalTime   << " " << repeat << endl;
 
             //Prioritize aircraft
-            string apt = fgGetString("/sim/presets/airport-id");
-            //cerr << "Airport information: " << apt << " " << departurePort << " " << arrivalPort << endl;
-            //if (departurePort == apt) score++;
-            //flights.push_back(new FGScheduledFlight(callsign,
+            // string apt = fgGetString("/sim/presets/airport-id");
+            // cerr << "Airport information: " << apt << " " << departurePort << " " << arrivalPort << endl;
+            // if (departurePort == apt) score++;
+            // flights.push_back(new FGScheduledFlight(callsign,
             //                                fltrules,
             //                                departurePort,
             //                                arrivalPort,
@@ -243,6 +243,7 @@ public:
             //                                departureTime,
             //                                arrivalTime,
             //                                repeat));
+
             if (requiredAircraft == "") {
                 char buffer[16];
                 snprintf(buffer, 16, "%d", acCounter);
@@ -255,7 +256,7 @@ public:
                    << cruiseAlt << " "
                    << departureTime << " "
                    << arrivalTime << " " << repeat << " " << requiredAircraft);
-            // For database maintainance purposes, it may be convenient to
+            // For database maintenance purposes, it may be convenient to
             //
             if (fgGetBool("/sim/traffic-manager/dumpdata") == true) {
                 SG_LOG(SG_AI, SG_ALERT, "Traffic Dump FLIGHT," << callsign << ","
@@ -322,7 +323,7 @@ private:
 
         if (!FGAISchedule::validModelPath(mdl)) {
             missingModels.insert(mdl);
-            SG_LOG(SG_AI, SG_DEV_WARN, "TrafficMgr: Missing model path:" << mdl);
+            simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AITrafficSchedule, "Missing traffic model path:" + mdl, _currentFile);
             requiredAircraft = homePort = "";
             return;
         }
@@ -350,8 +351,8 @@ private:
             homePort = departurePort;
         }
 
-        // caution, modifying the scheduled aircraft strucutre from the
-        // 'wrong' thread. This is safe becuase FGTrafficManager won't touch
+        // caution, modifying the scheduled aircraft structure from the
+        // 'wrong' thread. This is safe because FGTrafficManager won't touch
         // the structure while we exist.
         _trafficManager->scheduledAircraft.push_back(new FGAISchedule(mdl,
                                                      livery,
@@ -379,18 +380,22 @@ private:
         simgear::Dir trafficDir(path);
         simgear::PathList d = trafficDir.children(simgear::Dir::TYPE_DIR | simgear::Dir::NO_DOT_OR_DOTDOT);
 
+        simgear::ErrorReportContext("ai-traffic-dir", path.utf8Str());
+
         for (const auto& p : d) {
             simgear::Dir d2(p);
             SG_LOG(SG_AI, SG_DEBUG, "parsing traffic in:" << p);
             simgear::PathList trafficFiles = d2.children(simgear::Dir::TYPE_FILE, ".xml");
             for (const auto& xml : trafficFiles) {
+                _currentFile = xml;
                 try {
                     readXML(xml, *this);
                     if (_cancelThread) {
                         return;
                     }
                 } catch (sg_exception& e) {
-                    SG_LOG(SG_AI, SG_WARN, "XML error parsing traffic file:" << xml << "\n\t" << e.getFormattedMessage());
+                    simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AITrafficSchedule,
+                                           "XML errors parsing traffic:" + e.getFormattedMessage(), xml);
                 }
             }
         } // of sub-directories iteration
@@ -403,21 +408,22 @@ private:
   bool _isFinished;
   bool _cancelThread;
   simgear::PathList _trafficDirPaths;
+  SGPath _currentFile;
 
-// parser state
+  // parser state
 
-    string_list elementValueStack;
-    // record model paths which are missing, to avoid duplicate
-    // warnings when parsing traffic schedules.
-    std::set<std::string> missingModels;
+  string_list elementValueStack;
+  // record model paths which are missing, to avoid duplicate
+  // warnings when parsing traffic schedules.
+  std::set<std::string> missingModels;
 
-    std::string mdl, livery, registration, callsign, fltrules,
-    port, timeString, departurePort, departureTime, arrivalPort, arrivalTime,
-    repeat, acType, airline, m_class, flighttype, requiredAircraft, homePort;
-    int cruiseAlt;
-    int score, acCounter;
-    double radius, offset;
-    bool heavy;
+  std::string mdl, livery, registration, callsign, fltrules,
+      port, timeString, departurePort, departureTime, arrivalPort, arrivalTime,
+      repeat, acType, airline, m_class, flighttype, requiredAircraft, homePort;
+  int cruiseAlt;
+  int score, acCounter;
+  double radius, offset;
+  bool heavy;
 
 };
 
@@ -473,7 +479,7 @@ void FGTrafficManager::shutdown()
             // Note: Intuitively, this doesn't make sense, but I do need to create the full file path first
             // before creating the directories. The SimGear fgpath code has changed so that it first chops off
             // the trailing dir separator and then determines the directory part of the file path by searching
-            // for the last dir separator. Effecively, this causes a full element of the directory tree to be
+            // for the last dir separator. Effectively, this causes a full element of the directory tree to be
             // skipped.
             SG_LOG(SG_GENERAL, SG_DEBUG, "Trying to create dir for : " << cacheData);
             if (!cacheData.exists()) {
@@ -529,10 +535,8 @@ bool FGTrafficManager::doDataSync()
         }
 
         trafficSyncRequested = false;
-        return true;
-
-        SG_LOG(SG_AI, SG_INFO, "Traffic files sync complete");
     }
+
     return true;
 }
 
@@ -581,7 +585,6 @@ void FGTrafficManager::init()
     } else {
         fgSetBool("/sim/traffic-manager/heuristics", false);
         SGPath path = string(fgGetString("/sim/traffic-manager/datafile"));
-        string ext = path.extension();
         if (path.extension() == "xml") {
             if (path.exists()) {
                 // use a SchedulerParser to parse, but run it in this thread,
@@ -610,7 +613,7 @@ void FGTrafficManager::finishInit()
     assert(doingInit);
     SG_LOG(SG_AI, SG_INFO, "finishing AI-Traffic init");
     loadHeuristics();
-    PerformanceDB* perfDB = globals->get_subsystem<PerformanceDB>();
+    auto perfDB = globals->get_subsystem<PerformanceDB>();
     // Do sorting and scoring separately, to take advantage of the "homeport" variable
     for (auto schedule : scheduledAircraft) {
         schedule->setScore();
@@ -619,8 +622,7 @@ void FGTrafficManager::finishInit()
         }
     }
 
-    sort(scheduledAircraft.begin(), scheduledAircraft.end(),
-         compareSchedules);
+    sort(scheduledAircraft.begin(), scheduledAircraft.end(), FGAISchedule::compareSchedules);
     currAircraft = scheduledAircraft.begin();
     currAircraftClosest = scheduledAircraft.begin();
 
@@ -647,8 +649,8 @@ void FGTrafficManager::loadHeuristics()
                  airport[0], airport[1], airport[2]);
       cacheData.append(buffer);
       cacheData.append(airport + "-cache.txt");
-      string revisionStr;
       if (cacheData.exists()) {
+        string revisionStr;
         sg_ifstream data(cacheData);
         data >> revisionStr;
         if (revisionStr != "[TrafficManagerCachedata:ref:2011:09:04]") {
@@ -763,7 +765,7 @@ void FGTrafficManager::update(double dt)
     //cerr << "Processing << " << (*currAircraft)->getRegistration() << " with score " << (*currAircraft)->getScore() << endl;
     if ((*currAircraft)->update(now, userCart)) {
         // schedule is done - process another aircraft in next iteration
-        currAircraft++;
+        ++currAircraft;
     }
 }
 
@@ -783,9 +785,7 @@ void FGTrafficManager::readTimeTableFromFile(SGPath infileName)
     double offset;
 
     char buffer[256];
-    string buffString;
     vector <string> tokens, depTime,arrTime;
-    vector <string>::iterator it;
 
     sg_ifstream infile(infileName);
     while (1) {
@@ -794,7 +794,7 @@ void FGTrafficManager::readTimeTableFromFile(SGPath infileName)
              break;
          }
          //cerr << "Read line : " << buffer << endl;
-         buffString = string(buffer);
+         string buffString = string(buffer);
          tokens.clear();
          Tokenize(buffString, tokens, " \t");
          //for (it = tokens.begin(); it != tokens.end(); it++) {
@@ -807,7 +807,6 @@ void FGTrafficManager::readTimeTableFromFile(SGPath infileName)
                      throw sg_io_exception("Error parsing traffic file @ " + buffString, infileName);
                  }
 
-
                  model          = tokens[12];
                  livery         = tokens[6];
                  homePort       = tokens[1];
@@ -826,26 +825,21 @@ void FGTrafficManager::readTimeTableFromFile(SGPath infileName)
                  offset         = atof(tokens[7].c_str());;
 
                  if (!FGAISchedule::validModelPath(model)) {
-                     SG_LOG(SG_AI, SG_WARN, "TrafficMgr: Missing model path:" <<
-                            model << " from " << infileName);
+                     simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AITrafficSchedule, "Missing traffic model path:" + model, infileName);
                  } else {
-
-                 SG_LOG(SG_AI, SG_INFO, "Adding Aircraft" << model << " " << livery << " " << homePort << " "
-                                                                << registration << " " << flightReq << " " << isHeavy
-                                                                << " " << acType << " " << airline << " " << m_class
-                                                                << " " << FlightType << " " << radius << " " << offset);
-                 scheduledAircraft.push_back(new FGAISchedule(model,
-                                                              livery,
-                                                              homePort,
-                                                              registration,
-                                                              flightReq,
-                                                              isHeavy,
-                                                              acType,
-                                                              airline,
-                                                              m_class,
-                                                              FlightType,
-                                                              radius,
-                                                              offset));
+                     SG_LOG(SG_AI, SG_DEBUG, "Adding Aircraft" << model << " " << livery << " " << homePort << " " << registration << " " << flightReq << " " << isHeavy << " " << acType << " " << airline << " " << m_class << " " << FlightType << " " << radius << " " << offset);
+                     scheduledAircraft.push_back(new FGAISchedule(model,
+                                                                  livery,
+                                                                  homePort,
+                                                                  registration,
+                                                                  flightReq,
+                                                                  isHeavy,
+                                                                  acType,
+                                                                  airline,
+                                                                  m_class,
+                                                                  FlightType,
+                                                                  radius,
+                                                                  offset));
                  } // of valid model path
              }
              if (tokens[0] == string("FLIGHT")) {
@@ -883,20 +877,20 @@ void FGTrafficManager::readTimeTableFromFile(SGPath infileName)
                  for (int i = 0; i < 7; i++) {
                      int j = i+1;
                      if (weekdays[i] != '.') {
-                         char buffer[4];
-                         snprintf(buffer, 4, "%d/", j);
-                         string departureTime = string(buffer) + depTimeGen + string(":00");
+                         char l_buffer[4];
+                         snprintf(l_buffer, 4, "%d/", j);
+                         string departureTime = string(l_buffer) + depTimeGen + string(":00");
                          string arrivalTime;
                          if (!arrivalWeekdayNeedsIncrement) {
-                             arrivalTime   = string(buffer) + arrTimeGen + string(":00");
+                             arrivalTime   = string(l_buffer) + arrTimeGen + string(":00");
                          }
                          if (arrivalWeekdayNeedsIncrement && i != 6 ) {
-                             snprintf(buffer, 4, "%d/", j+1);
-                             arrivalTime   = string(buffer) + arrTimeGen + string(":00");
+                             snprintf(l_buffer, 4, "%d/", j+1);
+                             arrivalTime   = string(l_buffer) + arrTimeGen + string(":00");
                          }
                          if (arrivalWeekdayNeedsIncrement && i == 6 ) {
-                             snprintf(buffer, 4, "%d/", 0);
-                             arrivalTime   = string(buffer) + arrTimeGen  + string(":00");
+                             snprintf(l_buffer, 4, "%d/", 0);
+                             arrivalTime   = string(l_buffer) + arrTimeGen  + string(":00");
                          }
                          SG_LOG(SG_AI, SG_ALERT, "Adding flight " << callsign       << " "
                                                       << fltrules       << " "
