@@ -27,10 +27,11 @@
 #include "test_suite/FGTestApi/TestPilot.hxx"
 #include "test_suite/FGTestApi/TestDataLogger.hxx"
 
-#include <Navaids/NavDataCache.hxx>
-#include <Navaids/navrecord.hxx>
-#include <Navaids/navlist.hxx>
 #include <Navaids/FlightPlan.hxx>
+#include <Navaids/NavDataCache.hxx>
+#include <Navaids/fixlist.hxx>
+#include <Navaids/navlist.hxx>
+#include <Navaids/navrecord.hxx>
 
 #include <Instrumentation/gps.hxx>
 #include <Instrumentation/navradio.hxx>
@@ -38,6 +39,7 @@
 #include <Autopilot/route_mgr.hxx>
 
 using namespace flightgear;
+using namespace std::string_literals;
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -73,7 +75,10 @@ void GPSTests::setUp()
 {
     FGTestApi::setUp::initTestGlobals("gps");
     FGTestApi::setUp::initNavDataCache();
-    
+
+    globals->get_subsystem_mgr()->bind();
+    globals->get_subsystem_mgr()->init();
+
     setupRouteManager();
 }
 
@@ -100,13 +105,13 @@ GPS* GPSTests::setupStandardGPS(SGPropertyNode_ptr config,
     gps->bind();
     gps->init();
     
-    globals->add_subsystem("gps", gps, SGSubsystemMgr::POST_FDM);
+    globals->get_subsystem_mgr()->add("gps", gps);
     return gps;
 }
 
 void GPSTests::setupRouteManager()
 {
-    auto rm = globals->add_new_subsystem<FGRouteMgr>();
+    auto rm = globals->get_subsystem_mgr()->add<FGRouteMgr>();
     rm->bind();
     rm->init();
     rm->postinit();
@@ -267,7 +272,7 @@ void GPSTests::testLegMode()
 {
     auto rm = globals->get_subsystem<FGRouteMgr>();
     
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     FGTestApi::setUp::populateFPWithoutNasal(fp, "EBBR", "07L", "EGGD", "27",
                                    "NIK COA DVR TAWNY WOD");
@@ -412,7 +417,7 @@ void GPSTests::testBuiltinRevertToOBSAtEnd()
 
     auto rm = globals->get_subsystem<FGRouteMgr>();
     
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     FGTestApi::setUp::populateFPWithoutNasal(fp, "EBBR", "07L", "EGGD", "27",
                                              "NIK COA DVR TAWNY BDN");
@@ -461,7 +466,7 @@ void GPSTests::testDirectToLegOnFlightplan()
 {
     auto rm = globals->get_subsystem<FGRouteMgr>();
     
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     FGTestApi::setUp::populateFPWithoutNasal(fp, "EBBR", "07L", "EGGD", "27",
                                  "NIK COA DVR TAWNY WOD");
@@ -525,7 +530,7 @@ void GPSTests::testDirectToLegOnFlightplanAndResumeBuiltin()
     auto rm = globals->get_subsystem<FGRouteMgr>();
    // FGTestApi::setUp::logPositionToKML("gps_dto_resume_leg");
 
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     FGTestApi::setUp::populateFPWithoutNasal(fp, "EBBR", "07L", "EGGD", "27",
                                              "NIK COA DVR TAWNY WOD");
@@ -587,7 +592,7 @@ void GPSTests::testDirectToLegOnFlightplanAndResumeBuiltin()
 void GPSTests::testLongLeg()
 {
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     FGTestApi::setUp::populateFPWithoutNasal(fp, "KLAX", "25R", "KJFK", "22R","VNY TEB");
     
@@ -680,9 +685,9 @@ void GPSTests::testLongLeg()
 void GPSTests::testLongLegWestbound()
 {
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
-    FGTestApi::setUp::populateFPWithoutNasal(fp, "ENBR", "35", "KMVY", "24","VOO GAKTU");
+    FGTestApi::setUp::populateFPWithoutNasal(fp, "ENBR", "35", "BIKF", "29","VOO GAKTU");
     
     // takes the place of the Nasal delegates
     auto testDelegate = new TestFPDelegate;
@@ -758,7 +763,7 @@ void GPSTests::testOverflightSequencing()
     
   //  FGTestApi::setUp::logPositionToKML("gps_sequence");
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     
     // let's use New Zealand for some southern hemisphere confusion :)
@@ -857,7 +862,7 @@ void GPSTests::testOffcourseSequencing()
     
  //   FGTestApi::setUp::logPositionToKML("gps_sequence_off");
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     
     
@@ -1004,7 +1009,7 @@ void GPSTests::testOffsetFlight()
     
    // FGTestApi::setUp::logPositionToKML("gps_offset_flight");
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     
     
@@ -1094,7 +1099,7 @@ void GPSTests::testLegIntercept()
 {
   //  FGTestApi::setUp::logPositionToKML("gps_intercept");
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     
         FGTestApi::setUp::populateFPWithoutNasal(fp, "NZCH", "02", "NZAA", "05L",
@@ -1265,7 +1270,7 @@ void GPSTests::testTurnAnticipation()
 {
     //FGTestApi::setUp::logPositionToKML("gps_flyby_sequence");
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
        
    // this route has some deliberately sharp turns to work the turn code
@@ -1376,12 +1381,72 @@ void GPSTests::testTurnAnticipation()
 
 }
 
+void GPSTests::testExceedFlyByMaxAngleTurn()
+{
+    //  FGTestApi::setUp::logPositionToKML("gps_fly_by_exceed_max_angle");
+    auto rm = globals->get_subsystem<FGRouteMgr>();
+    auto fp = FlightPlan::create();
+    rm->setFlightPlan(fp);
+
+    FGTestApi::setUp::populateFPWithoutNasal(fp, "LFBD", "23", "EHAM", "18L", "SOMOS ROYAN TIRAV BOBRI ADABI");
+    FGTestApi::writeFlightPlanToKML(fp);
+
+    // takes the place of the Nasal delegates
+    auto testDelegate = new TestFPDelegate;
+    testDelegate->thePlan = fp;
+    CPPUNIT_ASSERT(rm->activate());
+    fp->addDelegate(testDelegate);
+    auto gps = setupStandardGPS();
+
+    fp->setCurrentIndex(4); // BOBRI
+
+    // somehwat before BOBRI
+    SGGeod initPos = fp->pointAlongRouteNorm(4, -0.1);
+    FGTestApi::setPositionAndStabilise(initPos);
+    FGTestApi::writePointToKML("init point", initPos);
+
+    auto gpsNode = globals->get_props()->getNode("instrumentation/gps");
+    gpsNode->setBoolValue("config/delegate-sequencing", true);
+    gpsNode->setDoubleValue("config/max-fly-by-turn-angle-deg", 75.0);
+    gpsNode->setStringValue("command", "leg");
+
+    FGPositioned::TypeFilter fixFilter(FGPositioned::FIX);
+    auto tiravFix = FGPositioned::findFirstWithIdent("TIRAV", &fixFilter);
+    auto bobriFix = FGPositioned::findFirstWithIdent("BOBRI", &fixFilter);
+    auto adabiFix = FGPositioned::findFirstWithIdent("ADABI", &fixFilter);
+
+    auto pilot = SGSharedPtr<FGTestApi::TestPilot>(new FGTestApi::TestPilot);
+    pilot->resetAtPosition(initPos);
+    pilot->setCourseTrue(fp->legAtIndex(4)->courseDeg());
+    pilot->setSpeedKts(280);
+    pilot->flyGPSCourse(gps);
+
+    bool ok = FGTestApi::runForTimeWithCheck(1200.0, [&fp]() {
+        return fp->currentIndex() == 5;
+    });
+    CPPUNIT_ASSERT(ok);
+    // ensure we did a fly-over
+    CPPUNIT_ASSERT(!gps->previousLegData().value().didFlyBy);
+    FGTestApi::writePointToKML("seq point", globals->get_aircraft_position());
+
+    // ensure leg course is the one we want
+    const auto crs = SGGeodesy::courseDeg(bobriFix->geod(), adabiFix->geod());
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(crs, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
+
+    // ensure we fly the comepnsation turns, so that end up back on the next leg's course
+    // exactly
+    FGTestApi::runForTime(60.0);
+
+
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(crs, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
+}
+
 void GPSTests::testRadialIntercept()
 {
    // FGTestApi::setUp::logPositionToKML("gps_radial_intercept");
 
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
        
     FGTestApi::setUp::populateFPWithoutNasal(fp, "LFKC", "36", "LIRF", "25", "BUNAX BEBEV AJO");
@@ -1399,7 +1464,7 @@ void GPSTests::testRadialIntercept()
     
     FGTestApi::writeFlightPlanToKML(fp);
     
-// position slightly before BUNAX
+    // position slightly before BUNAX
     SGGeod initPos = fp->pointAlongRoute(2, -3.0);
     
     // takes the place of the Nasal delegates
@@ -1418,7 +1483,7 @@ void GPSTests::testRadialIntercept()
     
     fp->setCurrentIndex(2);
 
-    CPPUNIT_ASSERT_EQUAL(string{"BUNAX"}, string{gpsNode->getStringValue("wp/wp[1]/ID")});
+    CPPUNIT_ASSERT_EQUAL("BUNAX"s, gpsNode->getStringValue("wp/wp[1]/ID"));
     CPPUNIT_ASSERT_DOUBLES_EQUAL(312, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
 
   
@@ -1437,39 +1502,100 @@ void GPSTests::testRadialIntercept()
     CPPUNIT_ASSERT_DOUBLES_EQUAL(185, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
     FGTestApi::runForTime(30.0);
     
-// repeat but with fly-by enabled
-  gpsNode->setBoolValue("config/enable-fly-by", true);
+    // repeat but with fly-by enabled
+    gpsNode->setBoolValue("config/enable-fly-by", true);
     gpsNode->setStringValue("command", "obs");
   
     FGTestApi::setPositionAndStabilise(initPos);
-      pilot->resetAtPosition(initPos);
-  fp->setCurrentIndex(2);
+    pilot->resetAtPosition(initPos);
+    fp->setCurrentIndex(2);
     gpsNode->setStringValue("command", "leg");
 
-  CPPUNIT_ASSERT_EQUAL(string{"BUNAX"}, string{gpsNode->getStringValue("wp/wp[1]/ID")});
-  CPPUNIT_ASSERT_DOUBLES_EQUAL(312, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
+    CPPUNIT_ASSERT_EQUAL("BUNAX"s, gpsNode->getStringValue("wp/wp[1]/ID"));
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(312, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
 
 
-  pilot->setCourseTrue(fp->legAtIndex(2)->courseDeg());
-  pilot->flyGPSCourse(gps);
+    pilot->setCourseTrue(fp->legAtIndex(2)->courseDeg());
+    pilot->flyGPSCourse(gps);
   
-  ok = FGTestApi::runForTimeWithCheck(600.0, [fp]() {
-      return fp->currentIndex() == 4;
-  });
-  CPPUNIT_ASSERT(ok);
+    ok = FGTestApi::runForTimeWithCheck(600.0, [fp]() {
+        return fp->currentIndex() == 4;
+    });
+    CPPUNIT_ASSERT(ok);
   
-  // flying to BEBEV now
-  CPPUNIT_ASSERT_DOUBLES_EQUAL(185, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
-  FGTestApi::runForTime(30.0);
+    // flying to BEBEV now
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(185, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
+    FGTestApi::runForTime(30.0);
 }
 
+void GPSTests::testSWIFT8()
+{
+    FGTestApi::setUp::logPositionToKML("gps_swift8");
+
+    auto rm = globals->get_subsystem<FGRouteMgr>();
+    auto fp = FlightPlan::create();
+    rm->setFlightPlan(fp);
+       
+    FGTestApi::setUp::populateFPWithoutNasal(fp, "YBCS", "15", "YBTL", "25", "SWIFT");
+    
+    auto hdgToAlt_400 = new HeadingToAltitude(fp, "(400)", 150);
+    hdgToAlt_400->setAltitude(400, RESTRICT_AT);
+    fp->insertWayptAtIndex(hdgToAlt_400, 1);
+	
+    SGGeod pos = SGGeod::fromDeg(145.743889,-16.850028);
+    auto intc = new RadialIntercept(fp, "(INTC)", pos, 30, 80);
+    fp->insertWayptAtIndex(intc, 2);
+	
+    auto hdgToAlt_4000 = new HeadingToAltitude(fp, "(4000)", 80);
+    hdgToAlt_4000->setAltitude(4000, RESTRICT_AT);
+    fp->insertWayptAtIndex(hdgToAlt_4000, 3);
+	
+    FGTestApi::writeFlightPlanToKML(fp);
+	
+    // position slightly before INTC
+    SGGeod initPos = fp->pointAlongRoute(1, -1);
+    
+    // takes the place of the Nasal delegates
+    auto testDelegate = new TestFPDelegate;
+    testDelegate->thePlan = fp;
+    CPPUNIT_ASSERT(rm->activate());
+    fp->addDelegate(testDelegate);
+    auto gps = setupStandardGPS();
+    
+    FGTestApi::setPositionAndStabilise(initPos);
+    
+    auto gpsNode = globals->get_props()->getNode("instrumentation/gps");
+    gpsNode->setBoolValue("config/delegate-sequencing", true);
+    gpsNode->setBoolValue("config/enable-fly-by", false);
+    gpsNode->setStringValue("command", "leg");
+
+    fp->setCurrentIndex(1);
+	
+    auto pilot = SGSharedPtr<FGTestApi::TestPilot>(new FGTestApi::TestPilot);
+    pilot->resetAtPosition(initPos);
+    pilot->setCourseTrue(fp->legAtIndex(1)->courseDeg());
+    pilot->setSpeedKts(200);
+    pilot->setTargetAltitudeFtMSL(6000);
+    pilot->flyGPSCourse(gps);
+    
+    bool ok = FGTestApi::runForTimeWithCheck(600.0, [fp]() {
+        return fp->currentIndex() == 4;
+    });
+    CPPUNIT_ASSERT(ok);
+	
+    FGTestApi::runForTime(600.0);
+	
+    // Flying to SWIFT
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(149, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
+    FGTestApi::runForTime(30.0);
+}
 
 void GPSTests::testDMEIntercept()
 {
    // FGTestApi::setUp::logPositionToKML("gps_dme_intercept");
     
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     
     // manually consturct something close to the publichsed approach transition for EGPH ILS 06
@@ -1513,9 +1639,9 @@ void GPSTests::testDMEIntercept()
     gpsNode->setStringValue("command", "leg");
     
     fp->setCurrentIndex(2);
-    
-    CPPUNIT_ASSERT_EQUAL(string{"TLA"}, string{gpsNode->getStringValue("wp/wp[1]/ID")});
-   // CPPUNIT_ASSERT_DOUBLES_EQUAL(312, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
+
+    CPPUNIT_ASSERT_EQUAL("TLA"s, gpsNode->getStringValue("wp/wp[1]/ID"));
+    // CPPUNIT_ASSERT_DOUBLES_EQUAL(312, gpsNode->getDoubleValue("wp/leg-true-course-deg"), 1.0);
     
     auto pilot = SGSharedPtr<FGTestApi::TestPilot>(new FGTestApi::TestPilot);
     pilot->resetAtPosition(initPos);
@@ -1534,7 +1660,7 @@ void GPSTests::testFinalLegCourse()
 {
     // FGTestApi::setUp::logPositionToKML("gps_final_course");
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
     
     // we can't use the standard function as it puts a waypoint on 
@@ -1562,7 +1688,7 @@ void GPSTests::testFinalLegCourse()
     testDelegate->thePlan = fp;
     CPPUNIT_ASSERT(rm->activate());
     fp->addDelegate(testDelegate);
-    auto gps = setupStandardGPS();
+    setupStandardGPS();
     
     fp->setCurrentIndex(2);
     
@@ -1577,6 +1703,8 @@ void GPSTests::testFinalLegCourse()
 
     // FGTestApi::writeFlightPlanToKML(fp);
     // check that the final leg course doesn't fall back to 233 deg
+
+    // EXPECTED FAIL: at the moment, we don't do the right thing for this, yet
     // CPPUNIT_ASSERT_DOUBLES_EQUAL(37, gpsNode->getDoubleValue("desired-course-deg"), 2.0);
 }
 
@@ -1586,7 +1714,7 @@ void GPSTests::testCourseLegIntermediateWaypoint()
 {
     // FGTestApi::setUp::logPositionToKML("gps_leg_course_intermediate");
     auto rm = globals->get_subsystem<FGRouteMgr>();
-    auto fp = new FlightPlan;
+    auto fp = FlightPlan::create();
     rm->setFlightPlan(fp);
 
     FGTestApi::setUp::populateFPWithoutNasal(fp, "EGAA", "25", "EGPH", "06", "LISBO BLACA");
@@ -1646,8 +1774,75 @@ void GPSTests::testCourseLegIntermediateWaypoint()
     pilot->setSpeedKts(280);
     pilot->flyGPSCourse(gps);
 
-    bool ok = FGTestApi::runForTimeWithCheck(1200.0, [fp]() {
+    bool ok = FGTestApi::runForTimeWithCheck(1200.0, [&fp]() {
         return fp->currentIndex() == 4;
     });
     CPPUNIT_ASSERT(ok);
 }
+
+// Test to check the situation where you have two legs forming a straight line
+// with the current waypoint being the third waypoint
+void GPSTests::testFlyOverMaxInterceptAngle()
+{
+    //FGTestApi::setUp::logPositionToKML("gps_fly_over_max_intercept_angle");
+    auto rm = globals->get_subsystem<FGRouteMgr>();
+    auto fp = FlightPlan::create();
+    rm->setFlightPlan(fp);
+
+    FGTestApi::setUp::populateFPWithoutNasal(fp, "EGAA", "25", "EGPH", "06", "LISBO BLACA");
+
+    // takes the place of the Nasal delegates
+    auto testDelegate = new TestFPDelegate;
+    testDelegate->thePlan = fp;
+    CPPUNIT_ASSERT(rm->activate());
+    fp->addDelegate(testDelegate);
+
+    FGTestApi::writeFlightPlanToKML(fp);
+    
+    SGGeod initPos = fp->pointAlongRouteNorm(1, -0.15);
+    FGTestApi::writePointToKML("Start pos", initPos);
+
+    auto gps = setupStandardGPS();
+    FGTestApi::setPositionAndStabilise(initPos);
+    
+    auto gpsNode = globals->get_props()->getNode("instrumentation/gps");
+    gpsNode->setBoolValue("config/delegate-sequencing", true);
+    gpsNode->setStringValue("command", "leg");
+
+    auto gpsErrorNmNode = gpsNode->getNode("wp/wp[1]/course-error-nm");
+
+    fp->setCurrentIndex(1); // BLACA
+
+    auto pilot = SGSharedPtr<FGTestApi::TestPilot>(new FGTestApi::TestPilot);
+    pilot->resetAtPosition(initPos);
+    pilot->setCourseTrue(fp->legAtIndex(1)->courseDeg());
+    pilot->setSpeedKts(280);
+    pilot->flyGPSCourse(gps);
+
+    bool ok = FGTestApi::runForTimeWithCheck(1200.0, [&fp]() {
+        return fp->currentIndex() == 2;
+    });
+    CPPUNIT_ASSERT(ok);
+    
+    FGPositioned::TypeFilter fixFilter(FGPositioned::FIX);
+    auto lisbo = FGPositioned::findClosestWithIdent("LISBO", fp->departureAirport()->geod(), &fixFilter);
+    auto blaca = FGPositioned::findClosestWithIdent("BLACA", fp->departureAirport()->geod(), &fixFilter);
+
+    const double crsToBlaca = SGGeodesy::courseDeg(lisbo->geod(), blaca->geod());
+    
+    ok = FGTestApi::runForTimeWithCheck(1200.0, [pilot, crsToBlaca, gpsErrorNmNode]()
+                                        {
+        const double heading = pilot->trueCourseDeg();
+        if (heading < (crsToBlaca - 45.0)) {
+            CPPUNIT_FAIL("Turned too far on intercept");
+        }
+        
+        // check if we're near the leg
+        if (fabs(gpsErrorNmNode->getDoubleValue()) > 0.05) {
+            return false;
+        }
+        
+        return pilot->isOnHeading(crsToBlaca);
+    });
+    CPPUNIT_ASSERT(ok);
+}
