@@ -1,3 +1,25 @@
+/*
+ * Copyright (C) 2020 James Turner
+ *
+ * This file is part of the program FlightGear.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#include "config.h"
+
+
 #include "test_flightplan.hxx"
 
 #include <algorithm>
@@ -22,7 +44,9 @@
 #include <Navaids/fix.hxx>
 
 #include <Airports/airport.hxx>
+#include <Main/globals.hxx>
 
+using namespace std::string_literals;
 using namespace flightgear;
 
 static bool static_haveProcedures = false;
@@ -124,6 +148,19 @@ public:
                 _plan->insertWayptAtIndex(w, insertIndex++);
             }
         }
+        
+        if (_plan->approach()) {
+            WayptVec approachRoute;
+            bool ok = _plan->approach()->routeFromVectors(approachRoute);
+            if (!ok)
+                throw sg_exception("failed to route approach");
+            int insertIndex = leg->index();
+            for (auto w : approachRoute) {
+                w->setFlag(WPT_ARRIVAL);
+                w->setFlag(WPT_GENERATED);
+                _plan->insertWayptAtIndex(w, insertIndex++);
+            }
+        }
     }
     
     
@@ -193,7 +230,7 @@ TestFPDelegate* TestFPDelegateFactory::delegateForPlan(FlightPlan* fp)
 // Set up function for each test.
 void FlightplanTests::setUp()
 {
-    FGTestApi::setUp::initTestGlobals("flightplan");
+    FGTestApi::setUp::initTestGlobals("flightplan"s);
     FGTestApi::setUp::initNavDataCache();
     
     SGPath proceduresPath = SGPath::fromEnv("FG_PROCEDURES_PATH");
@@ -221,43 +258,43 @@ void FlightplanTests::tearDown()
 
 static FlightPlanRef makeTestFP(const std::string& depICAO, const std::string& depRunway,
                          const std::string& destICAO, const std::string& destRunway,
-                         const std::string& waypoints)
+                         const std::string& waypoints, bool asRoute = false)
 {
-    FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = asRoute ? FlightPlan::createRoute() : FlightPlan::create();
     FGTestApi::setUp::populateFPWithoutNasal(f, depICAO, depRunway, destICAO, destRunway, waypoints);
     return f;
 }
 
 void FlightplanTests::testBasic()
 {
-    FlightPlanRef fp1 = makeTestFP("EGCC", "23L", "EHAM", "24",
-                                   "TNT CLN");
-    fp1->setIdent("testplan");
+    FlightPlanRef fp1 = makeTestFP("EGCC"s, "23L"s, "EHAM"s, "24"s,
+                                   "TNT CLN"s);
+    fp1->setIdent("testplan"s);
 
-    CPPUNIT_ASSERT(fp1->ident() == "testplan");
-    CPPUNIT_ASSERT(fp1->departureAirport()->ident() == "EGCC");
-    CPPUNIT_ASSERT(fp1->departureRunway()->ident() == "23L");
-    CPPUNIT_ASSERT(fp1->destinationAirport()->ident() == "EHAM");
-    CPPUNIT_ASSERT(fp1->destinationRunway()->ident() == "24");
+    CPPUNIT_ASSERT(fp1->ident() == "testplan"s);
+    CPPUNIT_ASSERT(fp1->departureAirport()->ident() == "EGCC"s);
+    CPPUNIT_ASSERT(fp1->departureRunway()->ident() == "23L"s);
+    CPPUNIT_ASSERT(fp1->destinationAirport()->ident() == "EHAM"s);
+    CPPUNIT_ASSERT(fp1->destinationRunway()->ident() == "24"s);
 
     CPPUNIT_ASSERT_EQUAL(5, fp1->numLegs());
 
-    CPPUNIT_ASSERT(fp1->legAtIndex(0)->waypoint()->source()->ident() == "23L");
+    CPPUNIT_ASSERT(fp1->legAtIndex(0)->waypoint()->source()->ident() == "23L"s);
 
-    CPPUNIT_ASSERT(fp1->legAtIndex(1)->waypoint()->source()->ident() == "TNT");
-    CPPUNIT_ASSERT(fp1->legAtIndex(1)->waypoint()->source()->name() == "TRENT VOR-DME");
+    CPPUNIT_ASSERT(fp1->legAtIndex(1)->waypoint()->source()->ident() == "TNT"s);
+    CPPUNIT_ASSERT(fp1->legAtIndex(1)->waypoint()->source()->name() == "TRENT VOR-DME"s);
 
-    CPPUNIT_ASSERT(fp1->legAtIndex(2)->waypoint()->source()->ident() == "CLN");
-    CPPUNIT_ASSERT(fp1->legAtIndex(2)->waypoint()->source()->name() == "CLACTON VOR-DME");
+    CPPUNIT_ASSERT(fp1->legAtIndex(2)->waypoint()->source()->ident() == "CLN"s);
+    CPPUNIT_ASSERT(fp1->legAtIndex(2)->waypoint()->source()->name() == "CLACTON VOR-DME"s);
 
-    CPPUNIT_ASSERT(fp1->legAtIndex(4)->waypoint()->source()->ident() == "24");
+    CPPUNIT_ASSERT(fp1->legAtIndex(4)->waypoint()->source()->ident() == "24"s);
 
 }
 
 void FlightplanTests::testRoutePathBasic()
 {
-    FlightPlanRef fp1 = makeTestFP("EGHI", "20", "EDDM", "08L",
-                                   "SFD LYD BNE CIV ELLX LUX SAA KRH WLD");
+    FlightPlanRef fp1 = makeTestFP("EGHI"s, "20"s, "EDDM"s, "08L"s,
+                                   "SFD LYD BNE CIV ELLX LUX SAA KRH WLD"s);
 
 
     RoutePath rtepath(fp1);
@@ -273,10 +310,10 @@ void FlightplanTests::testRoutePathBasic()
     // check some leg parameters
 
     // BOLOUGNE SUR MER, near LFAY (AMIENS)
-    FGNavRecordRef bne = FGNavList::findByFreq(113.8, FGAirport::getByIdent("LFAY")->geod());
+    FGNavRecordRef bne = FGNavList::findByFreq(113.8, FGAirport::getByIdent("LFAY"s)->geod());
 
     // CHIEVRES
-    FGNavRecordRef civ = FGNavList::findByFreq(113.2, FGAirport::getByIdent("EBCI")->geod());
+    FGNavRecordRef civ = FGNavList::findByFreq(113.2, FGAirport::getByIdent("EBCI"s)->geod());
 
     double distM = SGGeodesy::distanceM(bne->geod(), civ->geod());
     double trackDeg = SGGeodesy::courseDeg(bne->geod(), civ->geod());
@@ -291,8 +328,8 @@ void FlightplanTests::testRoutePathBasic()
 
 void FlightplanTests::testRoutePathSkipped()
 {
-    FlightPlanRef fp1 = makeTestFP("EHAM", "24", "EDDM", "08L",
-                                   "EHEH KBO TAU FFM FFM/100/0.01 FFM/120/0.02 WUR WLD");
+    FlightPlanRef fp1 = makeTestFP("EHAM"s, "24"s, "EDDM"s, "08L"s,
+                                   "EHEH KBO TAU FFM FFM/100/0.01 FFM/120/0.02 WUR WLD"s);
 
 
     RoutePath rtepath(fp1);
@@ -313,8 +350,8 @@ void FlightplanTests::testRoutePathSkipped()
 
 void FlightplanTests::testRoutePathTrivialFlightPlan()
 {
-    FlightPlanRef fp1 = makeTestFP("EGPH", "24", "EGPH", "06",
-                                   "");
+    FlightPlanRef fp1 = makeTestFP("EGPH"s, "24"s, "EGPH"s, "06"s,
+                                   ""s);
 
 
     RoutePath rtepath(fp1);
@@ -325,19 +362,19 @@ void FlightplanTests::testRoutePathTrivialFlightPlan()
         rtepath.distanceForIndex(leg);
     }
 
-    CPPUNIT_ASSERT_DOUBLES_EQUAL(19.68, fp1->totalDistanceNm(), 0.2);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(19.5, fp1->totalDistanceNm(), 0.1);
 }
 
 void FlightplanTests::testRoutePathVec()
 {
-    FlightPlanRef fp1 = makeTestFP("KNUQ", "14L", "PHNL", "22R",
-                                   "ROKME WOVAB");
+    FlightPlanRef fp1 = makeTestFP("KNUQ"s, "14L"s, "PHNL"s, "22R"s,
+                                   "ROKME WOVAB"s);
     RoutePath rtepath(fp1);
 
     SGGeodVec vec = rtepath.pathForIndex(0);
 
-    FGAirportRef ksfo = FGAirport::findByIdent("KSFO");
-    FGFixRef rokme = fgpositioned_cast<FGFix>(FGPositioned::findClosestWithIdent("ROKME", ksfo->geod()));
+    FGAirportRef ksfo = FGAirport::findByIdent("KSFO"s);
+    FGFixRef rokme = fgpositioned_cast<FGFix>(FGPositioned::findClosestWithIdent("ROKME"s, ksfo->geod()));
     auto depRwy = fp1->departureRunway();
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL(depRwy->geod().getLongitudeDeg(), vec.front().getLongitudeDeg(), 0.01);
@@ -354,13 +391,37 @@ void FlightplanTests::testRoutePathVec()
     //CPPUNIT_ASSERT(vec.front()
 }
 
+void FlightplanTests::testRoutePathFinalLegVQPR15()
+{
+    // test behaviour of RoutePath when the last leg prior to the arrival runway
+    // is beyond the runway. This occurs in Paro RNAVZ15 approach.
+    
+    if (!static_haveProcedures)
+        return;
+    
+    static_factory = std::make_shared<TestFPDelegateFactory>();
+    FlightPlan::registerDelegateFactory(static_factory);
+    
+    FlightPlanRef f = FlightPlan::create();
+//    auto ourDelegate = TestFPDelegateFactory::delegateForPlan(f);
+    
+    auto vidp = FGAirport::findByIdent("VIDP"s);
+    f->setDeparture(vidp->getRunwayByIdent("09"s));
+    
+    auto vqpr = FGAirport::findByIdent("VQPR"s);
+    f->setDestination(vqpr->getRunwayByIdent("15"s));
+    f->setApproach(vqpr->findApproachWithIdent("RNVZ15"s));
+    RoutePath rtepath(f);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(1100, rtepath.distanceForIndex(14), 100); // calculated from raw lat lon. 
+}
+
 void FlightplanTests::testRoutPathWpt0Midflight()
 {
     // test behaviour of RoutePath when WP0 is not a runway
     // happens for the Airbus ND which removes past wpts when sequencing
 
-    FlightPlanRef fp1 = makeTestFP("KNUQ", "14L", "PHNL", "22R",
-                                   "ROKME WOVAB");
+    FlightPlanRef fp1 = makeTestFP("KNUQ"s, "14L"s, "PHNL"s, "22R"s,
+                                   "ROKME WOVAB"s);
     // actually delete leg 0 so we start at ROKME
     fp1->deleteIndex(0);
 
@@ -368,8 +429,8 @@ void FlightplanTests::testRoutPathWpt0Midflight()
 
     SGGeodVec vec = rtepath.pathForIndex(0);
 
-    FGAirportRef ksfo = FGAirport::findByIdent("KSFO");
-    FGFixRef rokme = fgpositioned_cast<FGFix>(FGPositioned::findClosestWithIdent("ROKME", ksfo->geod()));
+    FGAirportRef ksfo = FGAirport::findByIdent("KSFO"s);
+    FGFixRef rokme = fgpositioned_cast<FGFix>(FGPositioned::findClosestWithIdent("ROKME"s, ksfo->geod()));
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL(rokme->geod().getLongitudeDeg(), vec.front().getLongitudeDeg(), 0.01);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(rokme->geod().getLatitudeDeg(), vec.front().getLatitudeDeg(), 0.01);
@@ -383,45 +444,110 @@ void FlightplanTests::testRoutPathWpt0Midflight()
 
 void FlightplanTests::testBasicAirways()
 {
-    Airway* awy = Airway::findByIdent("J547", Airway::HighLevel);
-    CPPUNIT_ASSERT_EQUAL(awy->ident(), std::string("J547"));
+    Airway* awy = Airway::findByIdent("J547"s, Airway::HighLevel);
+    CPPUNIT_ASSERT_EQUAL(awy->ident(), "J547"s);
 
-    FGAirportRef kord = FGAirport::findByIdent("KORD");
-    FlightPlanRef f = new FlightPlan;
+    FGAirportRef kord = FGAirport::findByIdent("KORD"s);
+    FlightPlanRef f = FlightPlan::create();
     f->setDeparture(kord);
 
-    CPPUNIT_ASSERT(awy->findEnroute("KITOK"));
-    CPPUNIT_ASSERT(awy->findEnroute("LESUB"));
+    CPPUNIT_ASSERT(awy->findEnroute("KITOK"s));
+    CPPUNIT_ASSERT(awy->findEnroute("LESUB"s));
 
-    auto wpt = awy->findEnroute("FNT");
+    auto wpt = awy->findEnroute("FNT"s);
     CPPUNIT_ASSERT(wpt);
-    CPPUNIT_ASSERT(wpt->ident() == "FNT");
-    CPPUNIT_ASSERT(wpt->source() == FGNavRecord::findClosestWithIdent("FNT", kord->geod()));
+    CPPUNIT_ASSERT(wpt->ident() == "FNT"s);
+    CPPUNIT_ASSERT(wpt->source() == FGNavRecord::findClosestWithIdent("FNT"s, kord->geod()));
 
-    auto wptKUBBS = f->waypointFromString("KUBBS");
-    auto wptFNT = f->waypointFromString("FNT");
+    auto wptKUBBS = f->waypointFromString("KUBBS"s);
+    auto wptFNT = f->waypointFromString("FNT"s);
 
     CPPUNIT_ASSERT(awy->canVia(wptKUBBS, wptFNT));
 
     WayptVec path = awy->via(wptKUBBS, wptFNT);
     CPPUNIT_ASSERT_EQUAL(4, static_cast<int>(path.size()));
 
-    CPPUNIT_ASSERT_EQUAL(std::string("PMM"), path.at(0)->ident());
-    CPPUNIT_ASSERT_EQUAL(std::string("HASTE"), path.at(1)->ident());
-    CPPUNIT_ASSERT_EQUAL(std::string("DEWIT"), path.at(2)->ident());
-    CPPUNIT_ASSERT_EQUAL(std::string("FNT"), path.at(3)->ident());
+    CPPUNIT_ASSERT_EQUAL("PMM"s, path.at(0)->ident());
+    CPPUNIT_ASSERT_EQUAL("HASTE"s, path.at(1)->ident());
+    CPPUNIT_ASSERT_EQUAL("DEWIT"s, path.at(2)->ident());
+    CPPUNIT_ASSERT_EQUAL("FNT"s, path.at(3)->ident());
+
+    f->insertWayptsAtIndex(path, 0);
+    CPPUNIT_ASSERT_EQUAL(4, static_cast<int>(f->numLegs()));
+    CPPUNIT_ASSERT_EQUAL("DEWIT"s, f->legAtIndex(2)->waypoint()->ident());
+    CPPUNIT_ASSERT_EQUAL(static_cast<RouteBase*>(awy), f->legAtIndex(2)->waypoint()->owner());
+}
+
+void FlightplanTests::testViaInsertIntoFP()
+{
+    Airway* awy = Airway::findByIdent("J547"s, Airway::HighLevel);
+    CPPUNIT_ASSERT_EQUAL(awy->ident(), "J547"s);
+
+    FGAirportRef kord = FGAirport::findByIdent("KORD"s);
+    FlightPlanRef f = FlightPlan::create();
+    f->setDeparture(kord);
+
+    CPPUNIT_ASSERT(awy->findEnroute("KITOK"s));
+    CPPUNIT_ASSERT(awy->findEnroute("LESUB"s));
+
+    auto wpt = awy->findEnroute("FNT"s);
+    CPPUNIT_ASSERT(wpt);
+
+    auto wptKUBBS = f->waypointFromString("KUBBS"s);
+    auto wptFNT = f->waypointFromString("FNT"s);
+    f->insertWayptAtIndex(wptKUBBS, -1);
+    CPPUNIT_ASSERT_EQUAL(1, static_cast<int>(f->numLegs()));
+
+    Via* via = new Via(f, awy, wptFNT->source());
+    f->insertWayptAtIndex(via, -1);
+    CPPUNIT_ASSERT_EQUAL(5, static_cast<int>(f->numLegs()));
+    CPPUNIT_ASSERT_EQUAL("PMM"s, f->legAtIndex(1)->waypoint()->ident());
+    CPPUNIT_ASSERT_EQUAL(static_cast<RouteBase*>(awy), f->legAtIndex(1)->waypoint()->owner());
+}
+
+void FlightplanTests::testViaInsertIntoRoute()
+{
+    Airway* awy = Airway::findByIdent("J547"s, Airway::HighLevel);
+    CPPUNIT_ASSERT_EQUAL(awy->ident(), "J547"s);
+
+    FGAirportRef kord = FGAirport::findByIdent("KORD"s);
+    FlightPlanRef f = FlightPlan::createRoute();
+    f->setDeparture(kord);
+
+    CPPUNIT_ASSERT(awy->findEnroute("KITOK"s));
+    CPPUNIT_ASSERT(awy->findEnroute("LESUB"s));
+
+    auto wpt = awy->findEnroute("FNT"s);
+    CPPUNIT_ASSERT(wpt);
+
+    auto wptKUBBS = f->waypointFromString("KUBBS"s);
+    auto wptFNT = f->waypointFromString("FNT"s);
+    f->insertWayptAtIndex(wptKUBBS, -1);
+    CPPUNIT_ASSERT_EQUAL(1, static_cast<int>(f->numLegs()));
+
+    Via* via = new Via(f, awy, wptFNT->source());
+    f->insertWayptAtIndex(via, -1);
+    CPPUNIT_ASSERT_EQUAL(2, static_cast<int>(f->numLegs()));
+
+// clone and convert to active flight-plan; Vias
+// should be expanded
+    FlightPlanRef activeFP = f->clone("ACT", true);
+
+    CPPUNIT_ASSERT_EQUAL(5, static_cast<int>(activeFP->numLegs()));
+    CPPUNIT_ASSERT_EQUAL("PMM"s, activeFP->legAtIndex(1)->waypoint()->ident());
+    CPPUNIT_ASSERT_EQUAL(static_cast<RouteBase*>(awy), activeFP->legAtIndex(1)->waypoint()->owner());
 }
 
 void FlightplanTests::testAirwayNetworkRoute()
 {
-    FGAirportRef egph = FGAirport::findByIdent("EGPH");
-    FlightPlanRef f = new FlightPlan;
+    FGAirportRef egph = FGAirport::findByIdent("EGPH"s);
+    FlightPlanRef f = FlightPlan::create();
     f->setDeparture(egph);
 
     auto highLevelNet = Airway::highLevel();
 
-    auto wptTLA = f->waypointFromString("TLA");
-    auto wptCNA = f->waypointFromString("CNA");
+    auto wptTLA = f->waypointFromString("TLA"s);
+    auto wptCNA = f->waypointFromString("CNA"s);
 
     WayptVec route;
     bool ok = highLevelNet->route(wptTLA, wptCNA, route);
@@ -432,10 +558,10 @@ void FlightplanTests::testAirwayNetworkRoute()
 
 void FlightplanTests::testParseICAORoute()
 {
-    FGAirportRef kord = FGAirport::findByIdent("KORD");
-    FlightPlanRef f = new FlightPlan;
+    FGAirportRef kord = FGAirport::findByIdent("KORD"s);
+    FlightPlanRef f = FlightPlan::create();
     f->setDeparture(kord);
-    f->setDestination(FGAirport::findByIdent("KSAN"));
+    f->setDestination(FGAirport::findByIdent("KSAN"s));
 
     const char* route = "DCT JOT J26 IRK J96 SLN J18 GCK J96 CIM J134 GUP J96 KEYKE J134 DRK J78 LANCY J96 PKE";
   //  const char* route = "DCT KUBBS J547 FNT Q824 HOCKE Q905 SIKBO Q907 MIILS N400A TUDEP NATW GISTI DCT SLANY UL9 DIKAS UL18 GAVGO UL9 KONAN UL607 UBIDU Y863 RUDUS T109 HAREM T104 ANORA STAR";
@@ -450,10 +576,10 @@ void FlightplanTests::testParseICANLowLevelRoute()
 {
     const char* route = "DCT DPA V6 IOW V216 LAA V210 GOSIP V83 ACHES V210 BLOKE V83 ALS V210 RSK V95 INW V12 HOXOL V264 OATES V12 JUWSO V264 PKE";
 
-    FGAirportRef kord = FGAirport::findByIdent("KORD");
-    FlightPlanRef f = new FlightPlan;
+    FGAirportRef kord = FGAirport::findByIdent("KORD"s);
+    FlightPlanRef f = FlightPlan::create();
     f->setDeparture(kord);
-    f->setDestination(FGAirport::findByIdent("KSAN"));
+    f->setDestination(FGAirport::findByIdent("KSAN"s));
 
     bool ok = f->parseICAORouteString(route);
     CPPUNIT_ASSERT(ok);
@@ -496,12 +622,12 @@ void FlightplanTests::testBug1814()
   )";
 
     std::istringstream stream(fpXML);
-    FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = FlightPlan::create();
     bool ok = f->load(stream);
     CPPUNIT_ASSERT(ok);
 
     auto leg = f->legAtIndex(1);
-    CPPUNIT_ASSERT_EQUAL(std::string("SIGUL"), leg->waypoint()->ident());
+    CPPUNIT_ASSERT_EQUAL("SIGUL"s, leg->waypoint()->ident());
 
     CPPUNIT_ASSERT_DOUBLES_EQUAL(137, leg->distanceNm(), 0.5);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(101, f->legAtIndex(2)->distanceNm(), 0.5);
@@ -547,7 +673,7 @@ void FlightplanTests::testLoadSaveMachRestriction()
      )";
     
      std::istringstream stream(fpXML);
-     FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = FlightPlan::create();
      bool ok = f->load(stream);
      CPPUNIT_ASSERT(ok);
 
@@ -566,7 +692,7 @@ void FlightplanTests::testLoadSaveMachRestriction()
     f->save(ss);
     
     std::istringstream iss(ss.str());
-    FlightPlanRef f2 = new FlightPlan;
+    FlightPlanRef f2 = FlightPlan::create();
     ok = f2->load(iss);
     CPPUNIT_ASSERT(ok);
     
@@ -577,10 +703,81 @@ void FlightplanTests::testLoadSaveMachRestriction()
     CPPUNIT_ASSERT_EQUAL(8, f2->legAtIndex(2)->holdCount());
 }
 
+void FlightplanTests::testLoadSaveBetweenRestriction()
+{
+    const std::string fpXML = R"(<?xml version="1.0" encoding="UTF-8"?>
+       <PropertyList>
+           <version type="int">2</version>
+           <departure>
+               <airport type="string">SAWG</airport>
+               <runway type="string">25</runway>
+           </departure>
+           <destination>
+               <airport type="string">SUMU</airport>
+           </destination>
+           <route>
+                <wp n="0">
+                  <type type="string">navaid</type>
+                  <ident type="string">PUGLI</ident>
+                  <lon type="double">-60.552200</lon>
+                  <lat type="double">-40.490000</lat>
+                </wp>
+                <wp n="1">
+                  <type type="string">basic</type>
+                  <alt-restrict type="string">between</alt-restrict>
+                  <altitude-ft type="double">36000</altitude-ft>
+                  <constraint-altitude type="double">32000</constraint-altitude>
+                  <ident type="string">SV002</ident>
+                  <lon type="double">-115.50531</lon>
+                  <lat type="double">37.89523</lat>
+                </wp>
+                <wp n="2">
+                     <type type="string">navaid</type>
+                     <ident type="string">SIGUL</ident>
+                     <lon type="double">-60.552200</lon>
+                     <lat type="double">-40.490000</lat>
+                   </wp>
+           </route>
+       </PropertyList>
+     )";
+    
+     std::istringstream stream(fpXML);
+    FlightPlanRef f = FlightPlan::create();
+     bool ok = f->load(stream);
+     CPPUNIT_ASSERT(ok);
+
+     auto leg = f->legAtIndex(1);
+    CPPUNIT_ASSERT_EQUAL(RESTRICT_BETWEEN, leg->altitudeRestriction());
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(36000, leg->altitudeFt(), 0.01);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(32000, leg->waypoint()->constraintAltitude(ALTITUDE_FEET), 0.01);
+}
+
+void FlightplanTests::testRestrictionUnits()
+{
+    FlightPlanRef fp1 = makeTestFP("LIRF"s, "34L"s, "LEBL"s, "25R"s,
+                                   "ESINO GITRI BALEN MUREN TOSNU"s);
+    auto leg = fp1->legAtIndex(2);
+    leg->setAltitude(RESTRICT_ABOVE, 120, ALTITUDE_FLIGHTLEVEL);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(leg->altitude(), 12000.0, 0.1);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(leg->altitude(ALTITUDE_METER), 12000.0 * SG_FEET_TO_METER, 0.1);
+
+    leg->setSpeed(RESTRICT_AT, 300);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(leg->speedKts(), 300, 0.1);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(leg->speed(), 300, 0.1);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(leg->speed(SPEED_KPH), 300 * SG_KT_TO_MPS * SG_MPS_TO_KMH, 0.1);
+
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(leg->speedMach(), 0.473, 0.1);
+
+    // setting a mach restriction assumes Mach units by default
+    leg->setSpeed(SPEED_RESTRICT_MACH, 0.7);
+    leg->setAltitude(RESTRICT_AT, 38000);
+    CPPUNIT_ASSERT_DOUBLES_EQUAL(leg->speedKts(), 401, 1);
+}
+
 void FlightplanTests::testBasicDiscontinuity()
 {
-    FlightPlanRef fp1 = makeTestFP("LIRF", "34L", "LEBL", "25R",
-                                   "ESINO GITRI BALEN MUREN TOSNU");
+    FlightPlanRef fp1 = makeTestFP("LIRF"s, "34L"s, "LEBL"s, "25R"s,
+                                   "ESINO GITRI BALEN MUREN TOSNU"s);
 
     const auto tdBefore = fp1->totalDistanceNm();
     
@@ -630,12 +827,12 @@ void FlightplanTests::testBasicDiscontinuity()
 
 void FlightplanTests::testOnlyDiscontinuityRoute()
 {
-    FlightPlanRef f = new FlightPlan;
-    FGAirportRef depApt = FGAirport::getByIdent("LFBD");
+    FlightPlanRef f = FlightPlan::create();
+    FGAirportRef depApt = FGAirport::getByIdent("LFBD"s);
     f->setDeparture(depApt);
 
 
-    FGAirportRef destApt = FGAirport::getByIdent("EHAM");
+    FGAirportRef destApt = FGAirport::getByIdent("EHAM"s);
     f->setDestination(destApt);
 
     f->insertWayptAtIndex(new Discontinuity(f), 0);
@@ -647,23 +844,23 @@ void FlightplanTests::testOnlyDiscontinuityRoute()
    // CPPUNIT_ASSERT_DOUBLES_EQUAL(rp1.distanceForIndex(0), d, 0.5);
     
     // start inserting waypoints ahead of the DISCON, Boeing FPL style
-    WayptRef wpt = f->waypointFromString("LMG");
+    WayptRef wpt = f->waypointFromString("LMG"s);
     f->insertWayptAtIndex(wpt, 0);
     
-    wpt = f->waypointFromString("KUKOR");
+    wpt = f->waypointFromString("KUKOR"s);
     f->insertWayptAtIndex(wpt, 1);
     
-    wpt = f->waypointFromString("EPL");
+    wpt = f->waypointFromString("EPL"s);
     f->insertWayptAtIndex(wpt, 2);
 }
 
 void FlightplanTests::testLeadingWPDynamic()
 {
-    FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = FlightPlan::create();
     // plan has no departure, so this discon is floating
     f->insertWayptAtIndex(new Discontinuity(f), 0);
     
-    auto ha = new HeadingToAltitude(f, "TO_3000", 90);
+    auto ha = new HeadingToAltitude(f, "TO_3000"s, 90);
     ha->setAltitude(3000, RESTRICT_AT);
     f->insertWayptAtIndex(ha, 1);
     
@@ -681,14 +878,14 @@ static double magVarFor(const SGGeod& geod)
 void FlightplanTests::testRadialIntercept()
 {
     // replicate AJO1R departure
-    FlightPlanRef f = makeTestFP("LFKC", "36", "LIRF", "25", "BUNAX BEBEV AJO");
+    FlightPlanRef f = makeTestFP("LFKC"s, "36"s, "LIRF"s, "25"s, "BUNAX BEBEV AJO"s);
     
     // set BUNAX as overflight
     f->legAtIndex(1)->waypoint()->setFlag(WPT_OVERFLIGHT);
-    f->insertWayptAtIndex(new BasicWaypt(SGGeod::fromDeg(8.78333, 42.566), "KC502", f), 1);
+    f->insertWayptAtIndex(new BasicWaypt(SGGeod::fromDeg(8.78333, 42.566), "KC502"s, f), 1);
 
     SGGeod pos = SGGeod::fromDeg(8.445556,42.216944);
-    auto intc = new RadialIntercept(f, "INTC", pos, 230, 5);
+    auto intc = new RadialIntercept(f, "INTC"s, pos, 230, 5);
     f->insertWayptAtIndex(intc, 3); // between BUNAX and BEBEV
     
     RoutePath rtepath(f);
@@ -700,9 +897,9 @@ void FlightplanTests::loadFGFPWithoutDepartureArrival()
     static_factory = std::make_shared<TestFPDelegateFactory>();
     FlightPlan::registerDelegateFactory(static_factory);
 
-    FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = FlightPlan::create();
     
-    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_without_dep_arr.fgfp";
+    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_without_dep_arr.fgfp"s;
     {
         sg_ofstream s(fgfpPath);
         s << R"(<?xml version="1.0" encoding="UTF-8"?>
@@ -766,9 +963,9 @@ void FlightplanTests::loadFGFPWithEmbeddedProcedures()
     static_factory = std::make_shared<TestFPDelegateFactory>();
     FlightPlan::registerDelegateFactory(static_factory);
 
-    FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = FlightPlan::create();
     
-    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_with_dep_arr.fgfp";
+    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_with_dep_arr.fgfp"s;
     {
         sg_ofstream s(fgfpPath);
         s << R"(<?xml version="1.0" encoding="UTF-8"?>
@@ -845,9 +1042,9 @@ void FlightplanTests::loadFGFPWithOldProcedures()
     if (!static_haveProcedures)
         return;
     
-    FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = FlightPlan::create();
     
-    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_old_procedure_idents.fgfp";
+    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_old_procedure_idents.fgfp"s;
     {
         sg_ofstream s(fgfpPath);
         s << R"(<?xml version="1.0" encoding="UTF-8"?>
@@ -874,8 +1071,8 @@ void FlightplanTests::loadFGFPWithOldProcedures()
         )";
     }
     
-    auto kjfk = FGAirport::findByIdent("KJFK");
-    auto eham = FGAirport::findByIdent("EHAM");
+    auto kjfk = FGAirport::findByIdent("KJFK"s);
+    auto eham = FGAirport::findByIdent("EHAM"s);
     CPPUNIT_ASSERT(f->load(fgfpPath));
     
     CPPUNIT_ASSERT(f->sid() == nullptr);
@@ -887,9 +1084,9 @@ void FlightplanTests::loadFGFPWithProcedureIdents()
     if (!static_haveProcedures)
         return;
     
-    FlightPlanRef f = new FlightPlan;
+    FlightPlanRef f = FlightPlan::create();
     
-    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_procedure_idents.fgfp";
+    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_procedure_idents.fgfp"s;
     {
         sg_ofstream s(fgfpPath);
         s << R"(<?xml version="1.0" encoding="UTF-8"?>
@@ -918,41 +1115,109 @@ void FlightplanTests::loadFGFPWithProcedureIdents()
         )";
     }
     
-    auto kjfk = FGAirport::findByIdent("KJFK");
-    auto eham = FGAirport::findByIdent("EHAM");
+    auto kjfk = FGAirport::findByIdent("KJFK"s);
+    auto eham = FGAirport::findByIdent("EHAM"s);
     CPPUNIT_ASSERT(f->load(fgfpPath));
     
-    CPPUNIT_ASSERT_EQUAL(f->sid()->ident(), string{"DEEZZ5.13L"});
-    CPPUNIT_ASSERT_EQUAL(f->sidTransition()->ident(), string{"CANDR"});
+    CPPUNIT_ASSERT_EQUAL(f->sid()->ident(), "DEEZZ5.13L"s);
+    CPPUNIT_ASSERT_EQUAL(f->sidTransition()->ident(), "CANDR"s);
     
-    CPPUNIT_ASSERT_EQUAL(f->star()->ident(), string{"EEL1A"});
-    CPPUNIT_ASSERT_EQUAL(f->starTransition()->ident(), string{"KUBAT"});
+    CPPUNIT_ASSERT_EQUAL(f->star()->ident(), "EEL1A"s);
+    CPPUNIT_ASSERT_EQUAL(f->starTransition()->ident(), "KUBAT"s);
 }
 
+void FlightplanTests::loadFGFPAsRoute()
+{
+    static_factory = std::make_shared<TestFPDelegateFactory>();
+    FlightPlan::registerDelegateFactory(static_factory);
+
+    FlightPlanRef f = FlightPlan::createRoute();
+    
+    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_as_route.fgfp"s;
+    {
+        sg_ofstream s(fgfpPath);
+        s << R"(<?xml version="1.0" encoding="UTF-8"?>
+            <PropertyList>
+              <version type="int">2</version>
+                <is-route>true</is-route>
+              <departure>
+                <airport type="string">EGLL</airport>
+                <runway type="string">27R</runway>
+              </departure>
+              <destination>
+                <airport type="string">EDDS</airport>
+              </destination>
+              <route>
+                <wp n="1">
+                  <type type="string">navaid</type>
+                  <ident type="string">BIG</ident>
+                        <lat>51.33087500</lat>
+                        <lon>000.03481100</lon>
+                </wp>
+                <wp n="2">
+                  <type type="string">navaid</type>
+                  <ident type="string">CLN</ident>
+                <lat> 51.84841700</lat>
+                <lon>001.14761100</lon>
+                </wp>
+                <wp n="3">
+                  <type type="string">via</type>
+                  <airway type="string">L620</airway>
+                  <level>1</level>
+                  <to type="string">BASNO</to>
+                    <lat>52.114586</lat>
+                    <lon>2.487947</lon>
+                </wp>
+              </route>
+            </PropertyList>
+        )";
+    }
+    
+    auto ourDelegate = TestFPDelegateFactory::delegateForPlan(f);
+    CPPUNIT_ASSERT(!ourDelegate->didLoad);
+    
+    CPPUNIT_ASSERT(f->load(fgfpPath));
+    CPPUNIT_ASSERT(f->isRoute());
+
+    CPPUNIT_ASSERT(ourDelegate->didLoad);
+    CPPUNIT_ASSERT(ourDelegate->sawArrivalChange);
+    CPPUNIT_ASSERT(ourDelegate->sawDepartureChange);
+    
+    CPPUNIT_ASSERT_EQUAL(5, f->numLegs());
+    CPPUNIT_ASSERT_EQUAL("via"s, f->legAtIndex(3)->waypoint()->type());
+    
+    auto actFP = f->clone("ACT", true);
+    CPPUNIT_ASSERT(!actFP->isRoute());
+    CPPUNIT_ASSERT(!actFP->isActive());
+    // check Via was expanded
+    CPPUNIT_ASSERT_EQUAL(8, actFP->numLegs());
+}
+
+
 void FlightplanTests::testCloningBasic()
 {
-    FlightPlanRef fp1 = makeTestFP("EGCC", "23L", "EHAM", "24",
-                                   "TNT CLN");
-    fp1->setIdent("testplan");
+    FlightPlanRef fp1 = makeTestFP("EGCC"s, "23L"s, "EHAM"s, "24"s,
+                                   "TNT CLN"s);
+    fp1->setIdent("testplan"s);
 
     fp1->setCruiseAltitudeFt(24000);
     fp1->setCruiseSpeedKnots(448);
 
-    auto fp2 = fp1->clone("testplan2");
+    auto fp2 = fp1->clone("testplan2"s);
 
-    CPPUNIT_ASSERT(fp2->ident() == "testplan2");
-    CPPUNIT_ASSERT(fp2->departureAirport()->ident() == "EGCC");
-    CPPUNIT_ASSERT(fp2->departureRunway()->ident() == "23L");
-    CPPUNIT_ASSERT(fp2->destinationAirport()->ident() == "EHAM");
-    CPPUNIT_ASSERT(fp2->destinationRunway()->ident() == "24");
+    CPPUNIT_ASSERT(fp2->ident() == "testplan2"s);
+    CPPUNIT_ASSERT(fp2->departureAirport()->ident() == "EGCC"s);
+    CPPUNIT_ASSERT(fp2->departureRunway()->ident() == "23L"s);
+    CPPUNIT_ASSERT(fp2->destinationAirport()->ident() == "EHAM"s);
+    CPPUNIT_ASSERT(fp2->destinationRunway()->ident() == "24"s);
     CPPUNIT_ASSERT_EQUAL(24000, fp2->cruiseAltitudeFt());
     CPPUNIT_ASSERT_EQUAL(448, fp2->cruiseSpeedKnots());
 
     CPPUNIT_ASSERT_EQUAL(5, fp2->numLegs());
 
-    CPPUNIT_ASSERT(fp2->legAtIndex(0)->waypoint()->source()->ident() == "23L");
-    CPPUNIT_ASSERT(fp2->legAtIndex(1)->waypoint()->source()->ident() == "TNT");
-    CPPUNIT_ASSERT(fp2->legAtIndex(2)->waypoint()->source()->ident() == "CLN");
+    CPPUNIT_ASSERT(fp2->legAtIndex(0)->waypoint()->source()->ident() == "23L"s);
+    CPPUNIT_ASSERT(fp2->legAtIndex(1)->waypoint()->source()->ident() == "TNT"s);
+    CPPUNIT_ASSERT(fp2->legAtIndex(2)->waypoint()->source()->ident() == "CLN"s);
 
 }
 
@@ -961,9 +1226,9 @@ void FlightplanTests::testCloningFGFP()
     static_factory = std::make_shared<TestFPDelegateFactory>();
     FlightPlan::registerDelegateFactory(static_factory);
 
-    FlightPlanRef fp1 = new FlightPlan;
+    FlightPlanRef fp1 = FlightPlan::create();
     
-    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_cloning.fgfp";
+    SGPath fgfpPath = simgear::Dir::current().path() / "test_fgfp_cloning.fgfp"s;
     {
         sg_ofstream s(fgfpPath);
         s << R"(<?xml version="1.0" encoding="UTF-8"?>
@@ -1029,14 +1294,14 @@ void FlightplanTests::testCloningFGFP()
     CPPUNIT_ASSERT(!secondDelegate->sawArrivalChange);
     CPPUNIT_ASSERT(!secondDelegate->sawDepartureChange);
     
-    CPPUNIT_ASSERT(fp2->departureAirport()->ident() == "EDDM");
-    CPPUNIT_ASSERT(fp2->departureRunway()->ident() == "08R");
-    CPPUNIT_ASSERT(fp2->destinationAirport()->ident() == "EDDF");
+    CPPUNIT_ASSERT(fp2->departureAirport()->ident() == "EDDM"s);
+    CPPUNIT_ASSERT(fp2->departureRunway()->ident() == "08R"s);
+    CPPUNIT_ASSERT(fp2->destinationAirport()->ident() == "EDDF"s);
 
-    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(0)->waypoint()->source()->ident(), string{"08R"});
-    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(1)->waypoint()->source()->ident(), string{"GIVMI"});
-    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(5)->waypoint()->source()->ident(), string{"PSA"});
-    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(6)->waypoint()->source()->ident(), string{"EDDF"});
+    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(0)->waypoint()->source()->ident(), "08R"s);
+    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(1)->waypoint()->source()->ident(), "GIVMI"s);
+    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(5)->waypoint()->source()->ident(), "PSA"s);
+    CPPUNIT_ASSERT_EQUAL(fp2->legAtIndex(6)->waypoint()->source()->ident(), "EDDF"s);
     CPPUNIT_ASSERT_EQUAL(7, fp2->numLegs());
     
 }
@@ -1049,29 +1314,69 @@ void FlightplanTests::testCloningProcedures() {
     static_factory = std::make_shared<TestFPDelegateFactory>();
     FlightPlan::registerDelegateFactory(static_factory);
     
-    auto egkk = FGAirport::findByIdent("EGKK");
-    auto sid = egkk->findSIDWithIdent("SAM3P");
+    auto egkk = FGAirport::findByIdent("EGKK"s);
+    auto sid = egkk->findSIDWithIdent("SAM3P"s);
     
-    FlightPlanRef fp1 = makeTestFP("EGKK", "08R", "EHAM", "18R",
-                                   "");
-    auto ourDelegate = TestFPDelegateFactory::delegateForPlan(fp1);
+    FlightPlanRef fp1 = makeTestFP("EGKK"s, "08R"s, "EHAM"s, "18R"s,
+                                   ""s);
+    //auto ourDelegate = TestFPDelegateFactory::delegateForPlan(fp1);
     
     fp1->setSID(sid);
-    auto eham = FGAirport::findByIdent("EHAM");
-    auto eel1A = eham->findSTARWithIdent("EEL1A");
-    fp1->setSTAR(eel1A, "BEDUM");
+    auto eham = FGAirport::findByIdent("EHAM"s);
+    auto eel1A = eham->findSTARWithIdent("EEL1A"s);
+    fp1->setSTAR(eel1A, "BEDUM"s);
     
     auto fp2 = fp1->clone();
-    CPPUNIT_ASSERT(fp2->departureAirport()->ident() == "EGKK");
-    CPPUNIT_ASSERT(fp2->departureRunway()->ident() == "08R");
-    CPPUNIT_ASSERT(fp2->destinationAirport()->ident() == "EHAM");
-    CPPUNIT_ASSERT(fp2->destinationRunway()->ident() == "18R");
+    CPPUNIT_ASSERT(fp2->departureAirport()->ident() == "EGKK"s);
+    CPPUNIT_ASSERT(fp2->departureRunway()->ident() == "08R"s);
+    CPPUNIT_ASSERT(fp2->destinationAirport()->ident() == "EHAM"s);
+    CPPUNIT_ASSERT(fp2->destinationRunway()->ident() == "18R"s);
 
-    CPPUNIT_ASSERT(fp2->legAtIndex(0)->waypoint()->source()->ident() == "08R");
-    CPPUNIT_ASSERT_EQUAL(fp2->sid()->ident(), string{"SAM3P"});
+    CPPUNIT_ASSERT(fp2->legAtIndex(0)->waypoint()->source()->ident() == "08R"s);
+    CPPUNIT_ASSERT_EQUAL(fp2->sid()->ident(), "SAM3P"s);
     
-    CPPUNIT_ASSERT_EQUAL(fp2->star()->ident(), string{"EEL1A"});
-    CPPUNIT_ASSERT_EQUAL(fp2->starTransition()->ident(), string{"BEDUM"});
+    CPPUNIT_ASSERT_EQUAL(fp2->star()->ident(), "EEL1A"s);
+    CPPUNIT_ASSERT_EQUAL(fp2->starTransition()->ident(), "BEDUM"s);
+}
 
 
+void FlightplanTests::testBug2616()
+{
+    auto edty = FGAirport::findByIdent("EDTY"s);
+    edty->testSuiteInjectProceduresXML(SGPath::fromUtf8(FG_TEST_SUITE_DATA) / "EDTY.procedures.xml");
+
+    auto ils28Approach = edty->findApproachWithIdent("ILS28"s);
+    CPPUNIT_ASSERT(ils28Approach);
+    
+        FlightPlanRef fp1 = makeTestFP("EDDS"s, "25"s, "EDTY"s, "28"s,
+                                   ""s);
+
+
+
+    fp1->setApproach(ils28Approach);
+
+      CPPUNIT_ASSERT(fp1->destinationRunway()->ident() == "28"s);
+      CPPUNIT_ASSERT(fp1->approach()->ident() == "ILS28"s);
+
+    fp1->activate();
+    CPPUNIT_ASSERT(fp1->isActive());
+}
+
+void FlightplanTests::testRoute()
+{
+  FlightPlanRef fp1 = makeTestFP("EGKK"s, "08R"s, "EHAM"s, "18R"s,
+                                   "CLN VIA-L620/REDFA SPY"s, true);
+
+  CPPUNIT_ASSERT(fp1->isRoute());
+  CPPUNIT_ASSERT_EQUAL("via"s, fp1->legAtIndex(2)->waypoint()->type());
+
+    // activating a route should fail
+    fp1->activate();
+    CPPUNIT_ASSERT(!fp1->isActive());
+
+    // setting index should still work
+    fp1->setCurrentIndex(2);
+    CPPUNIT_ASSERT_EQUAL(2, fp1->currentIndex());
+    CPPUNIT_ASSERT(!fp1->isActive());
+
 }
